@@ -149,6 +149,13 @@ typedef int jmp_buf[_JBLEN];
 #define _JBLEN 16
 #endif
 
+#if defined(_C4x)
+#define _JBLEN 10
+#endif
+#if defined(_C3x)
+#define _JBLEN 9
+#endif
+
 #ifdef __TIC80__
 #define _JBLEN 13
 #endif
@@ -196,6 +203,7 @@ typedef	_JBTYPE jmp_buf[_JBLEN];
 #else
 typedef	int jmp_buf[_JBLEN];
 #endif
+#endif
 
 _END_STD_C
 
@@ -228,4 +236,3 @@ typedef int sigjmp_buf[_JBLEN+2];
 }
 #endif
 #endif /* __CYGWIN__ or __rtems__ */
-#endif
