@@ -16,9 +16,9 @@
  */
 
 /*
-
 FUNCTION
         <<printf>>, <<fprintf>>, <<asprintf>>, <<sprintf>>, <<snprintf>>---format output
+
 INDEX
 	fprintf
 INDEX
@@ -205,7 +205,8 @@ DESCRIPTION
 		<<l>> forces the following <<d>>, <<i>>, <<o>>, <<u>>,
 		<<x>> or <<X>> conversion <[type]> to apply to a <<long>> or
 		<<unsigned long>>.  <<l>> also forces a following <<n>> <[type]> to
-		apply to a pointer to a <<long>>. If an <<h>>
+		apply to a pointer to a <<long>>.  <<l>> with <<c>>, <<s>> is
+		equivalent to <<C>>, <<S>> respectively.  If an <<h>>
 		or an <<l>> appears with another conversion
 		specifier, the behavior is undefined.  <<L>> forces a
 		following <<e>>, <<E>>, <<f>>, <<g>> or <<G>> conversion <[type]> to
@@ -224,10 +225,18 @@ DESCRIPTION
 		o c
 		prints <[arg]> as single character
 		
+		o C
+		prints wchar_t <[arg]> as single multibyte character
+		
 		o s
 		prints characters until precision is reached or a null terminator
 		is encountered; takes a string pointer
 
+		o S
+		converts wchar_t characters to multibyte output characters until
+		precision is reached or a null wchar_t terminator
+		is encountered; takes a wchar_t pointer
+
 		o d
 		prints a signed decimal integer; takes an <<int>> (same as <<i>>)
 
@@ -250,24 +259,24 @@ DESCRIPTION
 
 		o f
 		prints a signed value of the form <<[-]9999.9999>>; takes
-		a floating point number
+		a floating-point number
 	
 		o e
 		prints a signed	value of the form <<[-]9.9999e[+|-]999>>; takes a
-		floating point number
+		floating-point number
 
 		o E
 		prints the same way as <<e>>, but using <<E>> to introduce the
-		exponent; takes a floating point number
+		exponent; takes a floating-point number
 
 		o g
 		prints a signed value in either <<f>> or <<e>> form, based on given
 		value and precision---trailing zeros and the decimal point are
-		printed only if necessary; takes a floating point number
+		printed only if necessary; takes a floating-point number
 	
 		o G
 		prints the same way as <<g>>, but using <<E>> for the exponent if an
-		exponent is needed; takes a floating point number
+		exponent is needed; takes a floating-point number
 
 		o n
 		stores (in the same object) a count of the characters written;
@@ -296,6 +305,8 @@ Supporting OS subroutines required: <<close>>, <<fstat>>, <<isatty>>,
 <<lseek>>, <<read>>, <<sbrk>>, <<write>>.
 */
 
+#include <_ansi.h>
+#include <reent.h>
 #include <stdio.h>
 #ifdef _HAVE_STDC
 #include <stdarg.h>
@@ -303,18 +314,20 @@ Supporting OS subroutines required: <<close>>, <<fstat>>, <<isatty>>,
 #include <varargs.h>
 #endif
 #include <limits.h>
-#include <_ansi.h>
 #include "local.h"
 
 int
 #ifdef _HAVE_STDC
-_DEFUN (_sprintf_r, (ptr, str, fmt), struct _reent *ptr _AND char *str _AND _CONST char *fmt _DOTS)
+_DEFUN(_sprintf_r, (ptr, str, fmt),
+       struct _reent *ptr _AND
+       char *str          _AND
+       _CONST char *fmt _DOTS)
 #else
-_sprintf_r (ptr, str, fmt, va_alist)
-     struct _reent *ptr;
-     char *str;
-     _CONST char *fmt;
-     va_dcl
+_sprintf_r(ptr, str, fmt, va_alist)
+           struct _reent *ptr;
+           char *str;
+           _CONST char *fmt;
+           va_dcl
 #endif
 {
   int ret;
@@ -340,12 +353,14 @@ _sprintf_r (ptr, str, fmt, va_alist)
 
 int
 #ifdef _HAVE_STDC
-_DEFUN (sprintf, (str, fmt), char *str _AND _CONST char *fmt _DOTS)
+_DEFUN(sprintf, (str, fmt),
+       char *str _AND
+       _CONST char *fmt _DOTS)
 #else
-sprintf (str, fmt, va_alist)
-     char *str;
-     _CONST char *fmt;
-     va_dcl
+sprintf(str, fmt, va_alist)
+        char *str;
+        _CONST char *fmt;
+        va_dcl
 #endif
 {
   int ret;
