@@ -1,3 +1,20 @@
+/*-
+ * Copyright (c) 1990 The Regents of the University of California.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that the above copyright notice and this paragraph are
+ * duplicated in all such forms and that any documentation,
+ * advertising materials, and other materials related to such
+ * distribution and use acknowledge that the software was developed
+ * by the University of California, Berkeley.  The name of the
+ * University may not be used to endorse or promote products derived
+ * from this software without specific prior written permission.
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ */
+
 /*
 FUNCTION
 <<vscanf>>, <<vfscanf>>, <<vsscanf>>---format argument list
@@ -16,11 +33,11 @@ ANSI_SYNOPSIS
 	int vfscanf(FILE *<[fp]>, const char *<[fmt]>, va_list <[list]>);
 	int vsscanf(const char *<[str]>, const char *<[fmt]>, va_list <[list]>);
 
-	int _vscanf_r(void *<[reent]>, const char *<[fmt]>, 
+	int _vscanf_r(struct _reent *<[reent]>, const char *<[fmt]>, 
                        va_list <[list]>);
-	int _vfscanf_r(void *<[reent]>, FILE *<[fp]>, const char *<[fmt]>, 
+	int _vfscanf_r(struct _reent *<[reent]>, FILE *<[fp]>, const char *<[fmt]>, 
                        va_list <[list]>);
-	int _vsscanf_r(void *<[reent]>, const char *<[str]>, const char *<[fmt]>, 
+	int _vsscanf_r(struct _reent *<[reent]>, const char *<[str]>, const char *<[fmt]>, 
                        va_list <[list]>);
 
 TRAD_SYNOPSIS
@@ -41,18 +58,18 @@ TRAD_SYNOPSIS
 	va_list <[list]>;
 
 	int _vscanf_r( <[reent]>, <[fmt]>, <[ist]>)
-	char *<[reent]>;
+	struct _reent *<[reent]>;
 	char *<[fmt]>;
 	va_list <[list]>;
 
 	int _vfscanf_r( <[reent]>, <[fp]>, <[fmt]>, <[list]>)
-	char *<[reent]>;
+	struct _reent *<[reent]>;
 	FILE *<[fp]>;
 	char *<[fmt]>;
 	va_list <[list]>;
 	
 	int _vsscanf_r( <[reent]>, <[str]>, <[fmt]>, <[list]>)
-	char *<[reent]>;
+	struct _reent *<[reent]>;
 	char *<[str]>;
 	char *<[fmt]>;
 	va_list <[list]>;
@@ -85,25 +102,11 @@ These are GNU extensions.
 Supporting OS subroutines required:
 */
 
-/*-
- * Copyright (c) 1990 The Regents of the University of California.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation,
- * advertising materials, and other materials related to such
- * distribution and use acknowledge that the software was developed
- * by the University of California, Berkeley.  The name of the
- * University may not be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
- */
-
 #include <_ansi.h>
+#include <newlib.h>
+#include <reent.h>
 #include <ctype.h>
+#include <wctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <limits.h>
@@ -138,7 +141,13 @@ extern _LONG_DOUBLE _strtold _PARAMS((char *s, char **sptr));
 #endif
 
 #include "floatio.h"
-#define	BUF	(MAXEXP+MAXFRACT+3)	/* 3 = sign + decimal point + NUL */
+
+#if ((MAXEXP+MAXFRACT+3) > MB_LEN_MAX)
+#  define BUF (MAXEXP+MAXFRACT+3)        /* 3 = sign + decimal point + NUL */
+#else
+#  define BUF MB_LEN_MAX
+#endif
+
 /* An upper bound for how long a long prints in decimal.  4 / 13 approximates
    log (2).  Add one char for roundoff compensation and one for the sign.  */
 #define MAX_LONG_LEN ((CHAR_BIT * sizeof (long)  - 1) * 4 / 13 + 2)
@@ -172,6 +181,7 @@ extern _LONG_DOUBLE _strtold _PARAMS((char *s, char **sptr));
 
 #define	PFXOK		0x200	/* 0x prefix is (still) legal */
 #define	NZDIGITS	0x400	/* no zero digits detected */
+#define	NNZDIGITS	0x800	/* no non-zero digits detected */
 
 /*
  * Conversion types.
@@ -204,20 +214,20 @@ typedef unsigned long long u_long_long;
 #ifndef _REENT_ONLY
 
 int
-_DEFUN (vfscanf, (fp, fmt, ap), 
-    register FILE *fp _AND 
-    _CONST char *fmt _AND 
-    va_list ap)
+_DEFUN(vfscanf, (fp, fmt, ap), 
+       register FILE *fp _AND 
+       _CONST char *fmt _AND 
+       va_list ap)
 {
   CHECK_INIT(fp);
   return __svfscanf_r (_REENT, fp, fmt, ap);
 }
 
 int
-__svfscanf (fp, fmt0, ap)
-     register FILE *fp;
-     char _CONST *fmt0;
-     va_list ap;
+_DEFUN(__svfscanf, (fp, fmt0, ap),
+       register FILE *fp _AND
+       char _CONST *fmt0 _AND
+       va_list ap)
 {
   return __svfscanf_r (_REENT, fp, fmt0, ap);
 }
@@ -225,22 +235,22 @@ __svfscanf (fp, fmt0, ap)
 #endif /* !_REENT_ONLY */
 
 int
-_DEFUN (_vfscanf_r, (data, fp, fmt, ap),
-    struct _reent *data _AND 
-    register FILE *fp _AND 
-    _CONST char *fmt _AND 
-    va_list ap)
+_DEFUN(_vfscanf_r, (data, fp, fmt, ap),
+       struct _reent *data _AND 
+       register FILE *fp   _AND 
+       _CONST char *fmt    _AND 
+       va_list ap)
 {
   return __svfscanf_r (data, fp, fmt, ap);
 }
 
 
 int
-__svfscanf_r (rptr, fp, fmt0, ap)
-     struct _reent *rptr;
-     register FILE *fp;
-     char _CONST *fmt0;
-     va_list ap;
+_DEFUN(__svfscanf_r, (rptr, fp, fmt0, ap),
+       struct _reent *rptr _AND
+       register FILE *fp   _AND
+       char _CONST *fmt0   _AND
+       va_list ap)
 {
   register u_char *fmt = (u_char *) fmt0;
   register int c;		/* character from format, or conversion */
@@ -254,14 +264,14 @@ __svfscanf_r (rptr, fp, fmt0, ap)
   int base = 0;			/* base argument to strtol/strtoul */
   int nbytes = 1;               /* number of bytes read from fmt string */
   wchar_t wc;                   /* wchar to use to read format string */
+  wchar_t *wcp;                 /* handy wide character pointer */
+  size_t mbslen;                /* length of converted multibyte sequence */
+  mbstate_t state;              /* value to keep track of multibyte state */
 
   u_long (*ccfn) () = 0;	/* conversion function (strtol/strtoul) */
   char ccltab[256];		/* character class table for %[...] */
   char buf[BUF];		/* buffer for numeric conversions */
   char *lptr;                   /* literal pointer */
-#ifdef MB_CAPABLE
-  mbstate_t state;                /* value to keep track of multibyte state */
-#endif
 
   char *cp;
   short *sp;
@@ -272,19 +282,19 @@ __svfscanf_r (rptr, fp, fmt0, ap)
   long *lp;
 #ifndef _NO_LONGLONG
   long long *llp;
-#else
-	u_long _uquad;
 #endif
 
   /* `basefix' is used to avoid `if' tests in the integer scanner */
   static _CONST short basefix[17] =
     {10, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
 
+  _flockfile (fp);
+ 
   nassigned = 0;
   nread = 0;
   for (;;)
     {
-#ifndef MB_CAPABLE
+#ifndef _MB_CAPABLE
       wc = *fmt;
 #else
       memset (&state, '\0', sizeof (state));
@@ -292,7 +302,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 #endif
       fmt += nbytes;
       if (wc == 0)
-	return nassigned;
+	goto all_done;
       if (nbytes == 1 && isspace (wc))
 	{
 	  for (;;)
@@ -428,6 +438,9 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	  c = CT_FLOAT;
 	  break;
 #endif
+        case 'S':
+          flags |= LONG;
+          /* FALLTHROUGH */
 
 	case 's':
 	  c = CT_STRING;
@@ -439,6 +452,10 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	  c = CT_CCL;
 	  break;
 
+        case 'C':
+          flags |= LONG;
+          /* FALLTHROUGH */
+
 	case 'c':
 	  flags |= NOSKIP;
 	  c = CT_CHAR;
@@ -487,6 +504,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	   * Disgusting backwards compatibility hacks.	XXX
 	   */
 	case '\0':		/* compat */
+	  _funlockfile (fp);
 	  return EOF;
 
 	default:		/* compat */
@@ -516,9 +534,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	      if (--fp->_r > 0)
 		fp->_p++;
 	      else
-#ifndef CYGNUS_NEC
 	      if (__srefill (fp))
-#endif
 		goto input_failure;
 	    }
 	  /*
@@ -538,10 +554,47 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	  /* scan arbitrary characters (sets NOSKIP) */
 	  if (width == 0)
 	    width = 1;
-	  if (flags & SUPPRESS)
+          if (flags & LONG) 
+            {
+              if ((flags & SUPPRESS) == 0)
+                wcp = va_arg (ap, wchar_t *);
+              else
+                wcp = NULL;
+              n = 0;
+              while (width != 0) 
+                {
+                  if (n == MB_CUR_MAX)
+                    goto input_failure;
+                  buf[n++] = *fp->_p;
+                  fp->_r -= 1;
+                  fp->_p += 1;
+                  memset ((_PTR)&state, '\0', sizeof (mbstate_t));
+                  if ((mbslen = _mbrtowc_r (rptr, wcp, buf, n, &state)) 
+                                                         == (size_t)-1)
+                    goto input_failure; /* Invalid sequence */
+                  if (mbslen == 0 && !(flags & SUPPRESS))
+                    *wcp = L'\0';
+                  if (mbslen != (size_t)-2) /* Incomplete sequence */
+                    {
+                      nread += n;
+                      width -= 1;
+                      if (!(flags & SUPPRESS))
+                        wcp += 1;
+                      n = 0;
+                    }
+                  if (BufferEmpty) 
+	            {
+                      if (n != 0) 
+                        goto input_failure;
+                      break;
+                    }
+                }
+              if (!(flags & SUPPRESS))
+                nassigned++;
+            } 
+          else if (flags & SUPPRESS) 
 	    {
 	      size_t sum = 0;
-
 	      for (;;)
 		{
 		  if ((n = fp->_r) < (int)width)
@@ -549,16 +602,12 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		      sum += n;
 		      width -= n;
 		      fp->_p += n;
-#ifndef CYGNUS_NEC
 		      if (__srefill (fp))
 			{
-#endif
 			  if (sum == 0)
 			    goto input_failure;
 			  break;
-#ifndef CYGNUS_NEC
 			}
-#endif
 		    }
 		  else
 		    {
@@ -572,27 +621,11 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	    }
 	  else
 	    {
-#ifdef CYGNUS_NEC
-	      /* Kludge city for the moment */
-	      char *dest = va_arg (ap, char *);
-	      int n = width;
-	      if (fp->_r == 0)
-		goto input_failure;
-
-	      while (n && fp->_r)
-		{
-		  *dest++ = *(fp->_p++);
-		  n--;
-		  fp->_r--;
-		  nread++;
-		}
-#else
 	      size_t r = fread ((_PTR) va_arg (ap, char *), 1, width, fp);
 
 	      if (r == 0)
 		goto input_failure;
 	      nread += r;
-#endif
 	      nassigned++;
 	    }
 	  break;
@@ -648,8 +681,56 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	case CT_STRING:
 	  /* like CCL, but zero-length string OK, & no NOSKIP */
 	  if (width == 0)
-	    width = ~0;
-	  if (flags & SUPPRESS)
+            width = (size_t)~0;
+          if (flags & LONG) 
+            {
+              /* Process %S and %ls placeholders */
+              if ((flags & SUPPRESS) == 0)
+                wcp = va_arg (ap, wchar_t *);
+              else
+                wcp = &wc;
+              n = 0;
+              while (!isspace (*fp->_p) && width != 0) 
+                {
+                  if (n == MB_CUR_MAX)
+                    goto input_failure;
+                  buf[n++] = *fp->_p;
+                  fp->_r -= 1;
+                  fp->_p += 1;
+                  memset ((_PTR)&state, '\0', sizeof (mbstate_t));
+                  if ((mbslen = _mbrtowc_r (rptr, wcp, buf, n, &state)) 
+                                                        == (size_t)-1)
+                    goto input_failure;
+                  if (mbslen == 0)
+                    *wcp = L'\0';
+                  if (mbslen != (size_t)-2) /* Incomplete sequence */
+                    {
+                      if (iswspace(*wcp)) 
+                        {
+                          while (n != 0)
+                            ungetc (buf[--n], fp);
+                          break;
+                        }
+                      nread += n;
+                      width -= 1;
+                      if ((flags & SUPPRESS) == 0)
+                        wcp += 1;
+                      n = 0;
+                    }
+                  if (BufferEmpty) 
+                    {
+                      if (n != 0)
+                        goto input_failure;
+                      break;
+                    }
+                }
+              if (!(flags & SUPPRESS)) 
+                {
+                  *wcp = L'\0';
+                  nassigned++;
+                }
+            }
+          else if (flags & SUPPRESS) 
 	    {
 	      n = 0;
 	      while (!isspace (*fp->_p))
@@ -681,17 +762,20 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	  continue;
 
 	case CT_INT:
+	{
 	  /* scan an integer as if by strtol/strtoul */
+	  unsigned width_left = 0;
 #ifdef hardway
 	  if (width == 0 || width > sizeof (buf) - 1)
-	    width = sizeof (buf) - 1;
 #else
 	  /* size_t is unsigned, hence this optimisation */
-	  if (--width > sizeof (buf) - 2)
-	    width = sizeof (buf) - 2;
-	  width++;
+	  if (width - 1 > sizeof (buf) - 2)
 #endif
-	  flags |= SIGNOK | NDIGITS | NZDIGITS;
+	    {
+	      width_left = width - (sizeof (buf) - 1);
+	      width = sizeof (buf) - 1;
+	    }
+	  flags |= SIGNOK | NDIGITS | NZDIGITS | NNZDIGITS;
 	  for (p = buf; width; width--)
 	    {
 	      c = *fp->_p;
@@ -711,16 +795,25 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		   * will turn it off if we have scanned any nonzero digits).
 		   */
 		case '0':
+		  if (! (flags & NNZDIGITS))
+		    goto ok;
 		  if (base == 0)
 		    {
 		      base = 8;
 		      flags |= PFXOK;
 		    }
 		  if (flags & NZDIGITS)
-		    flags &= ~(SIGNOK | NZDIGITS | NDIGITS);
-		  else
-		    flags &= ~(SIGNOK | PFXOK | NDIGITS);
-		  goto ok;
+		    {
+		      flags &= ~(SIGNOK | NZDIGITS | NDIGITS);
+		      goto ok;
+		    }
+		  flags &= ~(SIGNOK | PFXOK | NDIGITS);
+		  if (width_left)
+		    {
+		      width_left--;
+		      width++;
+		    }
+		  goto skip;
 
 		  /* 1 through 7 always legal */
 		case '1':
@@ -731,7 +824,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		case '6':
 		case '7':
 		  base = basefix[base];
-		  flags &= ~(SIGNOK | PFXOK | NDIGITS);
+		  flags &= ~(SIGNOK | PFXOK | NDIGITS | NNZDIGITS);
 		  goto ok;
 
 		  /* digits 8 and 9 ok iff decimal or hex */
@@ -740,7 +833,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		  base = basefix[base];
 		  if (base <= 8)
 		    break;	/* not legal here */
-		  flags &= ~(SIGNOK | PFXOK | NDIGITS);
+		  flags &= ~(SIGNOK | PFXOK | NDIGITS | NNZDIGITS);
 		  goto ok;
 
 		  /* letters ok iff hex */
@@ -759,7 +852,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		  /* no need to fix base here */
 		  if (base <= 10)
 		    break;	/* not legal here */
-		  flags &= ~(SIGNOK | PFXOK | NDIGITS);
+		  flags &= ~(SIGNOK | PFXOK | NDIGITS | NNZDIGITS);
 		  goto ok;
 
 		  /* sign ok only as first character */
@@ -794,12 +887,11 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	       * c is legal: store it and look at the next.
 	       */
 	      *p++ = c;
+	    skip:
 	      if (--fp->_r > 0)
 		fp->_p++;
 	      else
-#ifndef CYGNUS_NEC
 	      if (__srefill (fp))
-#endif
 		break;		/* EOF */
 	    }
 	  /*
@@ -863,7 +955,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	    }
 	  nread += p - buf;
 	  break;
-
+	}
 #ifdef FLOATING_POINT
 	case CT_FLOAT:
 	{
@@ -875,15 +967,17 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	  long leading_zeroes = 0;
 	  long zeroes, exp_adjust;
 	  char *exp_start = NULL;
+	  unsigned width_left = 0;
 #ifdef hardway
 	  if (width == 0 || width > sizeof (buf) - 1)
-	    width = sizeof (buf) - 1;
 #else
 	  /* size_t is unsigned, hence this optimisation */
-	  if (--width > sizeof (buf) - 2)
-	    width = sizeof (buf) - 2;
-	  width++;
+	  if (width - 1 > sizeof (buf) - 2)
 #endif
+	    {
+	      width_left = width - (sizeof (buf) - 1);
+	      width = sizeof (buf) - 1;
+	    }
 	  flags |= SIGNOK | NDIGITS | DPTOK | EXPOK;
 	  zeroes = 0;
 	  exp_adjust = 0;
@@ -902,6 +996,11 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		    {
 		      flags &= ~SIGNOK;
 		      zeroes++;
+		      if (width_left)
+			{
+			  width_left--;
+			  width++;
+			}
 		      goto fskip;
 		    }
 		  /* Fall through.  */
@@ -961,9 +1060,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	      if (--fp->_r > 0)
 		fp->_p++;
 	      else
-#ifndef CYGNUS_NEC
 	      if (__srefill (fp))
-#endif
 		break;		/* EOF */
 	    }
 	  if (zeroes)
@@ -998,10 +1095,12 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	    }
 	  if ((flags & SUPPRESS) == 0)
 	    {
+	      double res = 0;
 #ifdef _NO_LONGDBL
-	      double res;
+#define QUAD_RES res;
 #else  /* !_NO_LONG_DBL */
-	      long double res;
+	      long double qres = 0;
+#define QUAD_RES qres;
 #endif /* !_NO_LONG_DBL */
 	      long new_exp = 0;
 
@@ -1023,11 +1122,17 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 		    exp_start = buf + sizeof (buf) - MAX_LONG_LEN - 1;
                  sprintf (exp_start, "e%ld", new_exp);
 		}
-#ifdef _NO_LONGDBL
-	      res = _strtod_r (rptr, buf, NULL);
-#else  /* !_NO_LONGDBL */
-	      res = _strtold (buf, NULL);
-#endif /* !_NO_LONGDBL */
+
+	      /* Current _strtold routine is markedly slower than 
+	         _strtod_r.  Only use it if we have a long double
+	         result.  */
+#ifndef _NO_LONGDBL /* !_NO_LONGDBL */
+	      if (flags & LONGDBL)
+	      	qres = _strtold (buf, NULL);
+	      else
+#endif
+	        res = _strtod_r (rptr, buf, NULL);
+
 	      if (flags & LONG)
 		{
 		  dp = va_arg (ap, double *);
@@ -1036,7 +1141,7 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	      else if (flags & LONGDBL)
 		{
 		  ldp = va_arg (ap, _LONG_DOUBLE *);
-		  *ldp = res;
+		  *ldp = QUAD_RES;
 		}
 	      else
 		{
@@ -1051,8 +1156,11 @@ __svfscanf_r (rptr, fp, fmt0, ap)
 	}
     }
 input_failure:
+  _funlockfile (fp);
   return nassigned ? nassigned : -1;
 match_failure:
+all_done:
+  _funlockfile (fp);
   return nassigned;
 }
 
@@ -1065,9 +1173,9 @@ match_failure:
 
 /*static*/
 u_char *
-__sccl (tab, fmt)
-     register char *tab;
-     register u_char *fmt;
+_DEFUN(__sccl, (tab, fmt),
+       register char *tab _AND
+       register u_char *fmt)
 {
   register int c, n, v;
 
