@@ -69,7 +69,7 @@ details. */
 /* Create a "decorated" name */
 #define mangle(name, n) #name "@" #n
 
-/* Standard DLL load macro.  Invokes a fatal warning if the function isn't
+/* Standard DLL load macro.  May invoke a fatal error if the function isn't
    found. */
 #define LoadDLLfunc(name, n, dllname) LoadDLLfuncEx (name, n, dllname, 0)
 #define LoadDLLfuncEx(name, n, dllname, notimp) LoadDLLfuncEx2(name, n, dllname, notimp, 0)
@@ -106,12 +106,8 @@ extern "C" void dll_func_load () __asm__ ("dll_func_load");
    functions from this DLL.  */
 extern "C" void dll_chain () __asm__ ("dll_chain");
 
-/* called by the secondary initialization function to call dll_func_load. */
-extern "C" void dll_chain1 () __asm__ ("dll_chain1");
-
 extern "C" {
 
-/* FIXME: This is not thread-safe? */
 __asm__ ("								\n\
 msg1:									\n\
 	.ascii	\"couldn't dynamically determine load address for '%s' (handle %p), %E\\0\"\n\
@@ -164,12 +160,6 @@ gotit:									\n\
 									\n\
 	.global	dll_chain						\n\
 dll_chain:								\n\
-	pushl	%eax		# Restore 'return address'		\n\
-	movl	(%eax),%eax	# Get address of DLL info block		\n\
-	movl	$dll_func_load,(%eax) # Just load func now		\n\
-	jmp	*%edx		# Jump to next init function		\n\
-									\n\
-dll_chain1:								\n\
 	pushl	%eax		# Restore 'return address'		\n\
 	jmp	*%edx		# Jump to next init function		\n\
 ");
@@ -264,7 +254,7 @@ wsock_init ()
       low_priority_sleep (0);
     }
 
-  if (!wsock_started && (winsock_active || winsock2_active))
+  if (!wsock_started)
     {
       /* Don't use autoload to load WSAStartup to eliminate recursion. */
       int (*wsastartup) (int, WSADATA *);
@@ -290,13 +280,13 @@ wsock_init ()
 
   InterlockedDecrement (&here);
 
-  /* Kludge alert.  Redirects the return address to dll_chain1. */
+  /* Kludge alert.  Redirects the return address to dll_chain. */
   __asm__ __volatile__ ("		\n\
-	movl	$dll_chain1,4(%ebp)	\n\
+	movl	$dll_chain,4(%ebp)	\n\
   ");
 
   volatile retchain ret;
-  /* Set "arguments for dll_chain1. */
+  /* Set "arguments for dll_chain. */
   ret.low = (long) dll_func_load;
   ret.high = (long) func;
   return ret.ll;
@@ -384,6 +374,7 @@ LoadDLLfunc (NetUserGetGroups, 28, netapi32)
 LoadDLLfunc (NetUserGetInfo, 16, netapi32)
 LoadDLLfunc (NetWkstaUserGetInfo, 12, netapi32)
 
+LoadDLLfuncEx (NtCreateFile, 44, ntdll, 1)
 LoadDLLfuncEx (NtCreateToken, 52, ntdll, 1)
 LoadDLLfuncEx (NtMapViewOfSection, 40, ntdll, 1)
 LoadDLLfuncEx (NtOpenFile, 24, ntdll, 1)
@@ -392,7 +383,9 @@ LoadDLLfuncEx (NtQueryInformationFile, 20, ntdll, 1)
 LoadDLLfuncEx (NtQueryInformationProcess, 20, ntdll, 1)
 LoadDLLfuncEx2 (NtQueryObject, 20, ntdll, 1, 1)
 LoadDLLfuncEx (NtQuerySystemInformation, 16, ntdll, 1)
+LoadDLLfuncEx (NtQuerySecurityObject, 20, ntdll, 1)
 LoadDLLfuncEx (NtQueryVirtualMemory, 24, ntdll, 1)
+LoadDLLfuncEx (NtSetSecurityObject, 12, ntdll, 1)
 LoadDLLfuncEx (NtUnmapViewOfSection, 8, ntdll, 1)
 LoadDLLfuncEx (RtlInitUnicodeString, 8, ntdll, 1)
 LoadDLLfuncEx (RtlNtStatusToDosError, 4, ntdll, 1)
@@ -530,6 +523,15 @@ LoadDLLfuncEx (timeGetTime, 0, winmm, 1)
 LoadDLLfuncEx (timeBeginPeriod, 4, winmm, 1)
 LoadDLLfuncEx (timeEndPeriod, 4, winmm, 1)
 
+LoadDLLfuncEx (waveInGetNumDevs, 0, winmm, 1)
+LoadDLLfuncEx (waveInOpen, 24, winmm, 1)
+LoadDLLfuncEx (waveInUnprepareHeader, 12, winmm, 1)
+LoadDLLfuncEx (waveInPrepareHeader, 12, winmm, 1)
+LoadDLLfuncEx (waveInAddBuffer, 12, winmm, 1)
+LoadDLLfuncEx (waveInStart, 4, winmm, 1)
+LoadDLLfuncEx (waveInReset, 4, winmm, 1)
+LoadDLLfuncEx (waveInClose, 4, winmm, 1)
+
 LoadDLLfuncEx (UuidCreate, 4, rpcrt4, 1)
 LoadDLLfuncEx (UuidCreateSequential, 4, rpcrt4, 1)
 }
