@@ -1,6 +1,6 @@
 /* child_info.h: shared child info for cygwin
 
-   Copyright 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -29,7 +29,7 @@ enum
 
 #define EXEC_MAGIC_SIZE sizeof(child_info)
 
-#define CURR_CHILD_INFO_MAGIC 0x8e0899faU
+#define CURR_CHILD_INFO_MAGIC 0xefaed2a6U
 
 /* NOTE: Do not make gratuitous changes to the names or organization of the
    below class.  The layout is checksummed to determine compatibility between
@@ -44,11 +44,12 @@ public:
   unsigned short type;	// type of record, exec, spawn, fork
   int cygpid;		// cygwin pid of child process
   HANDLE subproc_ready;	// used for synchronization with parent
-  HANDLE mount_h;
+  HANDLE user_h;
   HANDLE parent;
   HANDLE pppid_handle;
   init_cygheap *cygheap;
   void *cygheap_max;
+  DWORD cygheap_reserve_sz;
   HANDLE cygheap_h;
   unsigned fhandler_union_cb;
 };
@@ -88,8 +89,6 @@ public:
   child_info_spawn (): moreinfo (NULL) {}
   ~child_info_spawn ()
   {
-    if (parent)
-      CloseHandle (parent);
     if (moreinfo)
       {
 	if (moreinfo->old_title)
