@@ -1,12 +1,13 @@
 /* debug.cc
 
-   Copyright 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This software is a copyrighted work licensed under the terms of the
 Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
+#include <malloc.h>
 #include "sync.h"
 #include "sigproc.h"
 #include "pinfo.h"
@@ -15,8 +16,8 @@ details. */
 #include "security.h"
 #include "cygerrno.h"
 #ifdef DEBUGGING
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygheap.h"
 #endif
@@ -36,7 +37,6 @@ class lock_debug
  public:
   lock_debug () : acquired (0)
   {
-    extern int exit_state;
     if (locker && !exit_state)
       acquired = !!locker->acquire (INFINITE);
   }
@@ -54,7 +54,7 @@ class lock_debug
 
 muto NO_COPY *lock_debug::locker = NULL;
 
-static bool __stdcall mark_closed (const char *, int, HANDLE, const char *, BOOL);
+static bool __stdcall mark_closed (const char *, int, HANDLE, const char *, bool);
 
 void
 debug_init ()
@@ -78,7 +78,18 @@ out:
   return hl;
 }
 
-#ifdef DEBUGGING_AND_FDS_PROTECTED
+void
+verify_handle (const char *func, int ln, HANDLE h)
+{
+  handle_list *hl = find_handle (h);
+  if (!hl)
+    return;
+  system_printf ("%s:%d - multiple attempts to add handle %p", func, ln, h);
+
+  system_printf (" previously allocated by %s:%d(%s<%p>) winpid %d",
+		 hl->func, hl->ln, hl->name, hl->h, hl->pid);
+}
+
 void
 setclexec (HANDLE oh, HANDLE nh, bool not_inheriting)
 {
@@ -90,7 +101,6 @@ setclexec (HANDLE oh, HANDLE nh, bool not_inheriting)
       hl->h = nh;
     }
 }
-#endif
 
 /* Create a new handle record */
 static handle_list * __stdcall
@@ -113,6 +123,9 @@ add_handle (const char *func, int ln, HANDLE h, const char *name, bool inh)
   handle_list *hl;
   lock_debug here;
 
+  if (!cygheap)
+    return;
+
   if ((hl = find_handle (h)))
     {
       hl = hl->next;
@@ -168,11 +181,14 @@ debug_fixup_after_fork_exec ()
 }
 
 static bool __stdcall
-mark_closed (const char *func, int ln, HANDLE h, const char *name, BOOL force)
+mark_closed (const char *func, int ln, HANDLE h, const char *name, bool force)
 {
   handle_list *hl;
   lock_debug here;
 
+  if (!cygheap)
+    return true;
+
   if ((hl = find_handle (h)) && !force)
     {
       hl = hl->next;
@@ -180,7 +196,7 @@ mark_closed (const char *func, int ln, HANDLE h, const char *name, BOOL force)
       system_printf ("attempt to close protected handle %s:%d(%s<%p>) winpid %d",
 		     hl->func, hl->ln, hl->name, hl->h, hl->pid);
       system_printf (" by %s:%d(%s<%p>)", func, ln, name, h);
-      return FALSE;
+      return false;
     }
 
   handle_list *hln;
@@ -194,19 +210,19 @@ mark_closed (const char *func, int ln, HANDLE h, const char *name, BOOL force)
   if (hl)
     delete_handle (hl);
 
-  return TRUE;
+  return true;
 }
 
 /* Close a known handle.  Complain if !force and closing a known handle or
    if the name of the handle being closed does not match the registered name. */
-BOOL __stdcall
-close_handle (const char *func, int ln, HANDLE h, const char *name, BOOL force)
+bool __stdcall
+close_handle (const char *func, int ln, HANDLE h, const char *name, bool force)
 {
-  BOOL ret;
+  bool ret;
   lock_debug here;
 
   if (!mark_closed (func, ln, h, name, force))
-    return FALSE;
+    return false;
 
   ret = CloseHandle (h);
 
@@ -221,6 +237,7 @@ int __stdcall
 __set_errno (const char *func, int ln, int val)
 {
   debug_printf ("%s:%d val %d", func, ln, val);
+  _impure_ptr->_errno = val;
   return errno = val;
 }
 #endif /*DEBUGGING*/
