@@ -24,39 +24,6 @@ details. */
 #include "dtable.h"
 #include "cygheap.h"
 
-/* Cygwin internal */
-/* Return whether the directory of a file is writable.  Return 1 if it
-   is.  Otherwise, return 0, and set errno appropriately.  */
-int __stdcall
-writable_directory (const char *file)
-{
-#if 0
-  char dir[strlen (file) + 1];
-
-  strcpy (dir, file);
-
-  const char *usedir;
-  char *slash = strrchr (dir, '\\');
-  if (slash == NULL)
-    usedir = ".";
-  else if (slash == dir)
-    {
-      usedir = "\\";
-    }
-  else
-    {
-      *slash = '\0';
-      usedir = dir;
-    }
-
-  int acc = access (usedir, W_OK);
-
-  return acc == 0;
-#else
-  return 1;
-#endif
-}
-
 extern "C" int
 dirfd (DIR *dir)
 {
@@ -274,8 +241,6 @@ mkdir (const char *dir, mode_t mode)
     }
 
   nofinalslash (real_dir.get_win32 (), real_dir.get_win32 ());
-  if (! writable_directory (real_dir.get_win32 ()))
-    goto done;
 
   if (allow_ntsec && real_dir.has_acls ())
     set_security_attribute (S_IFDIR | ((mode & 07777) & ~cygheap->umask),
@@ -284,7 +249,7 @@ mkdir (const char *dir, mode_t mode)
   if (CreateDirectoryA (real_dir.get_win32 (), &sa))
     {
       if (!allow_ntsec && allow_ntea)
-	set_file_attribute (real_dir.has_acls (), real_dir.get_win32 (),
+	set_file_attribute (false, NULL, real_dir.get_win32 (),
 			    S_IFDIR | ((mode & 07777) & ~cygheap->umask));
 #ifdef HIDDEN_DOT_FILES
       char *c = strrchr (real_dir.get_win32 (), '\\');
@@ -328,6 +293,7 @@ rmdir (const char *dir)
 
       for (bool is_cwd = false; ; is_cwd = true)
         {
+	  DWORD err;
 	  int rc = RemoveDirectory (real_dir);
 	  DWORD att = GetFileAttributes (real_dir);
 
@@ -338,48 +304,51 @@ rmdir (const char *dir)
 	      /* RemoveDirectory on a samba drive doesn't return an error if the
 		 directory can't be removed because it's not empty. Checking for
 		 existence afterwards keeps us informed about success. */
-	      if (att != INVALID_FILE_ATTRIBUTES)
-		set_errno (ENOTEMPTY);
-	      else
-		res = 0;
+	      if (att == INVALID_FILE_ATTRIBUTES) 
+	        {
+		  res = 0;
+		  break;
+		}
+	      err = ERROR_DIR_NOT_EMPTY;
 	    }
 	  else
-	    {
-	      /* This kludge detects if we are attempting to remove the current working
-		 directory.  If so, we will move elsewhere to potentially allow the
-		 rmdir to succeed.  This means that cygwin's concept of the current working
-		 directory != Windows concept but, hey, whaddaregonnado?
-		 FIXME: A potential workaround for this is for cygwin apps to *never* call
-		 SetCurrentDirectory. */
-	      if (strcasematch (real_dir, cygheap->cwd.win32)
-		  && !strcasematch ("c:\\", cygheap->cwd.win32) && !is_cwd)
-	        {
-		  DWORD err = GetLastError ();
-		  if (!SetCurrentDirectory ("c:\\"))
-		    SetLastError (err);
-		  else 
-		    continue;
-		}
-	      if (res)
-	        {
-		  if (GetLastError () != ERROR_ACCESS_DENIED
-		      || !wincap.access_denied_on_delete ())
-		    __seterrno ();
-		  else
-		    set_errno (ENOTEMPTY);	/* On 9X ERROR_ACCESS_DENIED is
-						   returned if you try to remove a
-						   non-empty directory. */
+	    err = GetLastError ();
 
-		  /* If directory still exists, restore R/O attribute. */
-		  if (real_dir.has_attribute (FILE_ATTRIBUTE_READONLY))
-		    SetFileAttributes (real_dir, real_dir);
-		  if (is_cwd)
-		    SetCurrentDirectory (cygheap->cwd.win32);
-		}
-	    }
+	  /* This kludge detects if we are attempting to remove the current working
+	     directory.  If so, we will move elsewhere to potentially allow the
+	     rmdir to succeed.  This means that cygwin's concept of the current working
+	     directory != Windows concept but, hey, whaddaregonnado?
+	     Note that this will not cause something like the following to work:
+		     $ cd foo
+		     $ rmdir .
+	     since the shell will have foo "open" in the above case and so Windows will
+	     not allow the deletion. (Actually it does on 9X.)
+	     FIXME: A potential workaround for this is for cygwin apps to *never* call
+	     SetCurrentDirectory. */
+
+	  if (strcasematch (real_dir, cygheap->cwd.win32)
+	      && !strcasematch ("c:\\", cygheap->cwd.win32) 
+	      && !is_cwd
+	      && SetCurrentDirectory ("c:\\"))
+	    continue;
+	  
+	  /* On 9X ERROR_ACCESS_DENIED is returned 
+	     if you try to remove a non-empty directory. */
+	  if (err == ERROR_ACCESS_DENIED
+	      && wincap.access_denied_on_delete ())
+	    err = ERROR_DIR_NOT_EMPTY;
+	  
+	  __seterrno_from_win_error (err);
+	  
+	  /* Directory still exists, restore its characteristics. */
+	  if (real_dir.has_attribute (FILE_ATTRIBUTE_READONLY))
+	    SetFileAttributes (real_dir, real_dir);
+	  if (is_cwd)
+	    SetCurrentDirectory (real_dir);
 	  break;
 	}
     }
+
   syscall_printf ("%d = rmdir (%s)", res, dir);
   return res;
 }
