@@ -48,7 +48,7 @@ static NO_COPY struct
   X (NO_MORE_FILES,		ENMFILE),
   X (WRITE_PROTECT,		EROFS),
   X (BAD_UNIT,			ENODEV),
-  X (SHARING_VIOLATION,		EACCES),
+  X (SHARING_VIOLATION,		EBUSY),
   X (LOCK_VIOLATION,		EACCES),
   X (SHARING_BUFFER_EXCEEDED,	ENOLCK),
   X (HANDLE_EOF,		ENODATA),
@@ -101,10 +101,10 @@ static NO_COPY struct
   X (IO_DEVICE,			EIO),
   X (NOT_OWNER,			EPERM),
   X (END_OF_MEDIA,		ENOSPC),
-  X (EOM_OVERFLOW,		ENOSPC),
+  X (EOM_OVERFLOW,		EIO),
   X (BEGINNING_OF_MEDIA,	EIO),
   X (SETMARK_DETECTED,		EIO),
-  X (NO_DATA_DETECTED,		ENOSPC),
+  X (NO_DATA_DETECTED,		EIO),
   X (POSSIBLE_DEADLOCK,		EDEADLOCK),
   X (CRC,			EIO),
   X (NEGATIVE_SEEK,		EINVAL),
@@ -116,6 +116,10 @@ static NO_COPY struct
   X (INVALID_BLOCK_LENGTH,	EIO),
   X (BUS_RESET,			EIO),
   X (FILEMARK_DETECTED,		EIO),
+  X (NO_MEDIA_IN_DRIVE,		ENOMEDIUM),
+  X (DEVICE_REQUIRES_CLEANING,	EIO),
+  X (DEVICE_DOOR_OPEN,		EIO),
+  X (IO_PENDING,		EAGAIN),
   { 0, NULL, 0}
 };
 
