@@ -13,6 +13,7 @@ details. */
 #include <stdlib.h>
 #include <sys/cygwin.h>
 #include <sys/uio.h>
+#include <sys/acl.h>
 #include <signal.h>
 #include "cygerrno.h"
 #include "perprocess.h"
@@ -27,6 +28,8 @@ details. */
 #include <assert.h>
 #include <limits.h>
 #include <winioctl.h>
+#include <ntdef.h>
+#include "ntdll.h"
 
 static NO_COPY const int CHUNK_SIZE = 1024; /* Used for crlf conversions */
 
@@ -149,7 +152,6 @@ fhandler_base::set_name (path_conv &in_pc)
 {
   memcpy (&pc, &in_pc, in_pc.size ());
   pc.set_normalized_path (in_pc.normalized_path);
-  namehash = hash_path_name (0, get_win32_name ());
 }
 
 /* Detect if we are sitting at EOF for conditions where Windows
@@ -180,8 +182,8 @@ fhandler_base::set_flags (int flags, int supplied_bin)
   debug_printf ("flags %p, supplied_bin %p", flags, supplied_bin);
   if ((bin = flags & (O_BINARY | O_TEXT)))
     debug_printf ("O_TEXT/O_BINARY set in flags %p", bin);
-  else if (get_r_binset () && get_w_binset ())
-    bin = get_r_binary () ? O_BINARY : O_TEXT;	// FIXME: Not quite right
+  else if (rbinset () && wbinset ())
+    bin = rbinary () ? O_BINARY : O_TEXT;	// FIXME: Not quite right
   else if ((fmode = get_default_fmode (flags)) & O_BINARY)
     bin = O_BINARY;
   else if (fmode & O_TEXT)
@@ -189,14 +191,14 @@ fhandler_base::set_flags (int flags, int supplied_bin)
   else if (supplied_bin)
     bin = supplied_bin;
   else
-    bin = get_w_binary () || get_r_binary () || (binmode != O_TEXT)
+    bin = wbinary () || rbinary () || (binmode != O_TEXT)
 	  ? O_BINARY : O_TEXT;
 
   openflags = flags | bin;
 
   bin &= O_BINARY;
-  set_r_binary (bin);
-  set_w_binary (bin);
+  rbinary (bin ? true : false);
+  wbinary (bin ? true : false);
   syscall_printf ("filemode set to %s", bin ? "binary" : "text");
 }
 
@@ -424,7 +426,7 @@ done:
 
 /* Open system call handler function. */
 int
-fhandler_base::open (int flags, mode_t mode)
+fhandler_base::open_9x (int flags, mode_t mode)
 {
   int res = 0;
   HANDLE x;
@@ -432,9 +434,8 @@ fhandler_base::open (int flags, mode_t mode)
   int shared;
   int creation_distribution;
   SECURITY_ATTRIBUTES sa = sec_none;
-  security_descriptor sd;
 
-  syscall_printf ("(%s, %p) query_open %d", get_win32_name (), flags, get_query_open ());
+  syscall_printf ("(%s, %p)", get_win32_name (), flags);
 
   if (get_win32_name () == NULL)
     {
@@ -442,23 +443,13 @@ fhandler_base::open (int flags, mode_t mode)
       goto done;
     }
 
-  if (get_query_open ())
-    access = 0;
-  else if (get_major () == DEV_TAPE_MAJOR)
-    access = GENERIC_READ | GENERIC_WRITE;
-  else if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_RDONLY)
+  if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_RDONLY)
     access = GENERIC_READ;
   else if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_WRONLY)
     access = GENERIC_WRITE;
   else
     access = GENERIC_READ | GENERIC_WRITE;
 
-  /* Allow reliable lseek on disk devices. */
-  if (get_major () == DEV_FLOPPY_MAJOR)
-    access |= GENERIC_READ;
-
-  /* FIXME: O_EXCL handling?  */
-
   if ((flags & O_TRUNC) && ((flags & O_ACCMODE) != O_RDONLY))
     {
       if (flags & O_CREAT)
@@ -475,7 +466,7 @@ fhandler_base::open (int flags, mode_t mode)
     creation_distribution = CREATE_NEW;
 
   if (flags & O_APPEND)
-    set_append_p ();
+    append_mode (true);
 
   /* These flags are host dependent. */
   shared = wincap.shared ();
@@ -495,29 +486,158 @@ fhandler_base::open (int flags, mode_t mode)
     }
 #endif
 
-  /* CreateFile() with dwDesiredAccess == 0 when called on remote
-     share returns some handle, even if file doesn't exist. This code
-     works around this bug. */
-  if (get_query_open () && isremote () &&
-      creation_distribution == OPEN_EXISTING && !pc.exists ())
-    {
-      set_errno (ENOENT);
-      goto done;
-    }
-
   /* If mode has no write bits set, we set the R/O attribute. */
   if (!(mode & (S_IWUSR | S_IWGRP | S_IWOTH)))
     file_attributes |= FILE_ATTRIBUTE_READONLY;
 
-  /* If the file should actually be created and ntsec is on,
-     set files attributes. */
-  if (flags & O_CREAT && get_device () == FH_FS && allow_ntsec && has_acls ())
-    set_security_attribute (mode, &sa, sd);
-
   x = CreateFile (get_win32_name (), access, shared, &sa, creation_distribution,
 		  file_attributes, 0);
 
   if (x == INVALID_HANDLE_VALUE)
+    {
+      if (pc.isdir ())
+	{
+	  if (flags & (O_CREAT | O_EXCL) == (O_CREAT | O_EXCL))
+	    set_errno (EEXIST);
+	  else if (flags & (O_WRONLY | O_RDWR))
+	    set_errno (EISDIR);
+	  else
+	    nohandle (true);
+	}
+      else if (GetLastError () == ERROR_INVALID_HANDLE)
+	set_errno (ENOENT);
+      else
+	__seterrno ();
+      if (!nohandle ())
+	goto done;
+   }
+
+  syscall_printf ("%p = CreateFile (%s, %p, %p, %p, %p, %p, 0)",
+		  x, get_win32_name (), access, shared, &sa,
+		  creation_distribution, file_attributes);
+
+  set_io_handle (x);
+  set_flags (flags, pc.binmode ());
+
+  res = 1;
+  set_open_status ();
+done:
+  syscall_printf ("%d = fhandler_base::open (%s, %p)", res, get_win32_name (),
+		  flags);
+  return res;
+}
+
+/* Open system call handler function. */
+int
+fhandler_base::open (int flags, mode_t mode)
+{
+  if (!wincap.is_winnt ())
+    return fhandler_base::open_9x (flags, mode);
+
+  int res = 0;
+  HANDLE x;
+  ULONG file_attributes = 0;
+  ULONG shared = wincap.shared ();
+  ULONG create_disposition;
+  ULONG create_options;
+  SECURITY_ATTRIBUTES sa = sec_none;
+  security_descriptor sd;
+  UNICODE_STRING upath;
+  WCHAR wpath[CYG_MAX_PATH + 10];
+  OBJECT_ATTRIBUTES attr;
+  IO_STATUS_BLOCK io;
+  NTSTATUS status;
+
+  syscall_printf ("(%s, %p)", get_win32_name (), flags);
+  if (get_win32_name () == NULL)
+    {
+      set_errno (ENOENT);
+      goto done;
+    }
+
+  InitializeObjectAttributes (&attr, pc.get_nt_native_path (upath, wpath),
+			      OBJ_CASE_INSENSITIVE | OBJ_INHERIT,
+			      sa.lpSecurityDescriptor, NULL);
+
+  switch (query_open ())
+    {
+      case query_read_control:
+        access = READ_CONTROL;
+        create_options = FILE_OPEN_FOR_BACKUP_INTENT;
+        break;
+      case query_stat_control:
+        access = READ_CONTROL | FILE_READ_ATTRIBUTES;
+        create_options = FILE_OPEN_FOR_BACKUP_INTENT;
+        break;
+      case query_write_control:
+        access = READ_CONTROL | WRITE_OWNER | WRITE_DAC;
+        create_options = FILE_OPEN_FOR_BACKUP_INTENT | FILE_OPEN_FOR_RECOVERY;
+        break;
+      default:
+        create_options = 0;
+	if (get_major () == DEV_TAPE_MAJOR && (flags & O_TEXT))
+	  {
+	    /* O_TEXT is used to indicate write-through on tape devices */
+	    create_options |= FILE_WRITE_THROUGH;
+	    flags &= ~O_TEXT;
+	  }
+	if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_RDONLY)
+	  access = GENERIC_READ;
+	else if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_WRONLY)
+	  access = GENERIC_WRITE | FILE_READ_ATTRIBUTES;
+	else
+	  access = GENERIC_READ | GENERIC_WRITE;
+	if (get_major () != DEV_SERIAL_MAJOR && get_major () != DEV_TAPE_MAJOR)
+	  {
+	    create_options |= FILE_SYNCHRONOUS_IO_NONALERT;
+	    access |= SYNCHRONIZE;
+	  }
+	break;
+    }
+
+  if ((flags & O_TRUNC) && ((flags & O_ACCMODE) != O_RDONLY))
+    {
+      if (flags & O_CREAT)
+	create_disposition = FILE_SUPERSEDE;
+      else
+	create_disposition = FILE_OVERWRITE;
+    }
+  else if (flags & O_CREAT)
+    create_disposition = FILE_OPEN_IF;
+  else
+    create_disposition = FILE_OPEN;
+
+  if ((flags & O_EXCL) && (flags & O_CREAT))
+    create_disposition = FILE_CREATE;
+
+  if (flags & O_APPEND)
+    append_mode (true);
+
+  if (flags & O_CREAT && get_device () == FH_FS)
+    {
+      file_attributes = FILE_ATTRIBUTE_NORMAL;
+      /* If mode has no write bits set, we set the R/O attribute. */
+      if (!(mode & (S_IWUSR | S_IWGRP | S_IWOTH)))
+	file_attributes |= FILE_ATTRIBUTE_READONLY;
+#ifdef HIDDEN_DOT_FILES
+      char *c = strrchr (get_win32_name (), '\\');
+      if ((c && c[1] == '.') || *get_win32_name () == '.')
+	file_attributes |= FILE_ATTRIBUTE_HIDDEN;
+#endif
+      /* If the file should actually be created and ntsec is on,
+	 set files attributes. */
+      if (allow_ntsec && has_acls ())
+	{
+	  set_security_attribute (mode, &sa, sd);
+	  attr.SecurityDescriptor = sa.lpSecurityDescriptor;
+	}
+      /* The file attributes are needed for later use in, e.g. fchmod. */
+      pc.file_attributes (file_attributes);
+    }
+
+  status = NtCreateFile (&x, access, &attr, &io, NULL, file_attributes, shared,
+  			 create_disposition, create_options, NULL, 0);
+  if (!NT_SUCCESS (status))
     {
       if (!wincap.can_open_directories () && pc.isdir ())
 	{
@@ -526,19 +646,17 @@ fhandler_base::open (int flags, mode_t mode)
 	  else if (flags & (O_WRONLY | O_RDWR))
 	    set_errno (EISDIR);
 	  else
-	    set_nohandle (true);
+	    nohandle (true);
 	}
-      else if (GetLastError () == ERROR_INVALID_HANDLE)
-       set_errno (ENOENT);
-      else
-       __seterrno ();
-      if (!get_nohandle ())
-       goto done;
+      __seterrno_from_win_error (RtlNtStatusToDosError (status));
+      if (!nohandle ())
+	goto done;
    }
 
-  syscall_printf ("%p = CreateFile (%s, %p, %p, %p, %p, %p, 0)",
-		  x, get_win32_name (), access, shared, &sa,
-		  creation_distribution, file_attributes);
+  syscall_printf ("%x = NtCreateFile "
+  		  "(%p, %x, %s, io, NULL, %x, %x, %x, %x, NULL, 0)",
+		  status, x, access, get_win32_name (), file_attributes, shared,
+		  create_disposition, create_options);
 
   set_io_handle (x);
   set_flags (flags, pc.binmode ());
@@ -597,7 +715,7 @@ fhandler_base::read (void *in_ptr, size_t& len)
   else
     len = copied_chars;
 
-  if (get_r_binary () || len <= 0)
+  if (rbinary () || len <= 0)
     goto out;
 
   /* Scan buffer and turn \r\n into \n */
@@ -660,8 +778,7 @@ out:
   if (need_signal)
     SetEvent (read_state);
 
-  debug_printf ("returning %d, %s mode", len,
-		get_r_binary () ? "binary" : "text");
+  debug_printf ("returning %d, %s mode", len, rbinary () ? "binary" : "text");
   return;
 }
 
@@ -670,15 +787,15 @@ fhandler_base::write (const void *ptr, size_t len)
 {
   int res;
 
-  if (get_append_p ())
+  if (append_mode ())
     SetFilePointer (get_output_handle (), 0, 0, FILE_END);
-  else if (get_did_lseek ())
+  else if (did_lseek ())
     {
       _off64_t actual_length, current_position;
       DWORD size_high = 0;
       LONG pos_high = 0;
 
-      set_did_lseek (0); /* don't do it again */
+      did_lseek (false); /* don't do it again */
 
       actual_length = GetFileSize (get_output_handle (), &size_high);
       actual_length += ((_off64_t) size_high) << 32;
@@ -745,7 +862,7 @@ fhandler_base::write (const void *ptr, size_t len)
 	}
     }
 
-  if (get_w_binary ())
+  if (wbinary ())
     {
       debug_printf ("binary write");
       res = raw_write (ptr, len);
@@ -962,6 +1079,7 @@ fhandler_base::lseek (_off64_t offset, int whence)
   if (res == INVALID_SET_FILE_POINTER && GetLastError ())
     {
       __seterrno ();
+      res = -1;
     }
   else
     {
@@ -970,7 +1088,7 @@ fhandler_base::lseek (_off64_t offset, int whence)
 
       /* When next we write(), we will check to see if *this* seek went beyond
 	 the end of the file, and back-seek and fill with zeros if so - DJ */
-      set_did_lseek ();
+      did_lseek (true);
 
       /* If this was a SEEK_CUR with offset 0, we still might have
 	 readahead that we have to take into account when calculating
@@ -988,7 +1106,7 @@ fhandler_base::close ()
   int res = -1;
 
   syscall_printf ("closing '%s' handle %p", get_name (), get_handle ());
-  if (get_nohandle () || CloseHandle (get_handle ()))
+  if (nohandle () || CloseHandle (get_handle ()))
     res = 0;
   else
     {
@@ -1029,36 +1147,40 @@ fhandler_base::lock (int, struct __flock64 *)
 }
 
 extern "C" char * __stdcall
-rootdir (char *full_path)
+rootdir (const char *full_path, char *root_path)
 {
   /* Possible choices:
    * d:... -> d:/
    * \\server\share... -> \\server\share\
-   * else current drive.
    */
-  char *root = full_path;
+  int len;
+  char *rootp = root_path;
 
   if (full_path[1] == ':')
-    strcpy (full_path + 2, "\\");
+    {
+      *rootp++ = *full_path++;
+      *rootp++ = ':';
+    }
   else if (full_path[0] == '\\' && full_path[1] == '\\')
     {
-      char *cp = full_path + 2;
-      while (*cp && *cp != '\\')
-	cp++;
-      if (!*cp)
-	{
-	  set_errno (ENOTDIR);
-	  return NULL;
-	}
-      cp++;
-      while (*cp && *cp != '\\')
-	cp++;
-      strcpy (cp, "\\");
+      const char *cp = strchr (full_path + 2, '\\');
+      if (!cp)
+	goto error;
+      while (*++cp && *cp != '\\')
+	;
+      memcpy (root_path, full_path, (len = cp - full_path));
+      rootp = root_path + len;
     }
   else
-    root = NULL;
+    {
+    error:
+      set_errno (ENOTDIR);
+      return NULL;
+    }
 
-  return root;
+  *rootp++ = '\\';
+  *rootp = '\0';
+  return root_path;
 }
 
 int __stdcall
@@ -1109,7 +1231,7 @@ fhandler_base::init (HANDLE f, DWORD a, mode_t bin)
     flags = O_RDWR;
   set_flags (flags | bin);
   set_open_status ();
-  debug_printf ("created new fhandler_base for handle %p, bin %d", f, get_r_binary ());
+  debug_printf ("created new fhandler_base for handle %p, bin %d", f, rbinary ());
 }
 
 void
@@ -1124,7 +1246,7 @@ fhandler_base::dup (fhandler_base *child)
   debug_printf ("in fhandler_base dup");
 
   HANDLE nh;
-  if (!get_nohandle ())
+  if (!nohandle ())
     {
       if (!DuplicateHandle (hMainProc, get_handle (), hMainProc, &nh, 0, TRUE,
 			    DUPLICATE_SAME_ACCESS))
@@ -1148,10 +1270,10 @@ int fhandler_base::fcntl (int cmd, void *arg)
   switch (cmd)
     {
     case F_GETFD:
-      res = get_close_on_exec () ? FD_CLOEXEC : 0;
+      res = close_on_exec () ? FD_CLOEXEC : 0;
       break;
     case F_SETFD:
-      set_close_on_exec ((int) arg);
+      set_close_on_exec (((int) arg & FD_CLOEXEC) ? 1 : 0);
       res = 0;
       break;
     case F_GETFL:
@@ -1256,8 +1378,9 @@ fhandler_base::operator delete (void *p)
 }
 
 /* Normal I/O constructor */
-fhandler_base::fhandler_base ():
-  status (0),
+fhandler_base::fhandler_base () :
+  status (),
+  open_status (),
   access (0),
   io_handle (NULL),
   namehash (0),
@@ -1267,7 +1390,6 @@ fhandler_base::fhandler_base ():
   raixget (0),
   raixput (0),
   rabuflen (0),
-  open_status (0),
   fs_flags (0),
   read_state (NULL),
   archetype (NULL),
@@ -1320,9 +1442,9 @@ void
 fhandler_base::fork_fixup (HANDLE parent, HANDLE &h, const char *name)
 {
   HANDLE oh = h;
-  if (/* !is_socket () && */ !get_close_on_exec ())
+  if (/* !is_socket () && */ !close_on_exec ())
     debug_printf ("handle %p already opened", h);
-  else if (!DuplicateHandle (parent, h, hMainProc, &h, 0, !get_close_on_exec (),
+  else if (!DuplicateHandle (parent, h, hMainProc, &h, 0, !close_on_exec (),
 			     DUPLICATE_SAME_ACCESS))
     system_printf ("%s - %E, handle %s<%p>", get_name (), name, h);
   else if (oh != h)
@@ -1330,11 +1452,11 @@ fhandler_base::fork_fixup (HANDLE parent, HANDLE &h, const char *name)
 }
 
 void
-fhandler_base::set_close_on_exec (int val)
+fhandler_base::set_close_on_exec (bool val)
 {
-  if (!get_nohandle ())
+  if (!nohandle ())
     set_no_inheritance (io_handle, val);
-  set_close_on_exec_flag (val);
+  close_on_exec (val);
   debug_printf ("set close_on_exec for %s to %d", get_name (), val);
 }
 
@@ -1342,7 +1464,7 @@ void
 fhandler_base::fixup_after_fork (HANDLE parent)
 {
   debug_printf ("inheriting '%s' from parent", get_name ());
-  if (!get_nohandle ())
+  if (!nohandle ())
     fork_fixup (parent, io_handle, "io_handle");
 }
 
@@ -1401,3 +1523,24 @@ fhandler_base::closedir (DIR *)
   set_errno (ENOTDIR);
   return -1;
 }
+
+int
+fhandler_base::fchmod (mode_t mode)
+{
+  /* By default, just succeeds. */
+  return 0;
+}
+
+int
+fhandler_base::fchown (__uid32_t uid, __gid32_t gid)
+{
+  /* By default, just succeeds. */
+  return 0;
+}
+
+int
+fhandler_base::facl (int cmd, int nentries, __aclent32_t *aclbufp)
+{
+  /* By default, just succeeds. */
+  return 0;
+}
