@@ -11,44 +11,6 @@ details. */
 #ifndef _FHANDLER_H_
 #define _FHANDLER_H_
 
-enum
-{
-  FH_RBINARY	= 0x00001000,	/* binary read mode */
-  FH_WBINARY	= 0x00002000,	/* binary write mode */
-  FH_CLOEXEC	= 0x00004000,	/* close-on-exec */
-  FH_RBINSET	= 0x00008000,	/* binary read mode has been explicitly set */
-  FH_WBINSET	= 0x00010000,	/* binary write mode has been explicitly set */
-  FH_APPEND	= 0x00020000,	/* always append */
-  FH_ASYNC	= 0x00040000,	/* async I/O */
-  FH_ENC	= 0x00080000,	/* native path is encoded */
-  FH_SYMLINK	= 0x00100000,	/* is a symlink */
-  FH_EXECABL	= 0x00200000,	/* file looked like it would run:
-				 * ends in .exe or .bat or begins with #! */
-  FH_LSEEKED	= 0x00400000,	/* set when lseek is called as a flag that
-				 * _write should check if we've moved beyond
-				 * EOF, zero filling or making file sparse
-				   if so. */
-  FH_NOHANDLE	= 0x00800000,	/* No handle associated with fhandler. */
-  FH_NOEINTR	= 0x01000000,	/* Set if I/O should be uninterruptible. */
-  FH_FFIXUP	= 0x02000000,	/* Set if need to fixup after fork. */
-  FH_LOCAL	= 0x04000000,	/* File is unix domain socket */
-  FH_SHUTRD	= 0x08000000,	/* Socket saw a SHUT_RD */
-  FH_SHUTWR	= 0x10000000,	/* Socket saw a SHUT_WR */
-  FH_ISREMOTE	= 0x10000000,	/* File is on a remote drive */
-  FH_DCEXEC	= 0x20000000,	/* Don't care if this is executable */
-  FH_HASACLS	= 0x40000000,	/* True if fs of file has ACLS */
-  FH_QUERYOPEN	= 0x80000000,	/* open file without requesting either read
-				   or write access */
-};
-
-#define FHDEVN(n)	(n)
-#define FHISSETF(x)	__ISSETF (this, x, FH)
-#define FHSETF(x)	__SETF (this, x, FH)
-#define FHCLEARF(x)	__CLEARF (this, x, FH)
-#define FHCONDSETF(n, x) __CONDSETF(n, this, x, FH)
-
-#define FHSTATOFF	0
-
 /* fcntl flags used only internaly. */
 #define O_NOSYMLINK 0x080000
 #define O_DIROPEN   0x100000
@@ -62,10 +24,6 @@ enum
    both flags are set. */
 #define O_NONBLOCK_MASK (O_NONBLOCK | OLD_O_NDELAY)
 
-#define UNCONNECTED     0
-#define CONNECT_PENDING 1
-#define CONNECTED       2
-
 extern const char *windows_device_names[];
 extern struct __cygwin_perfile *perfile_table;
 #define __fmode (*(user_data->fmode_ptr))
@@ -77,6 +35,14 @@ class fhandler_disk_file;
 typedef struct __DIR DIR;
 struct dirent;
 struct iovec;
+struct __acl32;
+
+enum conn_state
+{
+  unconnected = 0,
+  connect_pending = 1,
+  connected = 2
+};
 
 enum line_edit_status
 {
@@ -95,12 +61,44 @@ enum bg_check_types
   bg_signalled = 2
 };
 
+enum query_state {
+  no_query = 0,
+  query_read_control = 1,
+  query_stat_control = 2,
+  query_write_control = 3
+};
+
 class fhandler_base
 {
   friend class dtable;
   friend void close_all_files ();
- protected:
-  DWORD status;
+
+  struct status_flags
+  {
+    unsigned rbinary            : 1; /* binary read mode */
+    unsigned rbinset            : 1; /* binary read mode explicitly set */
+    unsigned wbinary            : 1; /* binary write mode */
+    unsigned wbinset            : 1; /* binary write mode explicitly set */
+    unsigned nohandle           : 1; /* No handle associated with fhandler. */
+    unsigned uninterruptible_io : 1; /* Set if I/O should be uninterruptible. */
+    unsigned append_mode        : 1; /* always append */
+    unsigned did_lseek          : 1; /* set when lseek is called as a flag that
+					_write should check if we've moved
+					beyond EOF, zero filling or making
+					file sparse if so. */
+    unsigned query_open         : 2; /* open file without requesting either
+    					read or write access */
+    unsigned close_on_exec      : 1; /* close-on-exec */
+    unsigned need_fork_fixup    : 1; /* Set if need to fixup after fork. */
+
+   public:
+    status_flags () :
+      rbinary (0), rbinset (0), wbinary (0), wbinset (0), nohandle (0),
+      uninterruptible_io (0), append_mode (0), did_lseek (0),
+      query_open (no_query), close_on_exec (0), need_fork_fixup (0)
+      {}
+  } status, open_status;
+
  private:
   int access;
   HANDLE io_handle;
@@ -117,7 +115,6 @@ class fhandler_base
   size_t raixput;
   size_t rabuflen;
 
-  DWORD open_status;
   DWORD fs_flags;
   HANDLE read_state;
   path_conv pc;
@@ -150,98 +147,56 @@ class fhandler_base
   int get_access () const { return access; }
   void set_access (int x) { access = x; }
 
-  bool get_async () { return FHISSETF (ASYNC); }
-  void set_async (int x) { FHCONDSETF (x, ASYNC); }
-
   int get_flags () { return openflags; }
   void set_flags (int x, int supplied_bin = 0);
 
   bool is_nonblocking ();
   void set_nonblocking (int yes);
 
-  bool get_w_binary () { return FHISSETF (WBINSET) ? FHISSETF (WBINARY) : 1; }
-  bool get_r_binary () { return FHISSETF (RBINSET) ? FHISSETF (RBINARY) : 1; }
+  bool wbinary () const { return status.wbinset ? status.wbinary : 1; }
+  bool rbinary () const { return status.rbinset ? status.rbinary : 1; }
 
-  bool get_w_binset () { return FHISSETF (WBINSET); }
-  bool get_r_binset () { return FHISSETF (RBINSET); }
-
-  void set_w_binary (int b) { FHCONDSETF (b, WBINARY); FHSETF (WBINSET); }
-  void set_r_binary (int b) { FHCONDSETF (b, RBINARY); FHSETF (RBINSET); }
-  void clear_w_binary () {FHCLEARF (WBINARY); FHCLEARF (WBINSET); }
-  void clear_r_binary () {FHCLEARF (RBINARY); FHCLEARF (RBINSET); }
-
-  bool get_nohandle () { return FHISSETF (NOHANDLE); }
-  void set_nohandle (int x) { FHCONDSETF (x, NOHANDLE); }
+  void wbinary (bool b) {status.wbinary = b; status.wbinset = 1;}
+  void rbinary (bool b) {status.rbinary = b; status.rbinset = 1;}
 
   void set_open_status () {open_status = status;}
-  DWORD get_open_status () {return open_status;}
   void reset_to_open_binmode ()
   {
     set_flags ((get_flags () & ~(O_TEXT | O_BINARY))
-	       | ((open_status & (FH_WBINARY | FH_RBINARY)
-		   ? O_BINARY : O_TEXT)));
+	       | ((open_status.wbinary || open_status.rbinary)
+		   ? O_BINARY : O_TEXT));
   }
 
+  IMPLEMENT_STATUS_FLAG (bool, wbinset)
+  IMPLEMENT_STATUS_FLAG (bool, rbinset)
+  IMPLEMENT_STATUS_FLAG (bool, nohandle)
+  IMPLEMENT_STATUS_FLAG (bool, uninterruptible_io)
+  IMPLEMENT_STATUS_FLAG (bool, append_mode)
+  IMPLEMENT_STATUS_FLAG (bool, did_lseek)
+  IMPLEMENT_STATUS_FLAG (query_state, query_open)
+  IMPLEMENT_STATUS_FLAG (bool, close_on_exec)
+  IMPLEMENT_STATUS_FLAG (bool, need_fork_fixup)
+
   int get_default_fmode (int flags);
 
-  bool get_r_no_interrupt () { return FHISSETF (NOEINTR); }
-  void set_r_no_interrupt (bool b) { FHCONDSETF (b, NOEINTR); }
-
-  bool get_close_on_exec () { return FHISSETF (CLOEXEC); }
-  int set_close_on_exec_flag (int b) { return FHCONDSETF (b, CLOEXEC); }
+  virtual void set_close_on_exec (bool val);
 
   LPSECURITY_ATTRIBUTES get_inheritance (bool all = 0)
   {
     if (all)
-      return get_close_on_exec () ? &sec_all_nih : &sec_all;
+      return close_on_exec () ? &sec_all_nih : &sec_all;
     else
-      return get_close_on_exec () ? &sec_none_nih : &sec_none;
+      return close_on_exec () ? &sec_none_nih : &sec_none;
   }
 
-  void set_did_lseek (int b = 1) { FHCONDSETF (b, LSEEKED); }
-  bool get_did_lseek () { return FHISSETF (LSEEKED); }
-
-  bool get_need_fork_fixup () { return FHISSETF (FFIXUP); }
-  void set_need_fork_fixup () { FHSETF (FFIXUP); }
-
-  bool get_encoded () { return FHISSETF (ENC);}
-  void set_encoded () { FHSETF (ENC);}
-
-  virtual void set_close_on_exec (int val);
-
   virtual void fixup_before_fork_exec (DWORD) {}
   virtual void fixup_after_fork (HANDLE);
   virtual void fixup_after_exec () {}
 
-  bool get_symlink_p () { return FHISSETF (SYMLINK); }
-  void set_symlink_p (int val) { FHCONDSETF (val, SYMLINK); }
-  void set_symlink_p () { FHSETF (SYMLINK); }
-
-  bool get_socket_p () { return FHISSETF (LOCAL); }
-  void set_socket_p (int val) { FHCONDSETF (val, LOCAL); }
-  void set_socket_p () { FHSETF (LOCAL); }
-
-  bool get_execable_p () { return FHISSETF (EXECABL); }
-  void set_execable_p (executable_states val)
-  {
-    FHCONDSETF (val == is_executable, EXECABL);
-    FHCONDSETF (val == dont_care_if_executable, DCEXEC);
-  }
-  void set_execable_p () { FHSETF (EXECABL); }
-  bool dont_care_if_execable () { return FHISSETF (DCEXEC); }
-  bool exec_state_isknown () { return FHISSETF (DCEXEC) || FHISSETF (EXECABL); }
-
-  bool get_append_p () { return FHISSETF (APPEND); }
-  void set_append_p (int val) { FHCONDSETF (val, APPEND); }
-  void set_append_p () { FHSETF (APPEND); }
-
   void set_fs_flags (DWORD flags) { fs_flags = flags; }
   bool get_fs_flags (DWORD flagval = UINT32_MAX)
     { return (fs_flags & (flagval)); }
 
-  bool get_query_open () { return FHISSETF (QUERYOPEN); }
-  void set_query_open (bool val) { FHCONDSETF (val, QUERYOPEN); }
-
   bool get_readahead_valid () { return raixget < ralen; }
   int puts_readahead (const char *s, size_t len = (size_t) -1);
   int put_readahead (char value);
@@ -262,7 +217,7 @@ class fhandler_base
   bool has_attribute (DWORD x) const {return pc.has_attribute (x);}
   const char *get_name () const { return pc.normalized_path; }
   const char *get_win32_name () { return pc.get_win32 (); }
-  __ino64_t get_namehash () { return namehash; }
+    __ino64_t get_namehash () { return namehash ?: namehash = hash_path_name (0, get_win32_name ()); }
 
   virtual void hclose (HANDLE h) {CloseHandle (h);}
   virtual void set_no_inheritance (HANDLE &h, int not_inheriting);
@@ -271,6 +226,7 @@ class fhandler_base
   void fork_fixup (HANDLE parent, HANDLE &h, const char *name);
   virtual bool need_fixup_before () const {return false;}
 
+  int open_9x (int flags, mode_t mode = 0);
   virtual int open (int flags, mode_t mode = 0);
   int open_fs (int flags, mode_t mode = 0);
   virtual int close ();
@@ -289,6 +245,9 @@ class fhandler_base
     __attribute__ ((regparm (3)));
   int __stdcall fstat_by_handle (struct __stat64 *buf) __attribute__ ((regparm (2)));
   int __stdcall fstat_by_name (struct __stat64 *buf) __attribute__ ((regparm (2)));
+  virtual int __stdcall fchmod (mode_t mode) __attribute__ ((regparm (1)));
+  virtual int __stdcall fchown (__uid32_t uid, __gid32_t gid) __attribute__ ((regparm (2)));
+  virtual int __stdcall facl (int, int, __acl32 *) __attribute__ ((regparm (3)));
   virtual int ioctl (unsigned int cmd, void *);
   virtual int fcntl (int cmd, void *);
   virtual char const *ttyname () { return get_name (); }
@@ -376,7 +335,24 @@ class fhandler_socket: public fhandler_base
   HANDLE secret_event;
   struct _WSAPROTOCOL_INFOA *prot_info_ptr;
   char *sun_path;
-  int had_connect_or_listen;
+  struct status_flags
+  {
+    unsigned async_io              : 1; /* async I/O */
+    unsigned saw_shutdown_read     : 1; /* Socket saw a SHUT_RD */
+    unsigned saw_shutdown_write    : 1; /* Socket saw a SHUT_WR */
+    unsigned closed		   : 1;
+    unsigned owner		   : 1;
+    unsigned connect_state         : 2;
+   public:
+    status_flags () :
+      async_io (0), saw_shutdown_read (0), saw_shutdown_write (0),
+      closed (0), owner (0), connect_state (unconnected)
+      {}
+  } status;
+
+  bool prepare (HANDLE &event, long event_mask);
+  int wait (HANDLE event, int flags);
+  void release (HANDLE event);
 
  public:
   fhandler_socket ();
@@ -384,17 +360,12 @@ class fhandler_socket: public fhandler_base
   int get_socket () { return (int) get_handle(); }
   fhandler_socket *is_socket () { return this; }
 
-  bool saw_shutdown_read () const {return FHISSETF (SHUTRD);}
-  bool saw_shutdown_write () const {return FHISSETF (SHUTWR);}
-
-  void set_shutdown_read () {FHSETF (SHUTRD);}
-  void set_shutdown_write () {FHSETF (SHUTWR);}
-
-  bool is_unconnected () const {return had_connect_or_listen == UNCONNECTED;}
-  bool is_connect_pending () const {return had_connect_or_listen == CONNECT_PENDING;}
-  bool is_connected () const {return had_connect_or_listen == CONNECTED;}
-  void set_connect_state (int newstate) { had_connect_or_listen = newstate; }
-  int get_connect_state () const { return had_connect_or_listen; }
+  IMPLEMENT_STATUS_FLAG (bool, async_io)
+  IMPLEMENT_STATUS_FLAG (bool, saw_shutdown_read)
+  IMPLEMENT_STATUS_FLAG (bool, saw_shutdown_write)
+  IMPLEMENT_STATUS_FLAG (bool, closed)
+  IMPLEMENT_STATUS_FLAG (bool, owner)
+  IMPLEMENT_STATUS_FLAG (conn_state, connect_state)
 
   int bind (const struct sockaddr *name, int namelen);
   int connect (const struct sockaddr *name, int namelen);
@@ -421,7 +392,7 @@ class fhandler_socket: public fhandler_base
   void hclose (HANDLE) {close ();}
   int dup (fhandler_base *child);
 
-  void set_close_on_exec (int val);
+  void set_close_on_exec (bool val);
   virtual void fixup_before_fork_exec (DWORD);
   void fixup_after_fork (HANDLE);
   void fixup_after_exec ();
@@ -460,7 +431,7 @@ public:
   select_record *select_read (select_record *s);
   select_record *select_write (select_record *s);
   select_record *select_except (select_record *s);
-  void set_close_on_exec (int val);
+  void set_close_on_exec (bool val);
   void __stdcall read (void *ptr, size_t& len) __attribute__ ((regparm (3)));
   int close ();
   void create_guard (SECURITY_ATTRIBUTES *sa) {guard = CreateMutex (sa, FALSE, NULL);}
@@ -481,7 +452,6 @@ class fhandler_fifo: public fhandler_pipe
 {
   HANDLE output_handle;
   HANDLE owner;		// You can't have too many mutexes, now, can you?
-  ATOM upand;
   long read_use;
   long write_use;
 public:
@@ -496,7 +466,6 @@ public:
   void set_use ();
   int dup (fhandler_base *child);
   bool is_slow () {return 1;}
-  ATOM& get_atom () {return upand;}
 };
 
 class fhandler_dev_raw: public fhandler_base
@@ -506,33 +475,35 @@ class fhandler_dev_raw: public fhandler_base
   size_t devbufsiz;
   size_t devbufstart;
   size_t devbufend;
-  int eom_detected    : 1;
-  int eof_detected    : 1;
-  int lastblk_to_read : 1;
-  int is_writing      : 1;
-  int has_written     : 1;
-  int varblkop	      : 1;
+  struct status_flags
+  {
+    unsigned eom_detected    : 1;
+    unsigned eof_detected    : 1;
+    unsigned lastblk_to_read : 1;
+    unsigned is_writing      : 1;
+   public:
+    status_flags () :
+      eom_detected (0), eof_detected (0), lastblk_to_read (0), is_writing (0)
+      {}
+  } status;
+ 
+  IMPLEMENT_STATUS_FLAG (bool, eom_detected)
+  IMPLEMENT_STATUS_FLAG (bool, eof_detected)
+  IMPLEMENT_STATUS_FLAG (bool, lastblk_to_read)
+  IMPLEMENT_STATUS_FLAG (bool, is_writing)
 
-  virtual void clear (void);
   virtual BOOL write_file (const void *buf, DWORD to_write,
 			   DWORD *written, int *err);
   virtual BOOL read_file (void *buf, DWORD to_read, DWORD *read, int *err);
   virtual int writebuf (void);
 
   /* returns not null, if `win_error' determines an end of media condition */
-  virtual int is_eom(int win_error) = 0;
+  virtual int is_eom(int win_error);
   /* returns not null, if `win_error' determines an end of file condition */
-  virtual int is_eof(int win_error) = 0;
+  virtual int is_eof(int win_error);
 
   fhandler_dev_raw ();
 
-  inline void reset_devbuf (void)
-    {
-      devbufstart = devbufend = 0;
-      eom_detected = eof_detected = 0;
-      lastblk_to_read = is_writing = has_written = 0;
-    }
-
  public:
   ~fhandler_dev_raw (void);
 
@@ -571,18 +542,15 @@ class fhandler_dev_floppy: public fhandler_dev_raw
 
 class fhandler_dev_tape: public fhandler_dev_raw
 {
-  int lasterr;
-  TAPE_GET_DRIVE_PARAMETERS dp;
+  HANDLE mt_mtx;
+  HANDLE mt_evt;
 
-  bool is_rewind_device () { return get_unit () < 128; }
+  bool is_rewind_device () { return get_minor () < 128; }
+  unsigned int driveno () { return (unsigned int) get_minor () & 0x7f; }
+  void drive_init (void);
 
- protected:
-  virtual void clear (void);
-  virtual int is_eom (int win_error);
-  virtual int is_eof (int win_error);
-  virtual BOOL write_file (const void *buf, DWORD to_write,
-			   DWORD *written, int *err);
-  virtual BOOL read_file (void *buf, DWORD to_read, DWORD *read, int *err);
+  inline bool _lock ();
+  inline int unlock (int ret = 0);
 
  public:
   fhandler_dev_tape ();
@@ -590,33 +558,15 @@ class fhandler_dev_tape: public fhandler_dev_raw
   virtual int open (int flags, mode_t mode = 0);
   virtual int close (void);
 
+  void raw_read (void *ptr, size_t& ulen);
+  int raw_write (const void *ptr, size_t ulen);
+
   virtual _off64_t lseek (_off64_t offset, int whence);
 
   virtual int __stdcall fstat (struct __stat64 *buf) __attribute__ ((regparm (2)));
 
   virtual int dup (fhandler_base *child);
-
   virtual int ioctl (unsigned int cmd, void *buf);
-
- private:
-  inline bool tape_get_feature (DWORD parm)
-    {
-      return ((parm & TAPE_DRIVE_HIGH_FEATURES)
-	      ? ((dp.FeaturesHigh & parm) != 0)
-	      : ((dp.FeaturesLow & parm) != 0));
-    }
-  int tape_error (const char *txt);
-  int tape_write_marks (int marktype, DWORD len);
-  int tape_get_pos (unsigned long *block, unsigned long *partition = NULL);
-  int tape_set_pos (int mode, long count, bool sfm_func = false);
-  int _tape_set_pos (int mode, long count, int partition = 0);
-  int tape_erase (int mode);
-  int tape_prepare (int action);
-  int tape_set_blocksize (long count);
-  int tape_status (struct mtget *get);
-  int tape_compression (long count);
-  int tape_partition (long count);
-  int tape_set_partition (long count);
 };
 
 /* Standard disk file */
@@ -631,6 +581,9 @@ class fhandler_disk_file: public fhandler_base
   int lock (int, struct __flock64 *);
   bool isdevice () { return false; }
   int __stdcall fstat (struct __stat64 *buf) __attribute__ ((regparm (2)));
+  int __stdcall fchmod (mode_t mode) __attribute__ ((regparm (1)));
+  int __stdcall fchown (__uid32_t uid, __gid32_t gid) __attribute__ ((regparm (2)));
+  int __stdcall facl (int, int, __acl32 *) __attribute__ ((regparm (3)));
 
   HANDLE mmap (caddr_t *addr, size_t len, DWORD access, int flags, _off64_t off);
   int munmap (HANDLE h, caddr_t addr, size_t len);
@@ -729,7 +682,7 @@ class fhandler_termios: public fhandler_base
   fhandler_termios () :
   fhandler_base ()
   {
-    set_need_fork_fixup ();
+    need_fork_fixup (true);
   }
   HANDLE& get_output_handle () { return output_handle; }
   line_edit_status line_edit (const char *rptr, int nread, termios&);
@@ -875,7 +828,7 @@ class fhandler_console: public fhandler_termios
   select_record *select_write (select_record *s);
   select_record *select_except (select_record *s);
   void fixup_after_exec ();
-  void set_close_on_exec (int val);
+  void set_close_on_exec (bool val);
   void fixup_after_fork (HANDLE parent);
   void set_input_state ();
   void send_winch_maybe ();
@@ -911,7 +864,7 @@ class fhandler_tty_common: public fhandler_termios
   tty *get_ttyp () { return (tty *) tc; }
 
   int close ();
-  void set_close_on_exec (int val);
+  void set_close_on_exec (bool val);
   void fixup_after_fork (HANDLE parent);
   select_record *select_read (select_record *s);
   select_record *select_write (select_record *s);
@@ -972,7 +925,7 @@ public:
   _off64_t lseek (_off64_t, int) { return 0; }
   char *ptsname ();
 
-  void set_close_on_exec (int val);
+  void set_close_on_exec (bool val);
   bool hit_eof ();
   int get_unit () const { return slave.minor; }
 };
@@ -1099,7 +1052,7 @@ class fhandler_windows: public fhandler_base
   _off64_t lseek (_off64_t, int) { return 0; }
   int close (void) { return 0; }
 
-  void set_close_on_exec (int val);
+  void set_close_on_exec (bool val);
   void fixup_after_fork (HANDLE parent);
   select_record *select_read (select_record *s);
   select_record *select_write (select_record *s);
@@ -1107,14 +1060,20 @@ class fhandler_windows: public fhandler_base
   bool is_slow () {return 1;}
 };
 
-class fhandler_dev_dsp : public fhandler_base
+class fhandler_dev_dsp: public fhandler_base
 {
+ public:
+  class Audio;
+  class Audio_out;
+  class Audio_in;
  private:
   int audioformat_;
   int audiofreq_;
   int audiobits_;
   int audiochannels_;
-  bool setupwav(const char *pData, int nBytes);
+  static int open_count; // per process
+  Audio_out *audio_out_;
+  Audio_in  *audio_in_;
  public:
   fhandler_dev_dsp ();
   ~fhandler_dev_dsp();
@@ -1127,6 +1086,7 @@ class fhandler_dev_dsp : public fhandler_base
   int close (void);
   int dup (fhandler_base *child);
   void dump (void);
+  void fixup_after_fork (HANDLE parent);
   void fixup_after_exec ();
 };
 
@@ -1156,6 +1116,9 @@ class fhandler_virtual : public fhandler_base
   int open (int flags, mode_t mode = 0);
   int close (void);
   int __stdcall fstat (struct stat *buf) __attribute__ ((regparm (2)));
+  int __stdcall fchmod (mode_t mode) __attribute__ ((regparm (1)));
+  int __stdcall fchown (__uid32_t uid, __gid32_t gid) __attribute__ ((regparm (2)));
+  int __stdcall facl (int, int, __acl32 *) __attribute__ ((regparm (3)));
   virtual bool fill_filebuf ();
   void fixup_after_exec ();
 };
