@@ -29,6 +29,7 @@ details. */
 #include "pinfo.h"
 #include "shared_info.h"
 #include "cygthread.h"
+#include "cygtls.h"
 
 #define CONVERT_LIMIT 16384
 
@@ -154,10 +155,8 @@ set_console_state_for_spawn ()
   if (shared_console_info != NULL)
     {
       /* ACK.  Temporarily define for use in TTYSETF macro */
-#     define tc &shared_console_info->tty_min_state
       SetConsoleMode (h, ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT | ENABLE_PROCESSED_INPUT);
-      TTYSETF (RSTCONS);
-#     undef tc
+      shared_console_info->tty_min_state.rstcons (true);
     }
 
   CloseHandle (h);
@@ -252,7 +251,7 @@ fhandler_console::read (void *pv, size_t& buflen)
   char tmp[60];
 
   w4[0] = h;
-  if (cygthread::is ())
+  if (&_my_tls != _main_tls)
     nwait = 1;
   else
     {
@@ -538,7 +537,7 @@ sig_exit:
 void
 fhandler_console::set_input_state ()
 {
-  if (TTYISSETF (RSTCONS))
+  if (tc->rstcons ())
     input_tcsetattr (0, &tc->ti);
 }
 
@@ -635,7 +634,7 @@ fhandler_console::open (int flags, mode_t)
       return 0;
     }
   set_io_handle (h);
-  set_r_no_interrupt (1);	// Handled explicitly in read code
+  uninterruptible_io (true);	// Handled explicitly in read code
 
   h = CreateFile ("CONOUT$", GENERIC_READ | GENERIC_WRITE,
 		  FILE_SHARE_READ | FILE_SHARE_WRITE, &sec_none,
@@ -660,7 +659,7 @@ fhandler_console::open (int flags, mode_t)
       SetConsoleMode (get_io_handle (), ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT | cflags);
     }
 
-  TTYCLEARF (RSTCONS);
+  tc->rstcons (false);
   set_open_status ();
   cygheap->open_fhs++;
   debug_printf ("incremented open_fhs, now %d", cygheap->open_fhs);
@@ -791,7 +790,7 @@ fhandler_console::input_tcsetattr (int, struct termios const *t)
 
 #if 0
   /* Enable/disable LF -> CRLF conversions */
-  set_r_binary ((t->c_iflag & INLCR) ? 0 : 1);
+  rbinary ((t->c_iflag & INLCR) ? false : true);
 #endif
 
   /* There's some disparity between what we need and what's
@@ -844,7 +843,7 @@ fhandler_console::input_tcsetattr (int, struct termios const *t)
 		      res, t, flags, t->c_lflag, t->c_iflag);
     }
 
-  TTYCLEARF (RSTCONS);
+  tc->rstcons (false);
   return res;
 }
 
@@ -1744,7 +1743,7 @@ fhandler_console::igncr_enabled (void)
 }
 
 void
-fhandler_console::set_close_on_exec (int val)
+fhandler_console::set_close_on_exec (bool val)
 {
   fhandler_base::set_close_on_exec (val);
   set_no_inheritance (output_handle, val);
@@ -1763,7 +1762,7 @@ fhandler_console::fixup_after_fork (HANDLE)
   if (!open (O_NOCTTY | get_flags (), 0))
     system_printf ("error opening console after fork, %E");
 
-  if (!get_close_on_exec ())
+  if (!close_on_exec ())
     {
       CloseHandle (h);
       CloseHandle (oh);
@@ -1777,10 +1776,10 @@ set_console_title (char *title)
   char buf[257];
   strncpy (buf, title, sizeof (buf) - 1);
   buf[sizeof (buf) - 1] = '\0';
-  if ((rc = WaitForSingleObject (title_mutex, 15000)) != WAIT_OBJECT_0)
+  if ((rc = WaitForSingleObject (tty_mutex, 15000)) != WAIT_OBJECT_0)
     sigproc_printf ("wait for title mutex failed rc %d, %E", rc);
   SetConsoleTitle (buf);
-  ReleaseMutex (title_mutex);
+  ReleaseMutex (tty_mutex);
   debug_printf ("title '%s'", buf);
 }
 
