@@ -1,6 +1,6 @@
 /* fhandler_disk_file.cc
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -12,17 +12,20 @@ details. */
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/cygwin.h>
+#include <sys/acl.h>
 #include <signal.h>
 #include "cygerrno.h"
 #include "perprocess.h"
 #include "security.h"
 #include "cygwin/version.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygheap.h"
 #include "shared_info.h"
 #include "pinfo.h"
+#include <ntdef.h>
+#include "ntdll.h"
 #include <assert.h>
 #include <ctype.h>
 
@@ -92,31 +95,20 @@ path_conv::ndisk_links (DWORD nNumberOfLinks)
 }
 
 int __stdcall
-fhandler_disk_file::fstat_by_handle (struct __stat64 *buf, path_conv *pc)
+fhandler_base::fstat_by_handle (struct __stat64 *buf)
 {
-  int res = 0;
   BY_HANDLE_FILE_INFORMATION local;
-
-  /* NT 3.51 seems to have a bug when attempting to get vol serial
-     numbers.  This loop gets around this. */
-  for (int i = 0; i < 2; i++)
-    {
-      if (!(res = GetFileInformationByHandle (get_handle (), &local)))
-	break;
-      if (local.dwVolumeSerialNumber && (long) local.dwVolumeSerialNumber != -1)
-	break;
-    }
+  BOOL res = GetFileInformationByHandle (get_handle (), &local);
   debug_printf ("%d = GetFileInformationByHandle (%s, %d)",
 		res, get_win32_name (), get_handle ());
-  if (res == 0)
-    /* GetFileInformationByHandle will fail if it's given stdin/out/err
-       or a pipe*/
+  /* GetFileInformationByHandle will fail if it's given stdio handle or pipe*/
+  if (!res)
     {
       memset (&local, 0, sizeof (local));
       local.nFileSizeLow = GetFileSize (get_handle (), &local.nFileSizeHigh);
     }
 
-  return fstat_helper (buf, pc,
+  return fstat_helper (buf,
 		       local.ftCreationTime,
 		       local.ftLastAccessTime,
 		       local.ftLastWriteTime,
@@ -128,36 +120,36 @@ fhandler_disk_file::fstat_by_handle (struct __stat64 *buf, path_conv *pc)
 }
 
 int __stdcall
-fhandler_disk_file::fstat_by_name (struct __stat64 *buf, path_conv *pc)
+fhandler_base::fstat_by_name (struct __stat64 *buf)
 {
   int res;
   HANDLE handle;
   WIN32_FIND_DATA local;
 
-  if (!pc->exists ())
+  if (!pc.exists ())
     {
       debug_printf ("already determined that pc does not exist");
       set_errno (ENOENT);
       res = -1;
     }
-  else if ((handle = FindFirstFile (*pc, &local)) != INVALID_HANDLE_VALUE)
+  else if ((handle = FindFirstFile (pc, &local)) != INVALID_HANDLE_VALUE)
     {
       FindClose (handle);
-      res = fstat_helper (buf, pc,
+      res = fstat_helper (buf,
 			  local.ftCreationTime,
 			  local.ftLastAccessTime,
 			  local.ftLastWriteTime,
 			  local.nFileSizeHigh,
 			  local.nFileSizeLow);
     }
-  else if (pc->isdir ())
+  else if (pc.isdir ())
     {
       FILETIME ft = {};
-      res = fstat_helper (buf, pc, ft, ft, ft, 0, 0);
+      res = fstat_helper (buf, ft, ft, ft, 0, 0);
     }
   else
     {
-      debug_printf ("FindFirstFile failed for '%s', %E", (char *) *pc);
+      debug_printf ("FindFirstFile failed for '%s', %E", (char *) pc);
       __seterrno ();
       res = -1;
     }
@@ -165,63 +157,69 @@ fhandler_disk_file::fstat_by_name (struct __stat64 *buf, path_conv *pc)
 }
 
 int __stdcall
-fhandler_disk_file::fstat (struct __stat64 *buf, path_conv *pc)
+fhandler_base::fstat_fs (struct __stat64 *buf)
 {
   int res = -1;
   int oret;
-  int open_flags = O_RDONLY | O_BINARY | O_DIROPEN;
-  bool query_open_already;
+  int open_flags = O_RDONLY | O_BINARY;
 
   if (get_io_handle ())
     {
-      if (get_nohandle ())
-	return fstat_by_name (buf, pc);
+      if (nohandle ())
+	return fstat_by_name (buf);
       else
-	return fstat_by_handle (buf, pc);
+	return fstat_by_handle (buf);
     }
   /* If we don't care if the file is executable or we already know if it is,
      then just do a "query open" as it is apparently much faster. */
-  if (pc->exec_state () != dont_know_if_executable)
-    set_query_open (query_open_already = true);
-  else
-    query_open_already = false;
-
-  if (query_open_already && strncasematch (pc->volname (), "FAT", 3)
-      && !strpbrk (get_win32_name (), "?*|<>"))
-    oret = 0;
-  else if (!(oret = open (pc, open_flags, 0))
-	   && !query_open_already
-	   && get_errno () == EACCES)
+  if (pc.exec_state () != dont_know_if_executable)
     {
-      /* If we couldn't open the file, try a "query open" with no permissions.
-	 This will allow us to determine *some* things about the file, at least. */
-      pc->set_exec (0);
-      set_query_open (true);
-      oret = open (pc, open_flags, 0);
+      if (pc.fs_is_fat () && !strpbrk (get_win32_name (), "?*|<>"))
+	return fstat_by_name (buf);
+      query_open (query_stat_control);
+    }
+  if (!(oret = open_fs (open_flags, 0)) && get_errno () == EACCES)
+    {
+      /* If we couldn't open the file, try a query open with no permissions.
+	 This allows us to determine *some* things about the file, at least. */
+      pc.set_exec (0);
+      query_open (query_read_control);
+      oret = open_fs (open_flags, 0);
     }
 
-  if (!oret || get_nohandle ())
-    res = fstat_by_name (buf, pc);
-  else
+  if (oret)
     {
-      res = fstat_by_handle (buf, pc);
-      close ();
+      /* We now have a valid handle, regardless of the "nohandle" state.
+         Since fhandler_base::open only calls CloseHandle if !nohandle,
+	 we have to set it to false before calling close_fs and restore
+	 the state afterwards. */
+      res = fstat_by_handle (buf);
+      bool no_handle = nohandle ();
+      nohandle (false);
+      close_fs ();
+      nohandle (no_handle);
+      set_io_handle (NULL);
     }
+  else
+    res = fstat_by_name (buf);
 
   return res;
 }
 
 int __stdcall
-fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
-				  FILETIME ftCreationTime,
-				  FILETIME ftLastAccessTime,
-				  FILETIME ftLastWriteTime,
-				  DWORD nFileSizeHigh,
-				  DWORD nFileSizeLow,
-				  DWORD nFileIndexHigh,
-				  DWORD nFileIndexLow,
-				  DWORD nNumberOfLinks)
+fhandler_base::fstat_helper (struct __stat64 *buf,
+			     FILETIME ftCreationTime,
+			     FILETIME ftLastAccessTime,
+			     FILETIME ftLastWriteTime,
+			     DWORD nFileSizeHigh,
+			     DWORD nFileSizeLow,
+			     DWORD nFileIndexHigh,
+			     DWORD nFileIndexLow,
+			     DWORD nNumberOfLinks)
 {
+  IO_STATUS_BLOCK st;
+  FILE_COMPRESSION_INFORMATION fci;
+
   /* This is for FAT filesystems, which don't support atime/ctime */
   if (ftLastAccessTime.dwLowDateTime == 0
       && ftLastAccessTime.dwHighDateTime == 0)
@@ -233,21 +231,21 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
   to_timestruc_t (&ftLastAccessTime, &buf->st_atim);
   to_timestruc_t (&ftLastWriteTime, &buf->st_mtim);
   to_timestruc_t (&ftCreationTime, &buf->st_ctim);
-  buf->st_dev = pc->volser ();
-  buf->st_size = ((_off64_t)nFileSizeHigh << 32) + nFileSizeLow;
+  buf->st_dev = pc.volser ();
+  buf->st_size = ((_off64_t) nFileSizeHigh << 32) + nFileSizeLow;
   /* The number of links to a directory includes the
      number of subdirectories in the directory, since all
      those subdirectories point to it.
      This is too slow on remote drives, so we do without it.
      Setting the count to 2 confuses `find (1)' command. So
      let's try it with `1' as link count. */
-  buf->st_nlink = pc->ndisk_links (nNumberOfLinks);
+  buf->st_nlink = pc.ndisk_links (nNumberOfLinks);
 
   /* Assume that if a drive has ACL support it MAY have valid "inodes".
      It definitely does not have valid inodes if it does not have ACL
      support. */
-  switch (pc->has_acls () && (nFileIndexHigh || nFileIndexLow)
-	  ? pc->drive_type () : DRIVE_UNKNOWN)
+  switch (pc.has_acls () && (nFileIndexHigh || nFileIndexLow)
+	  ? pc.drive_type () : DRIVE_UNKNOWN)
     {
     case DRIVE_FIXED:
     case DRIVE_REMOVABLE:
@@ -267,16 +265,13 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
 
   buf->st_blksize = S_BLKSIZE;
 
-  /* GetCompressedFileSize() gets autoloaded.  It returns INVALID_FILE_SIZE
-     if it doesn't exist.  Since that's also a valid return value on 64bit
-     capable file systems, we must additionally check for the win32 error. */
-  nFileSizeLow = GetCompressedFileSizeA (pc->get_win32 (), &nFileSizeHigh);
-  if (nFileSizeLow != INVALID_FILE_SIZE || GetLastError () == NO_ERROR)
-    /* On systems supporting compressed (and sparsed) files,
-       GetCompressedFileSize() returns the actual amount of
-       bytes allocated on disk.  */
-    buf->st_blocks = (((_off64_t)nFileSizeHigh << 32)
-		     + nFileSizeLow + S_BLKSIZE - 1) / S_BLKSIZE;
+  /* On compressed and sparsed files, we request the actual amount of bytes
+     allocated on disk.  */
+  if (pc.has_attribute (FILE_ATTRIBUTE_COMPRESSED | FILE_ATTRIBUTE_SPARSE_FILE)
+      && get_io_handle ()
+      && !NtQueryInformationFile (get_io_handle (), &st, (PVOID) &fci,
+      				  sizeof fci, FileCompressionInformation))
+    buf->st_blocks = (fci.CompressedSize.QuadPart + S_BLKSIZE - 1) / S_BLKSIZE;
   else
     /* Just compute no. of blocks from file size. */
     buf->st_blocks  = (buf->st_size + S_BLKSIZE - 1) / S_BLKSIZE;
@@ -284,24 +279,25 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
   buf->st_mode = 0;
   /* Using a side effect: get_file_attibutes checks for
      directory. This is used, to set S_ISVTX, if needed.  */
-  if (pc->isdir ())
+  if (pc.isdir ())
     buf->st_mode = S_IFDIR;
-  else if (pc->issymlink ())
+  else if (pc.issymlink ())
     {
+      buf->st_size = pc.get_symlink_length ();
       /* symlinks are everything for everyone! */
       buf->st_mode = S_IFLNK | S_IRWXU | S_IRWXG | S_IRWXO;
-      get_file_attribute (pc->has_acls (), get_win32_name (), NULL,
-			  &buf->st_uid, &buf->st_gid);
+      get_file_attribute (pc.has_acls (), get_io_handle (), get_win32_name (),
+      			  NULL, &buf->st_uid, &buf->st_gid);
       goto done;
     }
-  else if (pc->issocket ())
+  else if (pc.issocket ())
     buf->st_mode = S_IFSOCK;
 
-  if (get_file_attribute (pc->has_acls (), get_win32_name (), &buf->st_mode,
-			  &buf->st_uid, &buf->st_gid) == 0)
+  if (!get_file_attribute (pc.has_acls (), get_io_handle (), get_win32_name (),
+  			   &buf->st_mode, &buf->st_uid, &buf->st_gid))
     {
       /* If read-only attribute is set, modify ntsec return value */
-      if (pc->has_attribute (FILE_ATTRIBUTE_READONLY) && !get_symlink_p ())
+      if (pc.has_attribute (FILE_ATTRIBUTE_READONLY) && !pc.issymlink ())
 	buf->st_mode &= ~(S_IWUSR | S_IWGRP | S_IWOTH);
 
       if (!(buf->st_mode & S_IFMT))
@@ -311,7 +307,7 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
     {
       buf->st_mode |= STD_RBITS;
 
-      if (!pc->has_attribute (FILE_ATTRIBUTE_READONLY))
+      if (!pc.has_attribute (FILE_ATTRIBUTE_READONLY))
 	buf->st_mode |= STD_WBITS;
       /* | S_IWGRP | S_IWOTH; we don't give write to group etc */
 
@@ -319,10 +315,15 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
 	buf->st_mode |= S_IFDIR | STD_XBITS;
       else if (buf->st_mode & S_IFMT)
 	/* nothing */;
+      else if (is_fs_special ())
+	{
+	  buf->st_dev = dev ();
+	  buf->st_mode = dev ().mode;
+	}
       else
 	{
 	  buf->st_mode |= S_IFREG;
-	  if (pc->exec_state () == dont_know_if_executable)
+	  if (pc.exec_state () == dont_know_if_executable)
 	    {
 	      DWORD cur, done;
 	      char magic[3];
@@ -339,8 +340,7 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
 		  if (ReadFile (get_handle (), magic, 3, &done, NULL)
 		      && has_exec_chars (magic, done))
 		    {
-		      set_execable_p ();
-		      pc->set_exec ();
+		      pc.set_exec ();
 		      buf->st_mode |= STD_XBITS;
 		    }
 		  (void) SetFilePointer (get_handle (), cur, NULL, FILE_BEGIN);
@@ -348,7 +348,7 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
 	    }
 	}
 
-      if (pc->exec_state () == is_executable)
+      if (pc.exec_state () == is_executable)
 	buf->st_mode |= STD_XBITS;
 
       /* This fakes the permissions of all files to match the current umask. */
@@ -362,30 +362,206 @@ fhandler_disk_file::fstat_helper (struct __stat64 *buf, path_conv *pc,
   return 0;
 }
 
-fhandler_disk_file::fhandler_disk_file (DWORD devtype) :
-  fhandler_base (devtype)
+int __stdcall
+fhandler_disk_file::fstat (struct __stat64 *buf)
 {
+  return fstat_fs (buf);
+}
+
+int __stdcall
+fhandler_disk_file::fchmod (mode_t mode)
+{
+  extern int chmod_device (path_conv& pc, mode_t mode);
+  int res = -1;
+  int oret = 0;
+
+  if (pc.is_fs_special ())
+    return chmod_device (pc, mode);
+
+  if (wincap.has_security ())
+    {
+      enable_restore_privilege ();
+      if (!get_io_handle ())
+	{
+	  query_open (query_write_control);
+	  if (!(oret = open_fs (O_BINARY, 0)))
+	    return -1;
+	}
+
+      if (!allow_ntsec && allow_ntea) /* Not necessary when manipulating SD. */
+	SetFileAttributes (pc, (DWORD) pc & ~FILE_ATTRIBUTE_READONLY);
+      if (pc.isdir ())
+	mode |= S_IFDIR;
+      if (!set_file_attribute (pc.has_acls (), get_io_handle (), pc,
+			       ILLEGAL_UID, ILLEGAL_GID, mode)
+	  && allow_ntsec)
+	res = 0;
+      
+      if (oret)
+	close_fs ();
+    }
+
+  /* if the mode we want has any write bits set, we can't be read only. */
+  if (mode & (S_IWUSR | S_IWGRP | S_IWOTH))
+    (DWORD) pc &= ~FILE_ATTRIBUTE_READONLY;
+  else
+    (DWORD) pc |= FILE_ATTRIBUTE_READONLY;
+
+  if (!SetFileAttributes (pc, pc))
+    __seterrno ();
+  else if (!allow_ntsec)
+    /* Correct NTFS security attributes have higher priority */
+    res = 0;
+
+  return res;
+}
+
+int __stdcall
+fhandler_disk_file::fchown (__uid32_t uid, __gid32_t gid)
+{
+  int oret = 0;
+
+  if (!pc.has_acls () || !allow_ntsec)
+    {
+      /* fake - if not supported, pretend we're like win95
+         where it just works */
+      return 0;
+    }
+
+  enable_restore_privilege ();
+  if (!get_io_handle ())
+    {
+      query_open (query_write_control);
+      if (!(oret = open_fs (O_BINARY, 0)))
+	return -1;
+    }
+
+  mode_t attrib = 0;
+  if (pc.isdir ())
+    attrib |= S_IFDIR;
+  int res = get_file_attribute (pc.has_acls (), get_io_handle (), pc, &attrib);
+  if (!res)
+    res = set_file_attribute (pc.has_acls (), get_io_handle (), pc,
+			      uid, gid, attrib);
+
+  if (oret)
+    close_fs ();
+
+  return res;
+}
+
+int _stdcall
+fhandler_disk_file::facl (int cmd, int nentries, __aclent32_t *aclbufp)
+{
+  int res = -1;
+  int oret = 0;
+
+  if (!pc.has_acls () || !allow_ntsec)
+    {
+      switch (cmd)
+        {
+	  struct __stat64 st;
+
+	  case SETACL:
+	    set_errno (ENOSYS);
+	    break;
+	  case GETACL:
+	    if (!aclbufp)
+	      set_errno(EFAULT);
+	    else if (nentries < MIN_ACL_ENTRIES)
+	      set_errno (ENOSPC);
+	    else
+	      {
+		if (!get_io_handle ())
+		  {
+		    query_open (query_read_control);
+		    if (!(oret = open_fs (O_BINARY, 0)))
+		      return -1;
+		  }
+	        if (!fstat_by_handle (&st))
+		  {
+		    aclbufp[0].a_type = USER_OBJ;
+		    aclbufp[0].a_id = st.st_uid;
+		    aclbufp[0].a_perm = (st.st_mode & S_IRWXU) >> 6;
+		    aclbufp[1].a_type = GROUP_OBJ;
+		    aclbufp[1].a_id = st.st_gid;
+		    aclbufp[1].a_perm = (st.st_mode & S_IRWXG) >> 3;
+		    aclbufp[2].a_type = OTHER_OBJ;
+		    aclbufp[2].a_id = ILLEGAL_GID;
+		    aclbufp[2].a_perm = st.st_mode & S_IRWXO;
+		    aclbufp[3].a_type = CLASS_OBJ; 
+		    aclbufp[3].a_id = ILLEGAL_GID;
+		    aclbufp[3].a_perm = S_IRWXU | S_IRWXG | S_IRWXO;
+		    res = MIN_ACL_ENTRIES;
+		  }
+	      }
+	    break;
+	  case GETACLCNT:
+	    res = MIN_ACL_ENTRIES;
+	    break;
+	  default:
+	    set_errno (EINVAL);
+	    break;
+	}
+    }
+  else
+    {
+      if (cmd == SETACL)
+	enable_restore_privilege ();
+      if (!get_io_handle ())
+	{
+	  query_open (cmd == SETACL ? query_write_control : query_read_control);
+	  if (!(oret = open_fs (O_BINARY, 0)))
+	    return -1;
+	}
+      switch (cmd)
+	{
+	  case SETACL:
+	    if (!aclsort32 (nentries, 0, aclbufp))
+	      res = setacl (get_io_handle (), pc, nentries, aclbufp);
+	    break;
+	  case GETACL:
+	    if (!aclbufp)
+	      set_errno(EFAULT);
+	    else
+	      res = getacl (get_io_handle (), pc, pc, nentries, aclbufp);
+	    break;
+	  case GETACLCNT:
+	    res = getacl (get_io_handle (), pc, pc, 0, NULL);
+	  default:
+	    set_errno (EINVAL);
+	    break;
+	}
+    }
+
+  if (oret)
+    close_fs ();
+
+  return res;
 }
 
 fhandler_disk_file::fhandler_disk_file () :
-  fhandler_base (FH_DISK)
+  fhandler_base ()
 {
 }
 
 int
-fhandler_disk_file::open (path_conv *real_path, int flags, mode_t mode)
+fhandler_disk_file::open (int flags, mode_t mode)
 {
-  if (real_path->case_clash && flags & O_CREAT)
+  return open_fs (flags, mode);
+}
+
+int
+fhandler_base::open_fs (int flags, mode_t mode)
+{
+  if (pc.case_clash && flags & O_CREAT)
     {
       debug_printf ("case clash detected");
       set_errno (ECASECLASH);
       return 0;
     }
 
-  set_has_acls (real_path->has_acls ());
-  set_isremote (real_path->isremote ());
-
-  int res = fhandler_base::open (real_path, flags | O_DIROPEN, mode);
+  int res = fhandler_base::open (flags | O_DIROPEN, mode);
   if (!res)
     goto out;
 
@@ -395,10 +571,10 @@ fhandler_disk_file::open (path_conv *real_path, int flags, mode_t mode)
      The only known file system to date is the SUN NFS Solstice Client 3.1
      which returns a valid handle when trying to open a file in a nonexistent
      directory. */
-  if (real_path->has_buggy_open () && !real_path->exists ())
+  if (pc.has_buggy_open () && !pc.exists ())
     {
       debug_printf ("Buggy open detected.");
-      close ();
+      close_fs ();
       set_errno (ENOENT);
       return 0;
     }
@@ -409,12 +585,9 @@ fhandler_disk_file::open (path_conv *real_path, int flags, mode_t mode)
   if (flags & O_CREAT
       && GetLastError () != ERROR_ALREADY_EXISTS
       && !allow_ntsec && allow_ntea)
-    set_file_attribute (has_acls (), get_win32_name (), mode);
+    set_file_attribute (false, NULL, get_win32_name (), mode);
 
-  set_fs_flags (real_path->fs_flags ());
-  set_symlink_p (real_path->issymlink ());
-  set_execable_p (real_path->exec_state ());
-  set_socket_p (real_path->issocket ());
+  set_fs_flags (pc.fs_flags ());
 
 out:
   syscall_printf ("%d = fhandler_disk_file::open (%s, %p)", res,
@@ -424,33 +597,33 @@ out:
 
 int
 fhandler_disk_file::close ()
+{
+  return close_fs ();
+}
+
+int
+fhandler_base::close_fs ()
 {
   int res = fhandler_base::close ();
   if (!res)
-    cygwin_shared->delqueue.process_queue ();
+    user_shared->delqueue.process_queue ();
   return res;
 }
 
-/*
- * FIXME !!!
- * The correct way to do this to get POSIX locking
- * semantics is to keep a linked list of posix lock
- * requests and map them into Win32 locks. The problem
- * is that Win32 does not deal correctly with overlapping
- * lock requests. Also another pain is that Win95 doesn't do
- * non-blocking or non exclusive locks at all. For '95 just
- * convert all lock requests into blocking,exclusive locks.
- * This shouldn't break many apps but denying all locking
- * would.
- * For now just convert to Win32 locks and hope for the best.
- */
+/* FIXME: The correct way to do this to get POSIX locking semantics is to
+   keep a linked list of posix lock requests and map them into Win32 locks.
+   he problem is that Win32 does not deal correctly with overlapping lock
+   requests. Also another pain is that Win95 doesn't do non-blocking or
+   non-exclusive locks at all. For '95 just convert all lock requests into
+   blocking,exclusive locks.  This shouldn't break many apps but denying all
+   locking would.  For now just convert to Win32 locks and hope for
+   the best.  */
 
 int
-fhandler_disk_file::lock (int cmd, struct flock *fl)
+fhandler_disk_file::lock (int cmd, struct __flock64 *fl)
 {
   _off64_t win32_start;
-  int win32_len;
-  DWORD win32_upper;
+  _off64_t win32_len;
   _off64_t startpos;
 
   /*
@@ -521,17 +694,25 @@ fhandler_disk_file::lock (int cmd, struct flock *fl)
       win32_start = 0;
     }
 
-  /*
-   * Special case if len == 0 for POSIX means lock
-   * to the end of the entire file (and all future extensions).
-   */
+  DWORD off_high, off_low, len_high, len_low;
+
+  off_low = (DWORD)(win32_start & UINT32_MAX);
+  off_high = (DWORD)(win32_start >> 32);
   if (win32_len == 0)
     {
-      win32_len = 0xffffffff;
-      win32_upper = wincap.lock_file_highword ();
+      /* Special case if len == 0 for POSIX means lock to the end of
+         the entire file (and all future extensions).  */
+      /* CV, 2003-12-03: And yet another Win 9x bugginess.  For some reason
+	 offset + length must be <= 0x100000000.  I'm using 0xffffffff as
+	 upper border here, this should be sufficient. */
+      len_low = UINT32_MAX - (wincap.lock_file_highword () ? 0 : off_low);
+      len_high = wincap.lock_file_highword ();
     }
   else
-    win32_upper = 0;
+    {
+      len_low = (DWORD)(win32_len & UINT32_MAX);
+      len_high = (DWORD)(win32_len >> 32);
+    }
 
   BOOL res;
 
@@ -544,18 +725,18 @@ fhandler_disk_file::lock (int cmd, struct flock *fl)
 
       ov.Internal = 0;
       ov.InternalHigh = 0;
-      ov.Offset = (DWORD)win32_start;
-      ov.OffsetHigh = 0;
+      ov.Offset = off_low;
+      ov.OffsetHigh = off_high;
       ov.hEvent = (HANDLE) 0;
 
       if (fl->l_type == F_UNLCK)
 	{
-	  res = UnlockFileEx (get_handle (), 0, (DWORD)win32_len, win32_upper, &ov);
+	  res = UnlockFileEx (get_handle (), 0, len_low, len_high, &ov);
 	}
       else
 	{
-	  res = LockFileEx (get_handle (), lock_flags, 0, (DWORD)win32_len,
-							win32_upper, &ov);
+	  res = LockFileEx (get_handle (), lock_flags, 0,
+			    len_low, len_high, &ov);
 	  /* Deal with the fail immediately case. */
 	  /*
 	   * FIXME !! I think this is the right error to check for
@@ -573,10 +754,9 @@ fhandler_disk_file::lock (int cmd, struct flock *fl)
     {
       /* Windows 95 -- use primitive lock call */
       if (fl->l_type == F_UNLCK)
-	res = UnlockFile (get_handle (), (DWORD)win32_start, 0, (DWORD)win32_len,
-							win32_upper);
+	res = UnlockFile (get_handle (), off_low, off_high, len_low, len_high);
       else
-	res = LockFile (get_handle (), (DWORD)win32_start, 0, (DWORD)win32_len, win32_upper);
+	res = LockFile (get_handle (), off_low, off_high, len_low, len_high);
     }
 
   if (res == 0)
@@ -589,60 +769,69 @@ fhandler_disk_file::lock (int cmd, struct flock *fl)
 }
 
 DIR *
-fhandler_disk_file::opendir (path_conv& real_name)
+fhandler_disk_file::opendir ()
 {
   DIR *dir;
   DIR *res = NULL;
   size_t len;
 
-  if (!real_name.isdir ())
+  if (!pc.isdir ())
     set_errno (ENOTDIR);
-  else if ((len = strlen (real_name))> MAX_PATH - 3)
+  else if ((len = strlen (pc)) > CYG_MAX_PATH - 3)
     set_errno (ENAMETOOLONG);
   else if ((dir = (DIR *) malloc (sizeof (DIR))) == NULL)
     set_errno (ENOMEM);
   else if ((dir->__d_dirname = (char *) malloc (len + 3)) == NULL)
     {
-      free (dir);
       set_errno (ENOMEM);
+      goto free_dir;
     }
   else if ((dir->__d_dirent =
 	    (struct dirent *) malloc (sizeof (struct dirent))) == NULL)
     {
-      free (dir->__d_dirname);
-      free (dir);
       set_errno (ENOMEM);
+      goto free_dirname;
     }
+  else if (fhaccess (R_OK) != 0)
+    goto free_dirent;
   else
     {
-      strcpy (dir->__d_dirname, real_name.get_win32 ());
+      strcpy (dir->__d_dirname, get_win32_name ());
       dir->__d_dirent->d_version = __DIRENT_VERSION;
       cygheap_fdnew fd;
-      if (fd >= 0)
-	{
-	  fd = this;
-	  fd->set_nohandle (true);
-	  dir->__d_dirent->d_fd = fd;
-	  dir->__fh = this;
-	  /* FindFirstFile doesn't seem to like duplicate /'s. */
-	  len = strlen (dir->__d_dirname);
-	  if (len == 0 || isdirsep (dir->__d_dirname[len - 1]))
-	    strcat (dir->__d_dirname, "*");
-	  else
-	    strcat (dir->__d_dirname, "\\*");  /**/
-	  dir->__d_cookie = __DIRENT_COOKIE;
-	  dir->__handle = INVALID_HANDLE_VALUE;
-	  dir->__d_position = 0;
-	  dir->__d_dirhash = get_namehash ();
 
-	  res = dir;
-	}
-      if (real_name.isencoded ())
-	set_encoded ();
+      if (fd < 0)
+	goto free_dirent;
+
+      fd = this;
+      fd->nohandle (true);
+      dir->__d_dirent->d_fd = fd;
+      dir->__fh = this;
+      /* FindFirstFile doesn't seem to like duplicate /'s. */
+      len = strlen (dir->__d_dirname);
+      if (len == 0 || isdirsep (dir->__d_dirname[len - 1]))
+	strcat (dir->__d_dirname, "*");
+      else
+	strcat (dir->__d_dirname, "\\*");  /**/
+      dir->__d_cookie = __DIRENT_COOKIE;
+      dir->__handle = INVALID_HANDLE_VALUE;
+      dir->__d_position = 0;
+      dir->__d_dirhash = get_namehash ();
+
+      res = dir;
+
     }
 
   syscall_printf ("%p = opendir (%s)", res, get_name ());
   return res;
+
+free_dirent:
+  free (dir->__d_dirent);
+free_dirname:
+  free (dir->__d_dirname);
+free_dir:
+  free (dir);
+  return res;
 }
 
 struct dirent *
@@ -679,29 +868,29 @@ fhandler_disk_file::readdir (DIR *dir)
       return res;
     }
 
-  /* We get here if `buf' contains valid data.  */
-  if (get_encoded ())
-    (void) fnunmunge (dir->__d_dirent->d_name, buf.cFileName);
-  else
-    strcpy (dir->__d_dirent->d_name, buf.cFileName);
-
   /* Check for Windows shortcut. If it's a Cygwin or U/WIN
      symlink, drop the .lnk suffix. */
   if (buf.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
     {
-      char *c = dir->__d_dirent->d_name;
+      char *c = buf.cFileName;
       int len = strlen (c);
       if (strcasematch (c + len - 4, ".lnk"))
 	{
-	  char fbuf[MAX_PATH + 1];
+	  char fbuf[CYG_MAX_PATH + 1];
 	  strcpy (fbuf, dir->__d_dirname);
-	  strcpy (fbuf + strlen (fbuf) - 1, dir->__d_dirent->d_name);
+	  strcpy (fbuf + strlen (fbuf) - 1, c);
 	  path_conv fpath (fbuf, PC_SYM_NOFOLLOW);
-	  if (fpath.issymlink ())
+	  if (fpath.issymlink () || fpath.isspecial ())
 	    c[len - 4] = '\0';
 	}
     }
 
+  /* We get here if `buf' contains valid data.  */
+  if (pc.isencoded ())
+    (void) fnunmunge (dir->__d_dirent->d_name, buf.cFileName);
+  else
+    strcpy (dir->__d_dirent->d_name, buf.cFileName);
+
   dir->__d_position++;
   res = dir->__d_dirent;
   syscall_printf ("%p = readdir (%p) (%s)",
@@ -749,8 +938,8 @@ fhandler_disk_file::closedir (DIR *dir)
   return 0;
 }
 
-fhandler_cygdrive::fhandler_cygdrive (int unit) :
-  fhandler_disk_file (FH_CYGDRIVE), unit (unit), ndrives (0), pdrive (NULL)
+fhandler_cygdrive::fhandler_cygdrive () :
+  fhandler_disk_file (), ndrives (0), pdrive (NULL)
 {
 }
 
@@ -759,17 +948,16 @@ void
 fhandler_cygdrive::set_drives ()
 {
   const int len = 2 + 26 * DRVSZ;
-  char *p = (char *) crealloc ((void *) win32_path_name, len);
-
-  win32_path_name = pdrive = p;
+  char *p = const_cast<char *> (get_win32_name ());
+  pdrive = p;
   ndrives = GetLogicalDriveStrings (len, p) / DRVSZ;
 }
 
 int
-fhandler_cygdrive::fstat (struct __stat64 *buf, path_conv *pc)
+fhandler_cygdrive::fstat (struct __stat64 *buf)
 {
   if (!iscygdrive_root ())
-    return fhandler_disk_file::fstat (buf, pc);
+    return fhandler_disk_file::fstat (buf);
   buf->st_mode = S_IFDIR | 0555;
   if (!ndrives)
     set_drives ();
@@ -778,11 +966,11 @@ fhandler_cygdrive::fstat (struct __stat64 *buf, path_conv *pc)
 }
 
 DIR *
-fhandler_cygdrive::opendir (path_conv& real_name)
+fhandler_cygdrive::opendir ()
 {
   DIR *dir;
 
-  dir = fhandler_disk_file::opendir (real_name);
+  dir = fhandler_disk_file::opendir ();
   if (dir && iscygdrive_root () && !ndrives)
     set_drives ();
 
@@ -823,7 +1011,7 @@ fhandler_cygdrive::seekdir (DIR *dir, _off64_t loc)
   if (!iscygdrive_root ())
     return fhandler_disk_file::seekdir (dir, loc);
 
-  for (pdrive = win32_path_name, dir->__d_position = -1; *pdrive;
+  for (pdrive = get_win32_name (), dir->__d_position = -1; *pdrive;
        pdrive = strchr (pdrive, '\0') + 1)
     if (++dir->__d_position >= loc)
       break;
@@ -836,7 +1024,7 @@ fhandler_cygdrive::rewinddir (DIR *dir)
 {
   if (!iscygdrive_root ())
     return fhandler_disk_file::rewinddir (dir);
-  pdrive = win32_path_name;
+  pdrive = get_win32_name ();
   dir->__d_position = 0;
   return;
 }
@@ -846,6 +1034,6 @@ fhandler_cygdrive::closedir (DIR *dir)
 {
   if (!iscygdrive_root ())
     return fhandler_disk_file::closedir (dir);
-  pdrive = win32_path_name;
+  pdrive = get_win32_name ();
   return -1;
 }
