@@ -1,6 +1,6 @@
 /* fhandler_proc.cc: fhandler for /proc virtual filesystem
 
-   Copyright 2002, 2003 Red Hat, Inc.
+   Copyright 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -17,8 +17,8 @@ details. */
 #include <ntdef.h>
 #include "cygerrno.h"
 #include "security.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "pinfo.h"
 #include "dtable.h"
 #include "cygheap.h"
@@ -146,23 +146,18 @@ fhandler_proc::exists ()
 }
 
 fhandler_proc::fhandler_proc ():
-  fhandler_virtual (FH_PROC)
-{
-}
-
-fhandler_proc::fhandler_proc (DWORD devtype):
-  fhandler_virtual (devtype)
+  fhandler_virtual ()
 {
 }
 
 int
-fhandler_proc::fstat (struct __stat64 *buf, path_conv *pc)
+fhandler_proc::fstat (struct __stat64 *buf)
 {
   const char *path = get_name ();
   debug_printf ("fstat (%s)", path);
 
   path += proc_len;
-  (void) fhandler_base::fstat (buf, pc);
+  (void) fhandler_base::fstat (buf);
 
   buf->st_mode &= ~_IFMT & NO_W;
 
@@ -216,15 +211,15 @@ fhandler_proc::readdir (DIR * dir)
 }
 
 int
-fhandler_proc::open (path_conv *pc, int flags, mode_t mode)
+fhandler_proc::open (int flags, mode_t mode)
 {
   int proc_file_no = -1;
 
-  int res = fhandler_virtual::open (pc, flags, mode);
+  int res = fhandler_virtual::open (flags, mode);
   if (!res)
     goto out;
 
-  set_nohandle (true);
+  nohandle (true);
 
   const char *path;
 
@@ -395,8 +390,42 @@ format_proc_meminfo (char *destbuf, size_t maxsize)
   GlobalMemoryStatus (&memory_status);
   mem_total = memory_status.dwTotalPhys;
   mem_free = memory_status.dwAvailPhys;
-  swap_total = memory_status.dwTotalPageFile - mem_total;
-  swap_free = memory_status.dwAvailPageFile - mem_total;
+  PSYSTEM_PAGEFILE_INFORMATION spi = NULL;
+  ULONG size = 512;
+  NTSTATUS ret;
+  spi = (PSYSTEM_PAGEFILE_INFORMATION) malloc (size);
+  if (spi)
+    {
+      ret = NtQuerySystemInformation (SystemPagefileInformation, (PVOID) spi,
+				      size, &size);
+      if (ret == STATUS_INFO_LENGTH_MISMATCH)
+        {
+	  free (spi);
+	  spi = (PSYSTEM_PAGEFILE_INFORMATION) malloc (size);
+	  if (spi)
+	    ret = NtQuerySystemInformation (SystemPagefileInformation,
+					    (PVOID) spi, size, &size);
+	}
+    }
+  if (!spi || ret || (!ret && GetLastError () == ERROR_PROC_NOT_FOUND))
+    {
+      swap_total = memory_status.dwTotalPageFile - mem_total;
+      swap_free = memory_status.dwAvailPageFile - mem_total;
+    }
+  else
+    {
+      PSYSTEM_PAGEFILE_INFORMATION spp = spi;
+      do
+        {
+	  swap_total += spp->CurrentSize * getpagesize ();
+	  swap_free += (spp->CurrentSize - spp->TotalUsed) * getpagesize ();
+	}
+      while (spp->NextEntryOffset
+	     && (spp = (PSYSTEM_PAGEFILE_INFORMATION)
+			   ((char *) spp + spp->NextEntryOffset)));
+    }
+  if (spi)
+    free (spi);
   return __small_sprintf (destbuf, "         total:      used:      free:\n"
 				   "Mem:  %10lu %10lu %10lu\n"
 				   "Swap: %10lu %10lu %10lu\n"
@@ -867,7 +896,7 @@ format_proc_partitions (char *destbuf, size_t maxsize)
     {
       for (int drive_number=0;;drive_number++)
 	{
-	  CHAR szDriveName[MAX_PATH];
+	  CHAR szDriveName[CYG_MAX_PATH];
 	  __small_sprintf (szDriveName, "\\\\.\\PHYSICALDRIVE%d", drive_number);
 	  HANDLE hDevice;
 	  hDevice = CreateFile (szDriveName,
@@ -903,12 +932,16 @@ format_proc_partitions (char *destbuf, size_t maxsize)
 		debug_printf ("DeviceIoControl %E");
 	      else
 		{
-		  bufptr += __small_sprintf (bufptr, "%5d %5d %9U sd%c\n",
-					     FH_FLOPPY,
-					     drive_number * 16 + 32,
+		  char devname[16];
+		  __small_sprintf (devname, "/dev/sd%c", drive_number + 'a');
+		  device dev;
+		  dev.parse (devname);
+		  bufptr += __small_sprintf (bufptr, "%5d %5d %9U %s\n",
+					     dev.major,
+					     dev.minor,
 					     (long long)((dg.Cylinders.QuadPart * dg.TracksPerCylinder *
-					      dg.SectorsPerTrack * dg.BytesPerSector) >> 6),
-					     drive_number + 'a');
+					      dg.SectorsPerTrack * dg.BytesPerSector) >> 10),
+					     devname + 5);
 		}
 	      while (dwRetCode = DeviceIoControl (hDevice,
 						  IOCTL_DISK_GET_DRIVE_LAYOUT,
@@ -929,12 +962,16 @@ format_proc_partitions (char *destbuf, size_t maxsize)
 		    {
 		      if (dli->PartitionEntry[partition].PartitionLength.QuadPart == 0)
 			continue;
-		      bufptr += __small_sprintf (bufptr, "%5d %5d %9U sd%c%d\n",
-						 FH_FLOPPY,
-						 drive_number * 16 + partition + 33,
-						 (long long)(dli->PartitionEntry[partition].PartitionLength.QuadPart >> 6),
-						 drive_number + 'a',
-						 partition + 1);
+		      char devname[16];
+		      __small_sprintf (devname, "/dev/sd%c%d",
+		      				drive_number + 'a',
+						partition + 1);
+		      device dev;
+		      dev.parse (devname);
+		      bufptr += __small_sprintf (bufptr, "%5d %5d %9U %s\n",
+						 dev.major, dev.minor,
+						 (long long)(dli->PartitionEntry[partition].PartitionLength.QuadPart >> 10),
+						 devname + 5);
 		    }
 		}
 
