@@ -1,6 +1,6 @@
 /* fhandler_process.cc: fhandler for /proc/<pid> virtual filesystem
 
-   Copyright 2002, 2003 Red Hat, Inc.
+   Copyright 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -15,9 +15,9 @@ details. */
 #include <ntdef.h>
 #include "cygerrno.h"
 #include "security.h"
+#include "path.h"
 #include "fhandler.h"
 #include "pinfo.h"
-#include "path.h"
 #include "shared_info.h"
 #include "dtable.h"
 #include "cygheap.h"
@@ -96,16 +96,16 @@ fhandler_process::exists ()
 }
 
 fhandler_process::fhandler_process ():
-  fhandler_proc (FH_PROCESS)
+  fhandler_proc ()
 {
 }
 
 int
-fhandler_process::fstat (struct __stat64 *buf, path_conv *pc)
+fhandler_process::fstat (struct __stat64 *buf)
 {
   const char *path = get_name ();
   int file_type = exists ();
-  (void) fhandler_base::fstat (buf, pc);
+  (void) fhandler_base::fstat (buf);
   path += proc_len + 1;
   pid = atoi (path);
   pinfo p (pid);
@@ -155,15 +155,15 @@ fhandler_process::readdir (DIR * dir)
 }
 
 int
-fhandler_process::open (path_conv *pc, int flags, mode_t mode)
+fhandler_process::open (int flags, mode_t mode)
 {
   int process_file_no = -1;
 
-  int res = fhandler_virtual::open (pc, flags, mode);
+  int res = fhandler_virtual::open (flags, mode);
   if (!res)
     goto out;
 
-  set_nohandle (true);
+  nohandle (true);
 
   const char *path;
   path = get_name () + proc_len + 1;
@@ -300,12 +300,15 @@ fhandler_process::fill_filebuf ()
 	filebuf = p->cmdline (fs);
 	filesize = fs;
 	if (!filebuf || !*filebuf)
-	  filebuf = strdup ("<defunct>");
+	  {
+	    filebuf = strdup ("<defunct>");
+	    filesize = strlen (filebuf) + 1;
+	  }
 	break;
       }
     case PROCESS_EXENAME:
       {
-	filebuf = (char *) realloc (filebuf, bufalloc = MAX_PATH);
+	filebuf = (char *) realloc (filebuf, bufalloc = CYG_MAX_PATH);
 	if (p->process_state & (PID_ZOMBIE | PID_EXITED))
 	  strcpy (filebuf, "<defunct>");
 	else
@@ -364,7 +367,7 @@ fhandler_process::fill_filebuf ()
 static _off64_t
 format_process_stat (_pinfo *p, char *destbuf, size_t maxsize)
 {
-  char cmd[MAX_PATH];
+  char cmd[CYG_MAX_PATH];
   int state = 'R';
   unsigned long fault_count = 0UL,
 		utime = 0UL, stime = 0UL,
@@ -501,7 +504,7 @@ format_process_stat (_pinfo *p, char *destbuf, size_t maxsize)
 static _off64_t
 format_process_status (_pinfo *p, char *destbuf, size_t maxsize)
 {
-  char cmd[MAX_PATH];
+  char cmd[CYG_MAX_PATH];
   int state = 'R';
   const char *state_str = "unknown";
   unsigned long vmsize = 0UL, vmrss = 0UL, vmdata = 0UL, vmlib = 0UL, vmtext = 0UL,
