@@ -1,6 +1,6 @@
 /* fhandler_raw.cc.  See fhandler.h for a description of the fhandler classes.
 
-   Copyright 1999, 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
    This file is part of Cygwin.
 
@@ -18,72 +18,56 @@
 #include "cygerrno.h"
 #include "perprocess.h"
 #include "security.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygheap.h"
 #include "ntdll.h"
 
-/* static wrapper functions to hide the effect of media changes and
-   bus resets which occurs after a new media is inserted. This is
-   also related to the used tape device.  */
-
-static BOOL write_file (HANDLE fh, const void *buf, DWORD to_write,
-			DWORD *written, int *err)
-{
-  BOOL ret;
-
-  *err = 0;
-  if (!(ret = WriteFile (fh, buf, to_write, written, 0)))
-    {
-      if ((*err = GetLastError ()) == ERROR_MEDIA_CHANGED
-	  || *err == ERROR_BUS_RESET)
-	{
-	  *err = 0;
-	  if (!(ret = WriteFile (fh, buf, to_write, written, 0)))
-	    *err = GetLastError ();
-	}
-    }
-  syscall_printf ("%d (err %d) = WriteFile (%d, %d, write %d, written %d, 0)",
-		  ret, *err, fh, buf, to_write, *written);
-  return ret;
-}
-
-static BOOL read_file (HANDLE fh, void *buf, DWORD to_read,
-		       DWORD *read, int *err)
-{
-  BOOL ret;
-
-  *err = 0;
-  if (!(ret = ReadFile (fh, buf, to_read, read, 0)))
-    {
-      if ((*err = GetLastError ()) == ERROR_MEDIA_CHANGED
-	  || *err == ERROR_BUS_RESET)
-	{
-	  *err = 0;
-	  if (!(ret = ReadFile (fh, buf, to_read, read, 0)))
-	    *err = GetLastError ();
-	}
-    }
-  syscall_printf ("%d (err %d) = ReadFile (%d, %d, to_read %d, read %d, 0)",
-		  ret, *err, fh, buf, to_read, *read);
-  return ret;
-}
-
 /**********************************************************************/
 /* fhandler_dev_raw */
 
-void
-fhandler_dev_raw::clear (void)
+int
+fhandler_dev_raw::is_eom (int win_error)
 {
-  devbuf = NULL;
-  devbufsiz = 0;
-  devbufstart = 0;
-  devbufend = 0;
-  eom_detected = 0;
-  eof_detected = 0;
-  lastblk_to_read = 0;
-  varblkop = 0;
+  return 0;
+}
+
+int
+fhandler_dev_raw::is_eof (int)
+{
+  return 0;
+}
+
+
+/* Wrapper functions to allow fhandler_dev_tape to detect and care for
+   media changes and bus resets. */
+
+BOOL
+fhandler_dev_raw::write_file (const void *buf, DWORD to_write,
+			      DWORD *written, int *err)
+{
+  BOOL ret;
+
+  *err = 0;
+  if (!(ret = WriteFile (get_handle (), buf, to_write, written, 0)))
+    *err = GetLastError ();
+  syscall_printf ("%d (err %d) = WriteFile (%d, %d, write %d, written %d, 0)",
+		  ret, *err, get_handle (), buf, to_write, *written);
+  return ret;
+}
+
+BOOL
+fhandler_dev_raw::read_file (void *buf, DWORD to_read, DWORD *read, int *err)
+{
+  BOOL ret;
+
+  *err = 0;
+  if (!(ret = ReadFile (get_handle (), buf, to_read, read, 0)))
+    *err = GetLastError ();
+  syscall_printf ("%d (err %d) = ReadFile (%d, %d, to_read %d, read %d, 0)",
+		  ret, *err, get_handle (), buf, to_read, *read);
+  return ret;
 }
 
 int
@@ -92,55 +76,65 @@ fhandler_dev_raw::writebuf (void)
   DWORD written;
   int ret = 0;
 
-  if (is_writing && devbuf && devbufend)
+  if (is_writing () && devbuf && devbufend)
     {
       DWORD to_write;
       int ret = 0;
 
       memset (devbuf + devbufend, 0, devbufsiz - devbufend);
-      if (get_device () != FH_TAPE)
-	to_write = ((devbufend - 1) / 512 + 1) * 512;
-      else if (varblkop)
-	to_write = devbufend;
-      else
-	to_write = devbufsiz;
-      if (!write_file (get_handle (), devbuf, to_write, &written, &ret)
+      to_write = ((devbufend - 1) / 512 + 1) * 512;
+      if (!write_file (devbuf, to_write, &written, &ret)
 	  && is_eom (ret))
-	eom_detected = 1;
-      if (written)
-	has_written = 1;
+	eom_detected (true);
       devbufstart = devbufend = 0;
     }
-  is_writing = 0;
+  is_writing (false);
   return ret;
 }
 
-fhandler_dev_raw::fhandler_dev_raw (DWORD devtype, int nunit)
-  : fhandler_base (devtype), unit (nunit)
+fhandler_dev_raw::fhandler_dev_raw ()
+  : fhandler_base (), status ()
 {
-  clear ();
-  set_need_fork_fixup ();
+  need_fork_fixup (true);
 }
 
 fhandler_dev_raw::~fhandler_dev_raw (void)
 {
   if (devbufsiz > 1L)
     delete [] devbuf;
-  clear ();
+}
+
+int __stdcall
+fhandler_dev_raw::fstat (struct __stat64 *buf)
+{
+  debug_printf ("here");
+
+  if (get_major () == DEV_TAPE_MAJOR)
+    buf->st_mode = S_IFCHR | STD_RBITS | STD_WBITS | S_IWGRP | S_IWOTH;
+  else
+    buf->st_mode = S_IFBLK | STD_RBITS | STD_WBITS | S_IWGRP | S_IWOTH;
+
+  buf->st_uid = geteuid32 ();
+  buf->st_gid = getegid32 ();
+  buf->st_nlink = 1;
+  buf->st_blksize = S_BLKSIZE;
+  time_as_timestruc_t (&buf->st_ctim);
+  buf->st_atim = buf->st_mtim = buf->st_ctim;
+  return 0;
 }
 
 int
-fhandler_dev_raw::open (path_conv *real_path, int flags, mode_t)
+fhandler_dev_raw::open (int flags, mode_t)
 {
   if (!wincap.has_raw_devices ())
     {
       set_errno (ENOENT);
-      debug_printf ("%s is accessible under NT/W2K only",real_path->get_win32());
+      debug_printf ("%s is accessible under NT/W2K only", get_win32_name ());
       return 0;
     }
 
   /* Check for illegal flags. */
-  if (flags & (O_APPEND | O_EXCL))
+  if (get_major () != DEV_TAPE_MAJOR && (flags & (O_APPEND | O_EXCL)))
     {
       set_errno (EINVAL);
       return 0;
@@ -150,36 +144,15 @@ fhandler_dev_raw::open (path_conv *real_path, int flags, mode_t)
   flags &= ~(O_CREAT | O_TRUNC);
   flags |= O_BINARY;
 
-  DWORD access = GENERIC_READ | SYNCHRONIZE;
-  if (get_device () == FH_TAPE
-      || (flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_WRONLY
-      || (flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_RDWR)
-    access |= GENERIC_WRITE;
+  /* Write-only doesn't work well with raw devices */
+  if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_WRONLY)
+    flags = ((flags & ~O_WRONLY) | O_RDWR);
 
-  extern void str2buf2uni (UNICODE_STRING &, WCHAR *, const char *);
-  UNICODE_STRING dev;
-  WCHAR devname[MAX_PATH + 1];
-  str2buf2uni (dev, devname, real_path->get_win32 ());
-  OBJECT_ATTRIBUTES attr;
-  InitializeObjectAttributes (&attr, &dev, OBJ_CASE_INSENSITIVE, NULL, NULL);
-
-  HANDLE h;
-  IO_STATUS_BLOCK io;
-  NTSTATUS status = NtOpenFile (&h, access, &attr, &io, wincap.shared (),
-				FILE_SYNCHRONOUS_IO_NONALERT);
-  if (!NT_SUCCESS (status))
-    {
-      __seterrno_from_win_error (RtlNtStatusToDosError (status));
-      return 0;
-    }
-
-  set_io_handle (h);
-  set_flags ((flags & ~O_TEXT) | O_BINARY);
-
-  if (devbufsiz > 1L)
+  int res = fhandler_base::open (flags, 0);
+  if (res && devbufsiz > 1L)
     devbuf = new char [devbufsiz];
 
-  return 1;
+  return res;
 }
 
 int
@@ -197,6 +170,7 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
   int ret;
   size_t len = ulen;
   char *tgt;
+  char *p = (char *) ptr;
 
   /* In mode O_RDWR the buffer has to be written to device first */
   ret = writebuf ();
@@ -210,15 +184,15 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
     }
 
   /* Checking a previous end of file */
-  if (eof_detected && !lastblk_to_read)
+  if (eof_detected () && !lastblk_to_read ())
     {
-      eof_detected = 0;
+      eof_detected (false);
       ulen = 0;
       return;
     }
 
   /* Checking a previous end of media */
-  if (eom_detected && !lastblk_to_read)
+  if (eom_detected () && !lastblk_to_read ())
     {
       set_errno (ENOSPC);
       goto err;
@@ -232,41 +206,36 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
 	    {
 	      bytes_to_read = min (len, devbufend - devbufstart);
 	      debug_printf ("read %d bytes from buffer (rest %d)",
-			    bytes_to_read, devbufstart - devbufend);
-	      memcpy (ptr, devbuf + devbufstart, bytes_to_read);
+			    bytes_to_read,
+			    devbufend - devbufstart - bytes_to_read);
+	      memcpy (p, devbuf + devbufstart, bytes_to_read);
 	      len -= bytes_to_read;
-	      ptr = (void *) ((char *) ptr + bytes_to_read);
+	      p += bytes_to_read;
 	      bytes_read += bytes_to_read;
 	      devbufstart += bytes_to_read;
 
-	      if (lastblk_to_read)
+	      if (lastblk_to_read ())
 		{
-		  lastblk_to_read = 0;
+		  lastblk_to_read (false);
 		  break;
 		}
 	    }
 	  if (len > 0)
 	    {
-	      if (!varblkop && len >= devbufsiz)
+	      if (len >= devbufsiz)
 		{
-		  if (get_device () == FH_TAPE)
-		    bytes_to_read = (len / devbufsiz) * devbufsiz;
-		  else
-		    bytes_to_read = (len / 512) * 512;
-		  tgt = (char *) ptr;
+		  bytes_to_read = (len / 512) * 512;
+		  tgt = p;
 		  debug_printf ("read %d bytes direct from file",bytes_to_read);
 		}
 	      else
 		{
-		  bytes_to_read = devbufsiz;
 		  tgt = devbuf;
-		  if (varblkop)
-		    debug_printf ("read variable bytes from file into buffer");
-		  else
-		    debug_printf ("read %d bytes from file into buffer",
-				  bytes_to_read);
+		  bytes_to_read = devbufsiz;
+		  debug_printf ("read %d bytes from file into buffer",
+				bytes_to_read);
 		}
-	      if (!read_file (get_handle (), tgt, bytes_to_read, &read2, &ret))
+	      if (!read_file (tgt, bytes_to_read, &read2, &ret))
 		{
 		  if (!is_eof (ret) && !is_eom (ret))
 		    {
@@ -275,9 +244,9 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
 		    }
 
 		  if (is_eof (ret))
-		    eof_detected = 1;
+		    eof_detected (true);
 		  else
-		    eom_detected = 1;
+		    eom_detected (true);
 
 		  if (!read2)
 		    {
@@ -289,9 +258,9 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
 			}
 		      break;
 		    }
-		  lastblk_to_read = 1;
+		  lastblk_to_read (true);
 		}
-	      if (! read2)
+	      if (!read2)
 	       break;
 	      if (tgt == devbuf)
 		{
@@ -300,14 +269,14 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
 		}
 	      else
 		{
-		  len -= bytes_to_read;
-		  ptr = (void *) ((char *) ptr + bytes_to_read);
-		  bytes_read += bytes_to_read;
+		  len -= read2;
+		  p += read2;
+		  bytes_read += read2;
 		}
 	    }
 	}
     }
-  else if (!read_file (get_handle (), ptr, len, &bytes_read, &ret))
+  else if (!read_file (p, len, &bytes_read, &ret))
     {
       if (!is_eof (ret) && !is_eom (ret))
 	{
@@ -317,9 +286,9 @@ fhandler_dev_raw::raw_read (void *ptr, size_t& ulen)
       if (bytes_read)
 	{
 	  if (is_eof (ret))
-	    eof_detected = 1;
+	    eof_detected (true);
 	  else
-	    eom_detected = 1;
+	    eom_detected (true);
 	}
       else if (is_eom (ret))
 	{
@@ -348,25 +317,21 @@ fhandler_dev_raw::raw_write (const void *ptr, size_t len)
   int ret;
 
   /* Checking a previous end of media on tape */
-  if (eom_detected)
+  if (eom_detected ())
     {
       set_errno (ENOSPC);
       return -1;
     }
 
-  if (!is_writing)
-    {
-      devbufend = devbufstart;
-      devbufstart = 0;
-    }
-  is_writing = 1;
+  if (!is_writing ())
+    devbufstart = devbufend = 0;
+  is_writing (true);
 
   if (devbuf)
     {
       while (len > 0)
 	{
-	  if (!varblkop &&
-	      (len < devbufsiz || devbufend > 0) && devbufend < devbufsiz)
+	  if ((len < devbufsiz || devbufend > 0) && devbufend < devbufsiz)
 	    {
 	      bytes_to_write = min (len, devbufsiz - devbufend);
 	      memcpy (devbuf + devbufend, p, bytes_to_write);
@@ -377,12 +342,7 @@ fhandler_dev_raw::raw_write (const void *ptr, size_t len)
 	    }
 	  else
 	    {
-	      if (varblkop)
-		{
-		  bytes_to_write = len;
-		  tgt = p;
-		}
-	      else if (devbufend == devbufsiz)
+	      if (devbufend == devbufsiz)
 		{
 		  bytes_to_write = devbufsiz;
 		  tgt = devbuf;
@@ -394,9 +354,7 @@ fhandler_dev_raw::raw_write (const void *ptr, size_t len)
 		}
 
 	      ret = 0;
-	      write_file (get_handle (), tgt, bytes_to_write, &written, &ret);
-	      if (written)
-		has_written = 1;
+	      write_file (tgt, bytes_to_write, &written, &ret);
 
 	      if (ret)
 		{
@@ -406,7 +364,7 @@ fhandler_dev_raw::raw_write (const void *ptr, size_t len)
 		      return -1;
 		    }
 
-		  eom_detected = 1;
+		  eom_detected (true);
 
 		  if (!written && !bytes_written)
 		    {
@@ -437,23 +395,20 @@ fhandler_dev_raw::raw_write (const void *ptr, size_t len)
     }
   else if (len > 0)
     {
-      if (!write_file (get_handle (), ptr, len, &bytes_written, &ret))
+      if (!write_file (p, len, &bytes_written, &ret))
 	{
-	  if (bytes_written)
-	    has_written = 1;
 	  if (!is_eom (ret))
 	    {
 	      __seterrno ();
 	      return -1;
 	    }
-	  eom_detected = 1;
+	  eom_detected (true);
 	  if (!bytes_written)
 	    {
 	      set_errno (ENOSPC);
 	      return -1;
 	    }
 	}
-      has_written = 1;
     }
   return bytes_written;
 }
@@ -472,11 +427,9 @@ fhandler_dev_raw::dup (fhandler_base *child)
 	fhc->devbuf = new char [devbufsiz];
       fhc->devbufstart = 0;
       fhc->devbufend = 0;
-      fhc->eom_detected = eom_detected;
-      fhc->eof_detected = eof_detected;
-      fhc->lastblk_to_read = 0;
-      fhc->varblkop = varblkop;
-      fhc->unit = unit;
+      fhc->eom_detected (eom_detected ());
+      fhc->eof_detected (eof_detected ());
+      fhc->lastblk_to_read (false);
     }
   return ret;
 }
@@ -486,17 +439,17 @@ fhandler_dev_raw::fixup_after_fork (HANDLE)
 {
   devbufstart = 0;
   devbufend = 0;
-  lastblk_to_read = 0;
+  lastblk_to_read (false);
 }
 
 void
-fhandler_dev_raw::fixup_after_exec (HANDLE)
+fhandler_dev_raw::fixup_after_exec ()
 {
   if (devbufsiz > 1L)
     devbuf = new char [devbufsiz];
   devbufstart = 0;
   devbufend = 0;
-  lastblk_to_read = 0;
+  lastblk_to_read (false);
 }
 
 int
@@ -514,7 +467,7 @@ fhandler_dev_raw::ioctl (unsigned int cmd, void *buf)
 	switch (op->rd_op)
 	  {
 	  case RDSETBLK:
-	    if (get_device () == FH_TAPE)
+	    if (get_major () == DEV_TAPE_MAJOR)
 	      {
 		struct mtop mop;
 
