@@ -1,6 +1,6 @@
 /* fhandler_registry.cc: fhandler for /proc/registry virtual filesystem
 
-   Copyright 2002, 2003 Red Hat, Inc.
+   Copyright 2002, 2003, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -16,8 +16,8 @@ details. */
 #include <sys/cygwin.h>
 #include "cygerrno.h"
 #include "security.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygheap.h"
 #include <assert.h>
@@ -97,7 +97,7 @@ int
 fhandler_registry::exists ()
 {
   int file_type = 0, index = 0, pathlen;
-  DWORD buf_size = MAX_PATH;
+  DWORD buf_size = CYG_MAX_PATH;
   LONG error;
   char buf[buf_size];
   const char *file;
@@ -152,7 +152,7 @@ fhandler_registry::exists ()
 	      file_type = 1;
 	      goto out;
 	    }
-	  buf_size = MAX_PATH;
+	  buf_size = CYG_MAX_PATH;
 	}
       if (error != ERROR_NO_MORE_ITEMS)
 	{
@@ -160,7 +160,7 @@ fhandler_registry::exists ()
 	  goto out;
 	}
       index = 0;
-      buf_size = MAX_PATH;
+      buf_size = CYG_MAX_PATH;
       while (ERROR_SUCCESS ==
 	     (error = RegEnumValue (hKey, index++, buf, &buf_size, NULL, NULL,
 				    NULL, NULL))
@@ -172,7 +172,7 @@ fhandler_registry::exists ()
 	      file_type = -1;
 	      goto out;
 	    }
-	  buf_size = MAX_PATH;
+	  buf_size = CYG_MAX_PATH;
 	}
       if (error != ERROR_NO_MORE_ITEMS)
 	{
@@ -187,14 +187,14 @@ out:
 }
 
 fhandler_registry::fhandler_registry ():
-fhandler_proc (FH_REGISTRY)
+fhandler_proc ()
 {
 }
 
 int
-fhandler_registry::fstat (struct __stat64 *buf, path_conv *pc)
+fhandler_registry::fstat (struct __stat64 *buf)
 {
-  fhandler_base::fstat (buf, pc);
+  fhandler_base::fstat (buf);
   buf->st_mode &= ~_IFMT & NO_W;
   int file_type = exists ();
   switch (file_type)
@@ -276,7 +276,7 @@ fhandler_registry::fstat (struct __stat64 *buf, path_conv *pc)
 struct dirent *
 fhandler_registry::readdir (DIR * dir)
 {
-  DWORD buf_size = MAX_PATH;
+  DWORD buf_size = CYG_MAX_PATH;
   char buf[buf_size];
   HANDLE handle;
   struct dirent *res = NULL;
@@ -324,7 +324,7 @@ retry:
     {
       /* If we're finished with sub-keys, start on values under this key.  */
       dir->__d_position |= REG_ENUM_VALUES_MASK;
-      buf_size = MAX_PATH;
+      buf_size = CYG_MAX_PATH;
       goto retry;
     }
   if (error != ERROR_SUCCESS && error != ERROR_MORE_DATA)
@@ -396,13 +396,13 @@ fhandler_registry::closedir (DIR * dir)
 }
 
 int
-fhandler_registry::open (path_conv * pc, int flags, mode_t mode)
+fhandler_registry::open (int flags, mode_t mode)
 {
   int pathlen;
   const char *file;
   HKEY handle;
 
-  int res = fhandler_virtual::open (pc, flags, mode);
+  int res = fhandler_virtual::open (flags, mode);
   if (!res)
     goto out;
 
@@ -592,7 +592,7 @@ fhandler_registry::fill_filebuf ()
     }
   return true;
 value_not_found:
-  DWORD buf_size = MAX_PATH;
+  DWORD buf_size = CYG_MAX_PATH;
   char buf[buf_size];
   int index = 0;
   while (ERROR_SUCCESS ==
@@ -604,7 +604,7 @@ value_not_found:
 	  set_errno (EISDIR);
 	  return false;
 	}
-      buf_size = MAX_PATH;
+      buf_size = CYG_MAX_PATH;
     }
   if (error != ERROR_NO_MORE_ITEMS)
     {
@@ -622,7 +622,7 @@ open_key (const char *name, REGSAM access, bool isValue)
   HKEY hKey = (HKEY) INVALID_HANDLE_VALUE;
   HKEY hParentKey = (HKEY) INVALID_HANDLE_VALUE;
   bool parentOpened = false;
-  char component[MAX_PATH];
+  char component[CYG_MAX_PATH];
 
   while (*name)
     {
