@@ -1,6 +1,6 @@
 /* fhandler_serial.cc
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -13,6 +13,7 @@ details. */
 #include <stdlib.h>
 #include "cygerrno.h"
 #include "security.h"
+#include "path.h"
 #include "fhandler.h"
 #include "sigproc.h"
 #include "pinfo.h"
@@ -22,10 +23,10 @@ details. */
 /**********************************************************************/
 /* fhandler_serial */
 
-fhandler_serial::fhandler_serial (int unit)
-  : fhandler_base (FH_SERIAL, unit), vmin_ (0), vtime_ (0), pgrp_ (myself->pgid)
+fhandler_serial::fhandler_serial ()
+  : fhandler_base (), vmin_ (0), vtime_ (0), pgrp_ (myself->pgid)
 {
-  set_need_fork_fixup ();
+  need_fork_fixup (true);
 }
 
 void
@@ -132,16 +133,16 @@ fhandler_serial::raw_read (void *ptr, size_t& ulen)
       continue;
 
     err:
-      PurgeComm (get_handle (), PURGE_RXABORT);
       debug_printf ("err %E");
-      if (GetLastError () == ERROR_OPERATION_ABORTED)
-	n = 0;
-      else
+      if (GetLastError () != ERROR_OPERATION_ABORTED)
 	{
+	  PurgeComm (get_handle (), PURGE_RXABORT);
 	  tot = -1;
 	  __seterrno ();
 	  break;
 	}
+
+      n = 0;
     }
 
 out:
@@ -168,6 +169,11 @@ fhandler_serial::raw_write (const void *ptr, size_t len)
       switch (GetLastError ())
 	{
 	case ERROR_OPERATION_ABORTED:
+	  DWORD ev;
+	  if (!ClearCommError (get_handle (), &ev, NULL))
+	    goto err;
+	  if (ev)
+	    termios_printf ("error detected %x", ev);
 	  continue;
 	case ERROR_IO_PENDING:
 	  break;
@@ -200,11 +206,11 @@ fhandler_serial::dump (void)
 void
 fhandler_serial::init (HANDLE f, DWORD flags, mode_t bin)
 {
-  (void) open (NULL, flags, bin & (O_BINARY | O_TEXT));
+  (void) open (flags, bin & (O_BINARY | O_TEXT));
 }
 
 int
-fhandler_serial::open (path_conv *, int flags, mode_t mode)
+fhandler_serial::open (int flags, mode_t mode)
 {
   int res;
   COMMTIMEOUTS to;
@@ -213,14 +219,14 @@ fhandler_serial::open (path_conv *, int flags, mode_t mode)
   syscall_printf ("fhandler_serial::open (%s, %p, %p)",
 			get_name (), flags, mode);
 
-  if (!fhandler_base::open (NULL, flags, mode))
+  if (!fhandler_base::open (flags, mode))
     return 0;
 
   res = 1;
 
   (void) SetCommMask (get_handle (), EV_RXCHAR);
 
-  set_r_no_interrupt (1);	// Handled explicitly in read code
+  uninterruptible_io (true);	// Handled explicitly in read code
 
   overlapped_setup ();
 
@@ -470,6 +476,20 @@ fhandler_serial::ioctl (unsigned int cmd, void *buffer)
 	    res = -1;
 	  }
 	break;
+      case TIOCCBRK:
+	if (ClearCommBreak (get_handle ()) == 0)
+	  {
+	    __seterrno ();
+	    res = -1;
+	  }
+	break;
+      case TIOCSBRK:
+	if (SetCommBreak (get_handle ()) == 0)
+	  {
+	    __seterrno ();
+	    res = -1;
+	  }
+	break;
      case TIOCINQ:
        if (ev & CE_FRAME || ev & CE_IOE || ev & CE_OVERRUN || ev & CE_RXOVER
 	   || ev & CE_RXPARITY)
@@ -496,21 +516,30 @@ fhandler_serial::ioctl (unsigned int cmd, void *buffer)
 int
 fhandler_serial::tcflush (int queue)
 {
-  if (queue == TCOFLUSH || queue == TCIOFLUSH)
-    PurgeComm (get_handle (), PURGE_TXABORT | PURGE_TXCLEAR);
+  DWORD flags;
 
-  if (queue == TCIFLUSH || queue == TCIOFLUSH)
-    /* Input flushing by polling until nothing turns up
-       (we stop after 1000 chars anyway) */
-    for (int max = 1000; max > 0; max--)
-      {
-	COMSTAT st;
-	if (!PurgeComm (get_handle (), PURGE_RXABORT | PURGE_RXCLEAR))
-	  break;
-	low_priority_sleep (100);
-	if (!ClearCommError (get_handle (), &ev, &st) || !st.cbInQue)
-	  break;
-      }
+  switch (queue)
+    {
+    case TCOFLUSH:
+      flags = PURGE_TXABORT | PURGE_TXCLEAR;
+      break;
+    case TCIFLUSH:
+      flags = PURGE_RXABORT | PURGE_RXCLEAR;
+      break;
+    case TCIOFLUSH:
+      flags = PURGE_TXABORT | PURGE_TXCLEAR | PURGE_RXABORT | PURGE_RXCLEAR;
+      break;
+    default:
+      termios_printf ("Invalid tcflush queue %d", queue);
+      set_errno (EINVAL);
+      return -1;
+    }
+	
+  if (!PurgeComm (get_handle (), flags))
+    {
+      __seterrno ();
+      return -1;
+    }
 
   return 0;
 }
@@ -525,7 +554,7 @@ fhandler_serial::tcsetattr (int action, const struct termios *t)
     TCSAFLUSH: flush output and discard input, then change attributes.
   */
 
-  BOOL dropDTR = FALSE;
+  bool dropDTR = false;
   COMMTIMEOUTS to;
   DCB ostate, state;
   unsigned int ovtime = vtime_, ovmin = vmin_;
@@ -556,7 +585,7 @@ fhandler_serial::tcsetattr (int action, const struct termios *t)
     case B0:
       /* Drop DTR - but leave DCB-resident bitrate as-is since
 	 0 is an invalid bitrate in Win32 */
-      dropDTR = TRUE;
+      dropDTR = true;
       break;
     case B110:
       state.BaudRate = CBR_110;
@@ -736,10 +765,10 @@ fhandler_serial::tcsetattr (int action, const struct termios *t)
       res = -1;
     }
 
-  set_r_binary ((t->c_iflag & IGNCR) ? 0 : 1);
-  set_w_binary ((t->c_oflag & ONLCR) ? 0 : 1);
+  rbinary ((t->c_iflag & IGNCR) ? false : true);
+  wbinary ((t->c_oflag & ONLCR) ? false : true);
 
-  if (dropDTR == TRUE)
+  if (dropDTR)
     {
       EscapeCommFunction (get_handle (), CLRDTR);
       tmpDtr = 0;
@@ -950,7 +979,7 @@ fhandler_serial::tcgetattr (struct termios *t)
   /* -------------- Parity errors ------------------ */
 
   /* fParity combines the function of INPCK and NOT IGNPAR */
-  if (state.fParity == TRUE)
+  if (state.fParity)
     t->c_iflag |= INPCK;
   else
     t->c_iflag |= IGNPAR;	/* not necessarily! */
@@ -975,8 +1004,7 @@ fhandler_serial::tcgetattr (struct termios *t)
      this is what we do. */
 
   /* Input flow-control */
-  if ((state.fRtsControl == RTS_CONTROL_HANDSHAKE) &&
-      (state.fOutxCtsFlow == TRUE))
+  if ((state.fRtsControl == RTS_CONTROL_HANDSHAKE) && state.fOutxCtsFlow)
     t->c_cflag |= CRTSCTS;
   if (state.fRtsControl == RTS_CONTROL_HANDSHAKE)
     t->c_cflag |= CRTSXOFF;
@@ -987,16 +1015,16 @@ fhandler_serial::tcgetattr (struct termios *t)
   /* FIXME: If tcsetattr() hasn't been called previously, this may
      give a false CLOCAL. */
 
-  if (state.fDsrSensitivity == FALSE)
+  if (!state.fDsrSensitivity)
     t->c_cflag |= CLOCAL;
 
   /* FIXME: need to handle IGNCR */
 #if 0
-  if (!get_r_binary ())
+  if (!rbinary ())
     t->c_iflag |= IGNCR;
 #endif
 
-  if (!get_w_binary ())
+  if (!wbinary ())
     t->c_oflag |= ONLCR;
 
   t->c_cc[VTIME] = vtime_ / 100;
@@ -1010,14 +1038,14 @@ fhandler_serial::tcgetattr (struct termios *t)
 void
 fhandler_serial::fixup_after_fork (HANDLE parent)
 {
-  if (get_close_on_exec ())
+  if (close_on_exec ())
     fhandler_base::fixup_after_fork (parent);
   overlapped_setup ();
   debug_printf ("io_status.hEvent %p", io_status.hEvent);
 }
 
 void
-fhandler_serial::fixup_after_exec (HANDLE)
+fhandler_serial::fixup_after_exec ()
 {
   overlapped_setup ();
   debug_printf ("io_status.hEvent %p", io_status.hEvent);
