@@ -1,6 +1,6 @@
 /* fhandler_socket.cc. See fhandler.h for a description of the fhandler classes.
 
-   Copyright 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
    This file is part of Cygwin.
 
@@ -25,19 +25,17 @@
 #include "security.h"
 #include "cygwin/version.h"
 #include "perprocess.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygheap.h"
 #include "sigproc.h"
-#include "wsock_event.h"
 #include "cygthread.h"
 #include "select.h"
+#include "wininfo.h"
 #include <unistd.h>
 
-#define ENTROPY_SOURCE_DEV_UNIT 9
-
-extern fhandler_socket *fdsock (int& fd, const char *name, SOCKET soc);
+extern bool fdsock (cygheap_fdmanip& fd, const device *, SOCKET soc);
 extern "C" {
 int sscanf (const char *, const char *, ...);
 } /* End of "C" section */
@@ -123,12 +121,22 @@ get_inet_addr (const struct sockaddr *in, int inlen,
 /**********************************************************************/
 /* fhandler_socket */
 
-fhandler_socket::fhandler_socket (int nunit)
-  : fhandler_base (FH_SOCKET), sun_path (NULL), unit (nunit)
+fhandler_socket::fhandler_socket () :
+  fhandler_base (),
+  sun_path (NULL),
+  status ()
 {
-  set_need_fork_fixup ();
+  need_fork_fixup (true);
   prot_info_ptr = (LPWSAPROTOCOL_INFOA) cmalloc (HEAP_BUF,
 						 sizeof (WSAPROTOCOL_INFOA));
+#if 0
+  if (pc.is_fs_special ())
+    {
+      fhandler_socket * fhs = (fhandler_socket *) fh;
+      fhs->set_addr_family (AF_LOCAL);
+      fhs->set_sun_path (posix_path);
+    }
+#endif
 }
 
 fhandler_socket::~fhandler_socket ()
@@ -145,10 +153,11 @@ fhandler_socket::set_connect_secret ()
   if (!entropy_source)
     {
       void *buf = malloc (sizeof (fhandler_dev_random));
-      entropy_source = new (buf) fhandler_dev_random (ENTROPY_SOURCE_DEV_UNIT);
+      entropy_source = new (buf) fhandler_dev_random ();
+      entropy_source->dev () = *urandom_dev;
     }
   if (entropy_source &&
-      !entropy_source->open (NULL, O_RDONLY))
+      !entropy_source->open (O_RDONLY))
     {
       delete entropy_source;
       entropy_source = NULL;
@@ -176,25 +185,24 @@ fhandler_socket::create_secret_event (int* secret)
   struct sockaddr_in sin;
   int sin_len = sizeof (sin);
 
+  if (secret_event)
+    return secret_event;
+
   if (::getsockname (get_socket (), (struct sockaddr*) &sin, &sin_len))
     {
       debug_printf ("error getting local socket name (%d)", WSAGetLastError ());
       return NULL;
     }
 
-  char event_name[MAX_PATH];
+  char event_name[CYG_MAX_PATH];
   secret_event_name (event_name, sin.sin_port, secret ?: connect_secret);
-  LPSECURITY_ATTRIBUTES sec = get_inheritance (true);
-  secret_event = CreateEvent (sec, FALSE, FALSE, event_name);
-  if (!secret_event && GetLastError () == ERROR_ALREADY_EXISTS)
-    secret_event = OpenEvent (EVENT_ALL_ACCESS, FALSE, event_name);
+  secret_event = CreateEvent (&sec_all, FALSE, FALSE, event_name);
 
   if (!secret_event)
-    /* nothing to do */;
-  else if (sec == &sec_all_nih || sec == &sec_none_nih)
-    ProtectHandle (secret_event);
-  else
-    ProtectHandleINH (secret_event);
+    debug_printf("create event %E");
+  else if (close_on_exec ())
+    /* Event allows inheritance, but handle will not be inherited */
+    set_no_inheritance (secret_event, 1);
 
   return secret_event;
 }
@@ -215,7 +223,7 @@ void
 fhandler_socket::close_secret_event ()
 {
   if (secret_event)
-    ForceCloseHandle (secret_event);
+    CloseHandle (secret_event);
   secret_event = NULL;
 }
 
@@ -223,15 +231,12 @@ int
 fhandler_socket::check_peer_secret_event (struct sockaddr_in* peer, int* secret)
 {
 
-  char event_name[MAX_PATH];
+  char event_name[CYG_MAX_PATH];
 
   secret_event_name (event_name, peer->sin_port, secret ?: connect_secret);
   HANDLE ev = CreateEvent (&sec_all_nih, FALSE, FALSE, event_name);
-  if (!ev && GetLastError () == ERROR_ALREADY_EXISTS)
-    {
-      debug_printf ("event \"%s\" already exists", event_name);
-      ev = OpenEvent (EVENT_ALL_ACCESS, FALSE, event_name);
-    }
+  if (!ev)
+    debug_printf("create event %E");
 
   signal_secret_event ();
 
@@ -240,7 +245,7 @@ fhandler_socket::check_peer_secret_event (struct sockaddr_in* peer, int* secret)
       DWORD rc = WaitForSingleObject (ev, 10000);
       debug_printf ("WFSO rc=%d", rc);
       CloseHandle (ev);
-      return (rc == WAIT_OBJECT_0 ? 1 : 0 );
+      return (rc == WAIT_OBJECT_0 ? 1 : 0);
     }
   else
     return 0;
@@ -300,11 +305,11 @@ fhandler_socket::fixup_after_fork (HANDLE parent)
 }
 
 void
-fhandler_socket::fixup_after_exec (HANDLE parent)
+fhandler_socket::fixup_after_exec ()
 {
   debug_printf ("here");
-  if (!get_close_on_exec ())
-    fixup_after_fork (parent);
+  if (!close_on_exec ())
+    fixup_after_fork (NULL);
 #if 0
   else if (!winsock2_active)
     closesocket (get_socket ());
@@ -320,7 +325,7 @@ fhandler_socket::dup (fhandler_base *child)
   if (get_addr_family () == AF_LOCAL)
     fhs->set_sun_path (get_sun_path ());
   fhs->set_socket_type (get_socket_type ());
-  fhs->set_connect_state (get_connect_state ());
+  fhs->connect_state (connect_state ());
 
   if (winsock2_active)
     {
@@ -356,30 +361,15 @@ fhandler_socket::dup (fhandler_base *child)
       __seterrno ();
       return -1;
     }
+  VerifyHandle (nh);
   fhs->set_io_handle (nh);
   return 0;
 }
 
 int __stdcall
-fhandler_socket::fstat (struct __stat64 *buf, path_conv *pc)
+fhandler_socket::fstat (struct __stat64 *buf)
 {
-  int res;
-  if (get_addr_family () == AF_LOCAL && get_sun_path () && !get_socket_type ())
-    {
-      path_conv spc (get_sun_path (), PC_SYM_NOFOLLOW | PC_NULLEMPTY | PC_FULL,
-		     NULL);
-      fhandler_base *fh = cygheap->fdtab.build_fhandler (-1, FH_DISK,
-							 get_sun_path (),
-							 spc, 0);
-      if (fh)
-	{
-	  res = fh->fstat (buf, &spc);
-	  buf->st_rdev = buf->st_size = buf->st_blocks = 0;
-	  return res;
-	}
-    }
-
-  res = fhandler_base::fstat (buf, pc);
+  int res = fhandler_base::fstat (buf);
   if (!res)
     {
       if (get_socket_type ()) /* fstat */
@@ -451,8 +441,9 @@ fhandler_socket::bind (const struct sockaddr *name, int namelen)
       if (!(mode & (S_IWUSR | S_IWGRP | S_IWOTH)))
 	attr |= FILE_ATTRIBUTE_READONLY;
       SECURITY_ATTRIBUTES sa = sec_none;
+      security_descriptor sd;
       if (allow_ntsec && pc.has_acls ())
-	set_security_attribute (mode, &sa, alloca (4096), 4096);
+	set_security_attribute (mode, &sa, sd);
       HANDLE fh = CreateFile (pc, GENERIC_WRITE, 0, &sa, CREATE_NEW, attr, 0);
       if (fh == INVALID_HANDLE_VALUE)
 	{
@@ -495,8 +486,8 @@ int
 fhandler_socket::connect (const struct sockaddr *name, int namelen)
 {
   int res = -1;
-  BOOL secret_check_failed = FALSE;
-  BOOL in_progress = FALSE;
+  bool secret_check_failed = false;
+  bool in_progress = false;
   sockaddr_in sin;
   int secret [4];
   DWORD err;
@@ -510,11 +501,11 @@ fhandler_socket::connect (const struct sockaddr *name, int namelen)
     {
       /* Special handling for connect to return the correct error code
 	 when called on a non-blocking socket. */
-      if (is_nonblocking () || is_connect_pending ())
+      if (is_nonblocking () || connect_state () == connect_pending)
 	{
 	  err = WSAGetLastError ();
 	  if (err == WSAEWOULDBLOCK || err == WSAEALREADY)
-	    in_progress = TRUE;
+	    in_progress = true;
 
 	  if (err == WSAEWOULDBLOCK)
 	    WSASetLastError (WSAEINPROGRESS);
@@ -529,7 +520,7 @@ fhandler_socket::connect (const struct sockaddr *name, int namelen)
 	{
 	  if (!create_secret_event (secret))
 	    {
-	      secret_check_failed = TRUE;
+	      secret_check_failed = true;
 	    }
 	  else if (in_progress)
 	    signal_secret_event ();
@@ -539,8 +530,8 @@ fhandler_socket::connect (const struct sockaddr *name, int namelen)
 	{
 	  if (!check_peer_secret_event (&sin, secret))
 	    {
-	      debug_printf ( "accept from unauthorized server" );
-	      secret_check_failed = TRUE;
+	      debug_printf ("accept from unauthorized server");
+	      secret_check_failed = true;
 	    }
        }
 
@@ -556,9 +547,9 @@ fhandler_socket::connect (const struct sockaddr *name, int namelen)
 
   err = WSAGetLastError ();
   if (err == WSAEINPROGRESS || err == WSAEALREADY)
-    set_connect_state (CONNECT_PENDING);
+    connect_state (connect_pending);
   else
-    set_connect_state (CONNECTED);
+    connect_state (connected);
 
   return res;
 }
@@ -570,7 +561,7 @@ fhandler_socket::listen (int backlog)
   if (res)
     set_winsock_errno ();
   else
-    set_connect_state (CONNECTED);
+    connect_state (connected);
   return res;
 }
 
@@ -578,8 +569,8 @@ int
 fhandler_socket::accept (struct sockaddr *peer, int *len)
 {
   int res = -1;
-  BOOL secret_check_failed = FALSE;
-  BOOL in_progress = FALSE;
+  bool secret_check_failed = false;
+  bool in_progress = false;
 
   /* Allows NULL peer and len parameters. */
   struct sockaddr_in peer_dummy;
@@ -602,14 +593,14 @@ fhandler_socket::accept (struct sockaddr *peer, int *len)
   res = ::accept (get_socket (), peer, len);
 
   if ((SOCKET) res == INVALID_SOCKET && WSAGetLastError () == WSAEWOULDBLOCK)
-    in_progress = TRUE;
+    in_progress = true;
 
   if (get_addr_family () == AF_LOCAL && get_socket_type () == SOCK_STREAM)
     {
       if ((SOCKET) res != INVALID_SOCKET || in_progress)
 	{
 	  if (!create_secret_event ())
-	    secret_check_failed = TRUE;
+	    secret_check_failed = true;
 	  else if (in_progress)
 	    signal_secret_event ();
 	}
@@ -620,7 +611,7 @@ fhandler_socket::accept (struct sockaddr *peer, int *len)
 	  if (!check_peer_secret_event ((struct sockaddr_in*) peer))
 	    {
 	      debug_printf ("connect from unauthorized client");
-	      secret_check_failed = TRUE;
+	      secret_check_failed = true;
 	    }
 	}
 
@@ -639,16 +630,13 @@ fhandler_socket::accept (struct sockaddr *peer, int *len)
   else
     {
       cygheap_fdnew res_fd;
-      fhandler_socket* res_fh = NULL;
-      if (res_fd >= 0)
-	  res_fh = fdsock (res_fd, get_name (), res);
-      if (res_fh)
-	{
+      if (res_fd >= 0 && fdsock (res_fd, &dev (), res))
+        {
 	  if (get_addr_family () == AF_LOCAL)
-	    res_fh->set_sun_path (get_sun_path ());
-	  res_fh->set_addr_family (get_addr_family ());
-	  res_fh->set_socket_type (get_socket_type ());
-	  res_fh->set_connect_state (CONNECTED);
+	    ((fhandler_socket *) res_fd)->set_sun_path (get_sun_path ());
+	  ((fhandler_socket *) res_fd)->set_addr_family (get_addr_family ());
+	  ((fhandler_socket *) res_fd)->set_socket_type (get_socket_type ());
+	  ((fhandler_socket *) res_fd)->connect_state (connected);
 	  res = res_fd;
 	}
       else
@@ -658,6 +646,7 @@ fhandler_socket::accept (struct sockaddr *peer, int *len)
 	}
     }
 
+  debug_printf ("res %d", res);
   return res;
 }
 
@@ -706,6 +695,111 @@ fhandler_socket::getpeername (struct sockaddr *name, int *namelen)
   return res;
 }
 
+bool
+fhandler_socket::prepare (HANDLE &event, long event_mask)
+{
+  WSASetLastError (0);
+  closed (false);
+  if ((event = WSACreateEvent ()) == WSA_INVALID_EVENT)
+    {
+      debug_printf ("WSACreateEvent: %E");
+      return false;
+    }
+  if (WSAEventSelect (get_socket (), event, event_mask) == SOCKET_ERROR)
+    {
+      debug_printf ("WSAEventSelect: %E");
+      return false;
+    }
+  return true;
+}
+
+int
+fhandler_socket::wait (HANDLE event, int flags)
+{
+  int ret = SOCKET_ERROR;
+  int wsa_err = 0;
+  WSAEVENT ev[2] = { event, signal_arrived };
+  WSANETWORKEVENTS evts;
+
+  switch (WSAWaitForMultipleEvents (2, ev, FALSE, 10, FALSE))
+    {
+      case WSA_WAIT_TIMEOUT:
+        ret = 0;
+	break;
+      case WSA_WAIT_EVENT_0:
+	if (!WSAEnumNetworkEvents (get_socket (), event, &evts))
+	  {
+	    if (!evts.lNetworkEvents)
+	      {
+	        ret = 0;
+		break;
+	      }
+	    if (evts.lNetworkEvents & FD_OOB)
+	      {
+		if (evts.iErrorCode[FD_OOB_BIT])
+		  wsa_err = evts.iErrorCode[FD_OOB_BIT];
+		else if (flags & MSG_OOB)
+		  ret = 0;
+		else
+		  {
+		    raise (SIGURG);
+		    WSASetLastError (WSAEINTR);
+		    break;
+		  }
+	      }
+	    if (evts.lNetworkEvents & FD_READ)
+	      {
+		if (evts.iErrorCode[FD_READ_BIT])
+		  wsa_err = evts.iErrorCode[FD_READ_BIT];
+		else
+		  ret = 0;
+	      }
+	    else if (evts.lNetworkEvents & FD_WRITE)
+	      {
+		if (evts.iErrorCode[FD_WRITE_BIT])
+		  wsa_err = evts.iErrorCode[FD_WRITE_BIT];
+		else
+		  ret = 0;
+	      }
+	    if (evts.lNetworkEvents & FD_CLOSE)
+	      {
+		closed (true);
+		if (!wsa_err)
+		  {
+		    if (evts.iErrorCode[FD_CLOSE_BIT])
+		      wsa_err = evts.iErrorCode[FD_CLOSE_BIT];
+		    else
+		      ret = 0;
+		  }
+	      }
+	    if (wsa_err)
+	      WSASetLastError (wsa_err);
+	  }
+	break;
+      case WSA_WAIT_EVENT_0 + 1:
+        WSASetLastError (WSAEINTR);
+	break;
+      default:
+	WSASetLastError (WSAEFAULT);
+	break;
+    }
+  return ret;
+}
+
+void
+fhandler_socket::release (HANDLE event)
+{
+  int last_err = WSAGetLastError ();
+  /* KB 168349: NT4 fails if the event parameter is not NULL. */
+  WSAEventSelect (get_socket (), NULL, 0);
+  unsigned long non_block = 0;
+  if (ioctlsocket (get_socket (), FIONBIO, &non_block))
+    debug_printf ("return to blocking failed: %d", WSAGetLastError ());
+  else
+    WSASetLastError (last_err);
+  WSACloseEvent (event);
+}
+
 int
 fhandler_socket::readv (const struct iovec *const iov, const int iovcnt,
 			ssize_t tot)
@@ -727,7 +821,7 @@ int
 fhandler_socket::recvfrom (void *ptr, size_t len, int flags,
 			   struct sockaddr *from, int *fromlen)
 {
-  int res;
+  int res = SOCKET_ERROR;
   DWORD ret;
 
   flags &= MSG_WINMASK;
@@ -739,19 +833,26 @@ fhandler_socket::recvfrom (void *ptr, size_t len, int flags,
     {
       WSABUF wsabuf = { len, (char *) ptr };
 
-      if (is_nonblocking ())
-	res = WSARecvFrom (get_socket (), &wsabuf, 1, &ret, (DWORD *) &flags,
-			   from, fromlen,
-			   NULL, NULL);
+      if (is_nonblocking () || closed () || async_io ())
+	res = WSARecvFrom (get_socket (), &wsabuf, 1, (ret = 0, &ret),
+			   (DWORD *) &flags, from, fromlen, NULL, NULL);
       else
 	{
-	  wsock_event wsock_evt;
-	  res = WSARecvFrom (get_socket (), &wsabuf, 1, &ret, (DWORD *) &flags,
-			     from, fromlen,
-			     wsock_evt.prepare (), NULL);
-
-	  if (res == SOCKET_ERROR && WSAGetLastError () == WSA_IO_PENDING)
-	    ret = res = wsock_evt.wait (get_socket (), (DWORD *) &flags);
+	  HANDLE evt;
+	  if (prepare (evt, FD_CLOSE | FD_READ | (owner () ? FD_OOB : 0)))
+	    {
+              do
+                {
+		  DWORD lflags = (DWORD) flags;
+		  res = WSARecvFrom (get_socket (), &wsabuf, 1, &ret, &lflags,
+				     from, fromlen, NULL, NULL);
+                }
+              while (res == SOCKET_ERROR
+                     && WSAGetLastError () == WSAEWOULDBLOCK
+		     && !closed ()
+		     && !(res = wait (evt, flags)));
+	      release (evt);
+	    }
 	}
     }
 
@@ -762,7 +863,6 @@ fhandler_socket::recvfrom (void *ptr, size_t len, int flags,
       if (WSAGetLastError () == WSAEMSGSIZE)
 	return len;
 
-      res = -1;
       set_winsock_errno ();
     }
   else
@@ -790,13 +890,12 @@ fhandler_socket::recvmsg (struct msghdr *msg, int flags, ssize_t tot)
   struct sockaddr *from = (struct sockaddr *) msg->msg_name;
   int *fromlen = from ? &msg->msg_namelen : NULL;
 
-  int res;
+  int res = SOCKET_ERROR;
 
   if (!winsock2_active)
     {
       if (iovcnt == 1)
-	res = recvfrom (iov->iov_base, iov->iov_len, flags,
-			from, fromlen);
+	res = recvfrom (iov->iov_base, iov->iov_len, flags, from, fromlen);
       else
 	{
 	  if (tot == -1)	// i.e. if not pre-calculated by the caller.
@@ -816,12 +915,11 @@ fhandler_socket::recvmsg (struct msghdr *msg, int flags, ssize_t tot)
 	  if (!buf)
 	    {
 	      set_errno (ENOMEM);
-	      res = -1;
+	      res = SOCKET_ERROR;
 	    }
 	  else
 	    {
-	      res = recvfrom (buf, tot, flags,
-			      from, fromlen);
+	      res = recvfrom (buf, tot, flags, from, fromlen);
 
 	      const struct iovec *iovptr = iov;
 	      int nbytes = res;
@@ -857,21 +955,26 @@ fhandler_socket::recvmsg (struct msghdr *msg, int flags, ssize_t tot)
 
       DWORD ret;
 
-      if (is_nonblocking ())
-	res = WSARecvFrom (get_socket (),
-			   wsabuf, iovcnt, &ret, (DWORD *) &flags,
-			   from, fromlen,
-			   NULL, NULL);
+      if (is_nonblocking () || closed () || async_io ())
+	res = WSARecvFrom (get_socket (), wsabuf, iovcnt, (ret = 0, &ret),
+			   (DWORD *) &flags, from, fromlen, NULL, NULL);
       else
 	{
-	  wsock_event wsock_evt;
-	  res = WSARecvFrom (get_socket (),
-			     wsabuf, iovcnt, &ret, (DWORD *) &flags,
-			     from, fromlen,
-			     wsock_evt.prepare (), NULL);
-
-	  if (res == SOCKET_ERROR && WSAGetLastError () == WSA_IO_PENDING)
-	    ret = res = wsock_evt.wait (get_socket (), (DWORD *) &flags);
+	  HANDLE evt;
+	  if (prepare (evt, FD_CLOSE | FD_READ | (owner () ? FD_OOB : 0)))
+	    {
+              do
+                {
+		  DWORD lflags = (DWORD) flags;
+		  res = WSARecvFrom (get_socket (), wsabuf, iovcnt, &ret,
+				     &lflags, from, fromlen, NULL, NULL);
+                }
+              while (res == SOCKET_ERROR
+		     && WSAGetLastError () == WSAEWOULDBLOCK
+		     && !closed ()
+		     && !(res = wait (evt, flags)));
+	      release (evt);
+	    }
 	}
 
       if (res == SOCKET_ERROR)
@@ -881,7 +984,6 @@ fhandler_socket::recvmsg (struct msghdr *msg, int flags, ssize_t tot)
 	  if (WSAGetLastError () == WSAEMSGSIZE)
 	    return len;
 
-	  res = -1;
 	  set_winsock_errno ();
 	}
       else
@@ -915,9 +1017,9 @@ fhandler_socket::sendto (const void *ptr, size_t len, int flags,
   sockaddr_in sin;
 
   if (to && !get_inet_addr (to, tolen, &sin, &tolen))
-    return -1;
+    return SOCKET_ERROR;
 
-  int res;
+  int res = SOCKET_ERROR;
   DWORD ret;
 
   if (!winsock2_active)
@@ -928,29 +1030,34 @@ fhandler_socket::sendto (const void *ptr, size_t len, int flags,
     {
       WSABUF wsabuf = { len, (char *) ptr };
 
-      if (is_nonblocking ())
-	res = WSASendTo (get_socket (), &wsabuf, 1, &ret,
+      if (is_nonblocking () || closed () || async_io ())
+	res = WSASendTo (get_socket (), &wsabuf, 1, (ret = 0, &ret),
 			 flags & MSG_WINMASK,
 			 (to ? (const struct sockaddr *) &sin : NULL), tolen,
 			 NULL, NULL);
       else
 	{
-	  wsock_event wsock_evt;
-	  res = WSASendTo (get_socket (), &wsabuf, 1, &ret,
-			   flags & MSG_WINMASK,
-			   (to ? (const struct sockaddr *) &sin : NULL), tolen,
-			   wsock_evt.prepare (), NULL);
-
-	  if (res == SOCKET_ERROR && WSAGetLastError () == WSA_IO_PENDING)
-	    ret = res = wsock_evt.wait (get_socket (), (DWORD *) &flags);
+	  HANDLE evt;
+	  if (prepare (evt, FD_CLOSE | FD_WRITE | (owner () ? FD_OOB : 0)))
+	    {
+	      do 
+		{
+		  res = WSASendTo (get_socket (), &wsabuf, 1, &ret,
+				   flags & MSG_WINMASK,
+				   (to ? (const struct sockaddr *) &sin : NULL),
+				   tolen, NULL, NULL);
+		}
+	      while (res == SOCKET_ERROR
+	             && WSAGetLastError () == WSAEWOULDBLOCK
+		     && !(res = wait (evt, 0))
+		     && !closed ());
+	      release (evt);
+	    }
 	}
     }
 
   if (res == SOCKET_ERROR)
-    {
-      res = -1;
-      set_winsock_errno ();
-    }
+    set_winsock_errno ();
   else
     res = ret;
 
@@ -959,7 +1066,7 @@ fhandler_socket::sendto (const void *ptr, size_t len, int flags,
      EPIPE is generated if the local end has been shut down on a connection
      oriented socket.  In this case the process will also receive a SIGPIPE
      unless MSG_NOSIGNAL is set.  */
-  if (res == -1 && get_errno () == ESHUTDOWN
+  if (res == SOCKET_ERROR && get_errno () == ESHUTDOWN
       && get_socket_type () == SOCK_STREAM)
     {
       set_errno (EPIPE);
@@ -985,7 +1092,7 @@ fhandler_socket::sendmsg (const struct msghdr *msg, int flags, ssize_t tot)
   struct iovec *const iov = msg->msg_iov;
   const int iovcnt = msg->msg_iovlen;
 
-  int res;
+  int res = SOCKET_ERROR;
 
   if (!winsock2_active)
     {
@@ -1012,7 +1119,7 @@ fhandler_socket::sendmsg (const struct msghdr *msg, int flags, ssize_t tot)
 	  if (!buf)
 	    {
 	      set_errno (ENOMEM);
-	      res = -1;
+	      res = SOCKET_ERROR;
 	    }
 	  else
 	    {
@@ -1054,32 +1161,49 @@ fhandler_socket::sendmsg (const struct msghdr *msg, int flags, ssize_t tot)
 
       DWORD ret;
 
-      if (is_nonblocking ())
-	res = WSASendTo (get_socket (), wsabuf, iovcnt, &ret, flags,
-			 (struct sockaddr *) msg->msg_name,
-			 msg->msg_namelen,
-			 NULL, NULL);
+      if (is_nonblocking () || closed () || async_io ())
+	res = WSASendTo (get_socket (), wsabuf, iovcnt, (ret = 0, &ret),
+			 flags, (struct sockaddr *) msg->msg_name,
+			 msg->msg_namelen, NULL, NULL);
       else
 	{
-	  wsock_event wsock_evt;
-	  res = WSASendTo (get_socket (), wsabuf, iovcnt, &ret, flags,
-			   (struct sockaddr *) msg->msg_name,
-			   msg->msg_namelen,
-			   wsock_evt.prepare (), NULL);
-
-	  if (res == SOCKET_ERROR && WSAGetLastError () == WSA_IO_PENDING)
-	    ret = res = wsock_evt.wait (get_socket (), (DWORD *) &flags);
+	  HANDLE evt;
+	  if (prepare (evt, FD_CLOSE | FD_WRITE | (owner () ? FD_OOB : 0)))
+	    {
+              do
+                {
+		  res = WSASendTo (get_socket (), wsabuf, iovcnt,
+		  		   &ret, flags,
+				   (struct sockaddr *) msg->msg_name,
+				   msg->msg_namelen, NULL, NULL);
+                }
+              while (res == SOCKET_ERROR
+	             && WSAGetLastError () == WSAEWOULDBLOCK
+		     && !(res = wait (evt, 0))
+	             && !closed ());
+	      release (evt);
+	    }
 	}
 
       if (res == SOCKET_ERROR)
-	{
-	  res = -1;
-	  set_winsock_errno ();
-	}
+	set_winsock_errno ();
       else
 	res = ret;
     }
 
+  /* Special handling for EPIPE and SIGPIPE.
+
+     EPIPE is generated if the local end has been shut down on a connection
+     oriented socket.  In this case the process will also receive a SIGPIPE
+     unless MSG_NOSIGNAL is set.  */
+  if (res == SOCKET_ERROR && get_errno () == ESHUTDOWN
+      && get_socket_type () == SOCK_STREAM)
+    {
+      set_errno (EPIPE);
+      if (! (flags & MSG_NOSIGNAL))
+	raise (SIGPIPE);
+    }
+
   return res;
 }
 
@@ -1094,14 +1218,14 @@ fhandler_socket::shutdown (int how)
     switch (how)
       {
       case SHUT_RD:
-	set_shutdown_read ();
+	saw_shutdown_read (true);
 	break;
       case SHUT_WR:
-	set_shutdown_write ();
+	saw_shutdown_write (true);
 	break;
       case SHUT_RDWR:
-	set_shutdown_read ();
-	set_shutdown_write ();
+	saw_shutdown_read (true);
+	saw_shutdown_write (true);
 	break;
       }
   return res;
@@ -1117,7 +1241,7 @@ fhandler_socket::close ()
      solution but it helps in many cases. */
   struct linger linger;
   linger.l_onoff = 1;
-  linger.l_linger = 240; /* seconds. default 2MSL value according to MSDN. */
+  linger.l_linger = 240; /* secs. default 2MSL value according to MSDN. */
   setsockopt (get_socket (), SOL_SOCKET, SO_LINGER,
 	      (const char *)&linger, sizeof linger);
 
@@ -1248,18 +1372,23 @@ fhandler_socket::ioctl (unsigned int cmd, void *p)
 	break;
       }
     case FIOASYNC:
-      res = WSAAsyncSelect (get_socket (), gethwnd (), WM_ASYNCIO,
+      res = WSAAsyncSelect (get_socket (), winmsg, WM_ASYNCIO,
 	      *(int *) p ? ASYNC_MASK : 0);
       syscall_printf ("Async I/O on socket %s",
 	      *(int *) p ? "started" : "cancelled");
-      set_async (*(int *) p);
+      async_io (*(int *) p != 0);
+      break;
+    case FIONREAD:
+      res = ioctlsocket (get_socket (), FIONREAD, (unsigned long *) p);
+      if (res == SOCKET_ERROR)
+        set_winsock_errno ();
       break;
     default:
       /* We must cancel WSAAsyncSelect (if any) before setting socket to
        * blocking mode
        */
       if (cmd == FIONBIO && *(int *) p == 0)
-	WSAAsyncSelect (get_socket (), gethwnd (), 0, 0);
+	WSAAsyncSelect (get_socket (), winmsg, 0, 0);
       res = ioctlsocket (get_socket (), cmd, (unsigned long *) p);
       if (res == SOCKET_ERROR)
 	  set_winsock_errno ();
@@ -1268,8 +1397,8 @@ fhandler_socket::ioctl (unsigned int cmd, void *p)
 	  syscall_printf ("socket is now %sblocking",
 			    *(int *) p ? "non" : "");
 	  /* Start AsyncSelect if async socket unblocked */
-	  if (*(int *) p && get_async ())
-	    WSAAsyncSelect (get_socket (), gethwnd (), WM_ASYNCIO, ASYNC_MASK);
+	  if (*(int *) p && async_io ())
+	    WSAAsyncSelect (get_socket (), winmsg, WM_ASYNCIO, ASYNC_MASK);
 
 	  set_nonblocking (*(int *) p);
 	}
@@ -1287,6 +1416,14 @@ fhandler_socket::fcntl (int cmd, void *arg)
 
   switch (cmd)
     {
+    case F_SETOWN:
+      {
+	/* Urgh!  Bad hack! */
+	pid_t pid = (pid_t) arg;
+	owner (pid == getpid ());
+	debug_printf ("owner set to %d", owner ());
+      }
+      break;
     case F_SETFL:
       {
 	/* Carefully test for the O_NONBLOCK or deprecated OLD_O_NDELAY flag.
@@ -1310,11 +1447,13 @@ fhandler_socket::fcntl (int cmd, void *arg)
 }
 
 void
-fhandler_socket::set_close_on_exec (int val)
+fhandler_socket::set_close_on_exec (bool val)
 {
+  if (secret_event)
+    set_no_inheritance (secret_event, val);
   if (!winsock2_active) /* < Winsock 2.0 */
-    set_inheritance (get_handle (), val);
-  set_close_on_exec_flag (val);
+    set_no_inheritance (get_handle (), val);
+  close_on_exec (val);
   debug_printf ("set close_on_exec for %s to %d", get_name (), val);
 }
 
