@@ -440,7 +440,7 @@ process_ioctl (void *)
 fhandler_tty_slave::fhandler_tty_slave ()
   : fhandler_tty_common ()
 {
-  set_r_no_interrupt (1);
+  uninterruptible_io (true);
 }
 
 /* FIXME: This function needs to close handles when it has
@@ -466,14 +466,14 @@ fhandler_tty_slave::open (int flags, mode_t)
 
   set_flags ((flags & ~O_TEXT) | O_BINARY);
   /* Create synchronisation events */
-  char buf[40];
+  char buf[CYG_MAX_PATH];
 
   /* output_done_event may or may not exist.  It will exist if the tty
      was opened by fhandler_tty_master::init, normally called at
      startup if use_tty is non-zero.  It will not exist if this is a
      pty opened by fhandler_pty_master::open.  In the former case, tty
      output is handled by a separate thread which controls output.  */
-  __small_sprintf (buf, OUTPUT_DONE_EVENT, get_unit ());
+  shared_name (buf, OUTPUT_DONE_EVENT, get_unit ());
   output_done_event = OpenEvent (EVENT_ALL_ACCESS, TRUE, buf);
 
   if (!(output_mutex = get_ttyp ()->open_output_mutex ()))
@@ -488,7 +488,7 @@ fhandler_tty_slave::open (int flags, mode_t)
       __seterrno ();
       return 0;
     }
-  __small_sprintf (buf, INPUT_AVAILABLE_EVENT, get_unit ());
+  shared_name (buf, INPUT_AVAILABLE_EVENT, get_unit ());
   if (!(input_available_event = OpenEvent (EVENT_ALL_ACCESS, TRUE, buf)))
     {
       termios_printf ("open input event failed, %E");
@@ -498,9 +498,9 @@ fhandler_tty_slave::open (int flags, mode_t)
 
   /* The ioctl events may or may not exist.  See output_done_event,
      above.  */
-  __small_sprintf (buf, IOCTL_REQUEST_EVENT, get_unit ());
+  shared_name (buf, IOCTL_REQUEST_EVENT, get_unit ());
   ioctl_request_event = OpenEvent (EVENT_ALL_ACCESS, TRUE, buf);
-  __small_sprintf (buf, IOCTL_DONE_EVENT, get_unit ());
+  shared_name (buf, IOCTL_DONE_EVENT, get_unit ());
   ioctl_done_event = OpenEvent (EVENT_ALL_ACCESS, TRUE, buf);
 
   /* FIXME: Needs a method to eliminate tty races */
@@ -1157,13 +1157,14 @@ fhandler_pty_master::fhandler_pty_master ()
 int
 fhandler_pty_master::open (int flags, mode_t)
 {
-  int ntty = cygwin_shared->tty.allocate_tty (0);
+  int ntty = cygwin_shared->tty.allocate_tty (false);
   if (ntty < 0)
     return 0;
 
   slave = *ttys_dev;
   slave.setunit (ntty);
   cygwin_shared->tty[ntty]->common_init (this);
+  ReleaseMutex (tty_mutex);	// lock was set in allocate_tty
   inuse = get_ttyp ()->create_inuse (TTY_MASTER_ALIVE);
   set_flags ((flags & ~O_TEXT) | O_BINARY);
   set_open_status ();
@@ -1334,10 +1335,10 @@ fhandler_pty_master::ptsname ()
 }
 
 void
-fhandler_tty_common::set_close_on_exec (int val)
+fhandler_tty_common::set_close_on_exec (bool val)
 {
   if (archetype)
-    set_close_on_exec_flag (val);
+    close_on_exec (val);
   else
     {
       if (output_done_event)
@@ -1359,7 +1360,7 @@ fhandler_tty_common::set_close_on_exec (int val)
 	 It is here because we need to specify the "from_pty" stuff here or
 	 we'll get warnings from ForceCloseHandle when debugging. */
       set_no_inheritance (get_io_handle (), val);
-      set_close_on_exec_flag (val);
+      close_on_exec (val);
 #endif
     }
 }
@@ -1391,7 +1392,7 @@ fhandler_tty_common::fixup_after_fork (HANDLE parent)
 }
 
 void
-fhandler_pty_master::set_close_on_exec (int val)
+fhandler_pty_master::set_close_on_exec (bool val)
 {
   fhandler_tty_common::set_close_on_exec (val);
 
@@ -1414,6 +1415,6 @@ fhandler_tty_master::init_console ()
 
   console->init (INVALID_HANDLE_VALUE, GENERIC_READ | GENERIC_WRITE, O_BINARY);
   cygheap->open_fhs--;  	/* handled when individual fds are opened */
-  console->set_r_no_interrupt (1);
+  console->uninterruptible_io (true);
   return 0;
 }
