@@ -1,6 +1,6 @@
 /* version.h -- Cygwin version numbers and accompanying documentation.
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -34,15 +34,15 @@ details. */
    that didn't invalidate existing executables.  Both numbers were
    recorded in the executable and in the dll.
 
-   In October 1998 (starting with Cygwin 19.6), we started a new
-   means of Cygwin versioning: */
+   In October 1998 (starting with Cygwin 19.6), we started a new method
+   of Cygwin versioning: */
 
       /* The DLL major and minor numbers correspond to the "version of
-	 the Cygwin library".  This version is used to track important
+	 the Cygwin shared library".  This version is used to track important
 	 changes to the DLL and is mainly informative in nature. */
 
-#define CYGWIN_VERSION_DLL_MAJOR 1003
-#define CYGWIN_VERSION_DLL_MINOR 22
+#define CYGWIN_VERSION_DLL_MAJOR 1005
+#define CYGWIN_VERSION_DLL_MINOR 10
 
       /* Major numbers before CYGWIN_VERSION_DLL_EPOCH are
 	 incompatible. */
@@ -84,6 +84,10 @@ details. */
 #define CYGWIN_VERSION_CHECK_FOR_OLD_O_NONBLOCK \
   (CYGWIN_VERSION_DLL_MAKE_COMBINED (user_data->api_major, user_data->api_minor) <= \
   28)
+
+#define CYGWIN_VERSION_CHECK_FOR_USING_BIG_TYPES \
+  (CYGWIN_VERSION_DLL_MAKE_COMBINED (user_data->api_major, user_data->api_minor) >= \
+  79)
      /* We used to use the DLL major/minor to track
 	non-backward-compatible interface changes to the API.  Now we
 	use an API major/minor number for this purpose. */
@@ -188,20 +192,74 @@ details. */
        77: thread-safe exit/at_exit
        78: Use stat and fstat rather than _stat, and _fstat.
 	   Export btowc and trunc.
+       79: Export acl32 aclcheck32 aclfrommode32 aclfrompbits32 aclfromtext32
+		  aclsort32 acltomode32 acltopbits32 acltotext32 facl32
+		  fgetpos64 fopen64 freopen64 fseeko64 fsetpos64 ftello64
+		  _open64 _lseek64 _fstat64 _stat64 mknod32
+       80: Export pthread_rwlock stuff
+       81: CW_CHECK_NTSEC addition to external.cc
+       82: Export wcscoll wcswidth wcwidth
+       83: Export gethostid
+       84: Pty open allocates invisible console.  64 bit interface
+       85: Export new 32/64 functions from API 0.79 only with leading
+	   underscore.  No problems with backward compatibility since no
+	   official release has been made so far.  This change removes
+	   exported symbols like fopen64, which might confuse configure.
+       86: Export ftok
+       87: Export vsyslog
+       88: Export _getreent
+       89: Export __mempcpy
+       90: Export _fopen64
+       91: Export argz_add argz_add_sep argz_append argz_count argz_create
+	   argz_create_sep argz_delete argz_extract argz_insert
+	   argz_next argz_replace argz_stringify envz_add envz_entry
+	   envz_get envz_merge envz_remove envz_strip
+       92: Export getusershell, setusershell, endusershell
+       93: Export daemon, forkpty, openpty, iruserok, ruserok, login_tty,
+	   openpty, forkpty, revoke, logwtmp, updwtmp
+       94: Export getopt, getopt_long, optarg, opterr, optind, optopt,
+	   optreset, __check_rhosts_file, __rcmd_errstr.
+       95: Export shmat, shmctl, shmdt, shmget.
+       96: CW_GET_ERRNO_FROM_WINERROR addition to external.cc
+       97: Export sem_open, sem_close, sem_timedwait, sem_getvalue.
+       98: Export _tmpfile64.
+       99: CW_GET_POSIX_SECURITY_ATTRIBUTE addition to external.cc.
+      100: CW_GET_SHMLBA addition to external.cc.
+      101: Export err, errx, verr, verrx, warn, warnx, vwarn, vwarnx.
+      102: CW_GET_UID_FROM_SID and CW_GET_GID_FROM_SID addition to external.cc.
+      103: Export getprogname, setprogname.
+      104: Export msgctl, msgget, msgrcv, msgsnd, semctl, semget, semop.
+      105: Export sigwait.
+      106: Export flock.
+      107: Export fcntl64.
+      108: Remove unused (hopefully) reent_data export.
+      109: Oh well.  Someone uses reent_data.
+      110: Export clock_gettime, sigwaitinfo, timer_create, timer_delete,
+           timer_settime
+      111: Export sigqueue, sighold.
+      112: Redefine some mtget fields.
+      113: Again redefine some mtget fields.  Use mt_fileno and mt_blkno as
+           on Linux.
+      114: Export rand_r, ttyname_r.
+      115: Export flockfile, ftrylockfile, funlockfile, getgrgid_r, getgrnam_r,
+           getlogin_r.
+      116: Export atoll.
      */
 
      /* Note that we forgot to bump the api for ualarm, strtoll, strtoull */
 
 #define CYGWIN_VERSION_API_MAJOR 0
-#define CYGWIN_VERSION_API_MINOR 78
+#define CYGWIN_VERSION_API_MINOR 116
 
      /* There is also a compatibity version number associated with the
 	shared memory regions.  It is incremented when incompatible
 	changes are made to the shared memory region *or* to any named
 	shared mutexes, semaphores, etc.   The arbitrary starting
-	version was 0 (cygwin release 98r2). */
+	version was 0 (cygwin release 98r2).
+	Bump to 4 since this hasn't been rigorously updated in a
+	while.  */
 
-#define CYGWIN_VERSION_SHARED_DATA 3
+#define CYGWIN_VERSION_SHARED_DATA 4
 
      /* An identifier used in the names used to create shared objects.
 	The full names include the CYGWIN_VERSION_SHARED_DATA version
