@@ -147,7 +147,7 @@ strcasestr (const char *searchee, const char *lookfor)
 int __stdcall
 check_null_str (const char *name)
 {
-  if (name && !IsBadStringPtr (name, MAX_PATH))
+  if (name && !IsBadStringPtr (name, CYG_MAX_PATH))
     return 0;
 
   return EFAULT;
@@ -156,7 +156,7 @@ check_null_str (const char *name)
 int __stdcall
 check_null_empty_str (const char *name)
 {
-  if (name && !IsBadStringPtr (name, MAX_PATH))
+  if (name && !IsBadStringPtr (name, CYG_MAX_PATH))
     return !*name ? ENOENT : 0;
 
   return EFAULT;
@@ -198,6 +198,14 @@ __check_null_invalid_struct_errno (void *s, unsigned sz)
   return err;
 }
 
+int __stdcall
+__check_invalid_read_ptr (const void *s, unsigned sz)
+{
+  if (s && !IsBadReadPtr (s, sz))
+    return 0;
+  return EFAULT;
+}
+
 int __stdcall
 __check_invalid_read_ptr_errno (const void *s, unsigned sz)
 {
@@ -206,6 +214,19 @@ __check_invalid_read_ptr_errno (const void *s, unsigned sz)
   return set_errno (EFAULT);
 }
 
+int __stdcall
+check_invalid_virtual_addr (const void *s, unsigned sz)
+{
+  MEMORY_BASIC_INFORMATION mbuf;
+  const void *end;
+
+  for (end = (char *) s + sz; s < end;
+       s = (char *) mbuf.BaseAddress + mbuf.RegionSize)
+    if (!VirtualQuery (s, &mbuf, sizeof mbuf))
+      return EINVAL;
+  return 0;
+}
+
 ssize_t
 check_iovec_for_read (const struct iovec *iov, int iovcnt)
 {
@@ -294,7 +315,7 @@ sys_mbstowcs (WCHAR *tgt, const char *src, int len)
   return MultiByteToWideChar (get_cp (), 0, src, -1, tgt, len);
 }
 
-int __stdcall
+extern "C" int
 low_priority_sleep (DWORD secs)
 {
   HANDLE thisthread = GetCurrentThread ();
@@ -310,7 +331,7 @@ low_priority_sleep (DWORD secs)
 
   if (!secs && wincap.has_switch_to_thread ())
     {
-      for (int i = 0; i < 10; i++)
+      for (int i = 0; i < 3; i++)
 	SwitchToThread ();
     }
   else
@@ -326,7 +347,7 @@ low_priority_sleep (DWORD secs)
 	SetThreadPriority (thisthread, new_prio);
       Sleep (secs);
 
-      if (!staylow || curr_prio == new_prio)
+      if (!staylow && curr_prio != new_prio)
 	SetThreadPriority (thisthread, curr_prio);
     }
 
