@@ -1,6 +1,6 @@
 /* mmap.cc
 
-   Copyright 1996, 1997, 1998, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -14,8 +14,8 @@ details. */
 #include <stddef.h>
 #include <sys/mman.h>
 #include "security.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygerrno.h"
 #include "cygheap.h"
@@ -39,9 +39,9 @@ static fhandler_disk_file fh_paging_file;
    for duplicating all mmaps after fork() since mmaps are not propagated
    to child processes by Windows.  All information must be duplicated
    by hand, see fixup_mmaps_after_fork().
-  
+
    The class structure:
-  
+
    One member of class map per process, global variable mmapped_areas.
    Contains a dynamic class list array.  Each list entry represents all
    mapping to a file, keyed by file descriptor and file name hash.
@@ -56,31 +56,31 @@ class mmap_record
   private:
     int fdesc_;
     HANDLE mapping_handle_;
-    int devtype_;
     DWORD access_mode_;
     _off64_t offset_;
     DWORD size_to_map_;
     caddr_t base_address_;
     DWORD *page_map_;
+    device dev;
 
   public:
     mmap_record (int fd, HANDLE h, DWORD ac, _off64_t o, DWORD s, caddr_t b) :
        fdesc_ (fd),
        mapping_handle_ (h),
-       devtype_ (0),
        access_mode_ (ac),
        offset_ (o),
        size_to_map_ (s),
        base_address_ (b),
        page_map_ (NULL)
       {
+	dev.devn = 0;
 	if (fd >= 0 && !cygheap->fdtab.not_open (fd))
-	  devtype_ = cygheap->fdtab[fd]->get_device ();
+	  dev = cygheap->fdtab[fd]->dev ();
       }
 
     int get_fd () const { return fdesc_; }
     HANDLE get_handle () const { return mapping_handle_; }
-    DWORD get_device () const { return devtype_; }
+    device& get_device () { return dev; }
     DWORD get_access () const { return access_mode_; }
     DWORD get_offset () const { return offset_; }
     DWORD get_size () const { return size_to_map_; }
@@ -92,7 +92,7 @@ class mmap_record
 
     DWORD find_unused_pages (DWORD pages);
     _off64_t map_pages (_off64_t off, DWORD len);
-    BOOL unmap_pages (caddr_t addr, DWORD len);
+    bool unmap_pages (caddr_t addr, DWORD len);
     int access (caddr_t address);
 
     fhandler_base *alloc_fh ();
@@ -230,7 +230,7 @@ mmap_record::map_pages (_off64_t off, DWORD len)
   return off * getpagesize ();
 }
 
-BOOL
+bool
 mmap_record::unmap_pages (caddr_t addr, DWORD len)
 {
   DWORD old_prot;
@@ -248,8 +248,8 @@ mmap_record::unmap_pages (caddr_t addr, DWORD len)
      the whole chunk. */
   for (len = MAPSIZE (PAGE_CNT (size_to_map_)); len > 0; )
     if (page_map_[--len])
-      return FALSE;
-  return TRUE;
+      return false;
+  return true;
 }
 
 void
@@ -300,7 +300,7 @@ mmap_record::alloc_fh ()
      the call to fork(). This requires creating a fhandler
      of the correct type to be sure to call the method of the
      correct class. */
-  return cygheap->fdtab.build_fhandler (-1, get_device ());
+  return build_fh_dev (get_device ());
 }
 
 void
@@ -317,13 +317,13 @@ list::add_record (mmap_record r, _off64_t off, DWORD len)
     {
       mmap_record *new_recs;
       if (maxrecs == 0)
-        new_recs = (mmap_record *)
+	new_recs = (mmap_record *)
 			cmalloc (HEAP_MMAP, 5 * sizeof (mmap_record));
       else
 	new_recs = (mmap_record *)
 			crealloc (recs, (maxrecs + 5) * sizeof (mmap_record));
       if (!new_recs)
-        return NULL;
+	return NULL;
       maxrecs += 5;
       recs = new_recs;
     }
@@ -424,11 +424,11 @@ map::add_list (int fd)
     {
       list *new_lists;
       if (maxlists == 0)
-        new_lists = (list *) cmalloc (HEAP_MMAP, 5 * sizeof (list));
+	new_lists = (list *) cmalloc (HEAP_MMAP, 5 * sizeof (list));
       else
 	new_lists = (list *) crealloc (lists, (maxlists + 5) * sizeof (list));
       if (!new_lists)
-        return NULL;
+	return NULL;
       maxlists += 5;
       lists = new_lists;
     }
@@ -443,7 +443,7 @@ map::del_list (int i)
     {
       lists[i].free_recs ();
       for (; i < nlists - 1; i++)
-        lists[i] = lists[i + 1];
+	lists[i] = lists[i + 1];
       nlists--;
     }
 }
@@ -454,13 +454,7 @@ mmap64 (void *addr, size_t len, int prot, int flags, int fd, _off64_t off)
   syscall_printf ("addr %x, len %u, prot %x, flags %x, fd %d, off %D",
 		  addr, len, prot, flags, fd, off);
 
-  static DWORD granularity;
-  if (!granularity)
-    {
-      SYSTEM_INFO si;
-      GetSystemInfo (&si);
-      granularity = si.dwAllocationGranularity;
-    }
+  static DWORD granularity = getshmlba ();
 
   /* Error conditions according to SUSv2 */
   if (off % getpagesize ()
@@ -509,7 +503,7 @@ mmap64 (void *addr, size_t len, int prot, int flags, int fd, _off64_t off)
 	  return MAP_FAILED;
 	}
       fh = cfd;
-      if (fh->get_device () == FH_DISK)
+      if (fh->get_device () == FH_FS)
 	{
 	  DWORD high;
 	  DWORD low = GetFileSize (fh->get_handle (), &high);
@@ -646,7 +640,7 @@ munmap (void *addr, size_t len)
 
   /* Error conditions according to SUSv3 */
   if (!addr || ((DWORD)addr % getpagesize ()) || !len
-      || IsBadReadPtr (addr, len))
+      || check_invalid_virtual_addr (addr, len))
     {
       set_errno (EINVAL);
       syscall_printf ("-1 = munmap(): Invalid parameters");
@@ -673,17 +667,20 @@ munmap (void *addr, size_t len)
       DWORD u_len;
 
       while ((record_idx = map_list->search_record((caddr_t)addr, len, u_addr,
-      						   u_len, record_idx)) >= 0)
+						   u_len, record_idx)) >= 0)
 	{
 	  mmap_record *rec = map_list->get_record (record_idx);
 	  if (rec->unmap_pages (u_addr, u_len))
 	    {
-	      /* The whole record has been unmapped, so... */
+	      /* The whole record has been unmapped, so we now actually
+		 unmap it from the system in full length... */
 	      fhandler_base *fh = rec->alloc_fh ();
-	      fh->munmap (rec->get_handle (), (caddr_t)addr, len);
+	      fh->munmap (rec->get_handle (),
+			  rec->get_address (),
+			  rec->get_size ());
 	      rec->free_fh (fh);
 
-	      /* ...delete the record. */
+	      /* ...and delete the record. */
 	      if (map_list->del_record (record_idx--))
 		{
 		  /* Yay, the last record has been removed from the list,
@@ -737,7 +734,7 @@ msync (void *addr, size_t len, int flags)
     {
       mmap_record *rec;
       for (int record_idx = 0;
-      	   (rec = map_list->get_record (record_idx));
+	   (rec = map_list->get_record (record_idx));
 	   ++record_idx)
 	{
 	  if (rec->access ((caddr_t)addr))
@@ -748,7 +745,7 @@ msync (void *addr, size_t len, int flags)
 		  goto invalid_address_range;
 	      fhandler_base *fh = rec->alloc_fh ();
 	      int ret = fh->msync (rec->get_handle (), (caddr_t)addr, len,
-	      			   flags);
+				   flags);
 	      rec->free_fh (fh);
 
 	      if (ret)
@@ -862,7 +859,7 @@ fhandler_base::msync (HANDLE h, caddr_t addr, size_t len, int flags)
   return -1;
 }
 
-BOOL
+bool
 fhandler_base::fixup_mmap_after_fork (HANDLE h, DWORD access, DWORD offset,
 				      DWORD size, void *address)
 {
@@ -902,7 +899,7 @@ fhandler_disk_file::mmap (caddr_t *addr, size_t len, DWORD access,
       /* Grrr, the whole stuff is just needed to try to get a reliable
 	 mapping of the same file. Even that uprising isn't bullet
 	 proof but it does it's best... */
-      char namebuf[MAX_PATH];
+      char namebuf[CYG_MAX_PATH];
       cygwin_conv_to_full_posix_path (get_name (), namebuf);
       for (int i = strlen (namebuf) - 1; i >= 0; --i)
 	namebuf[i] = cyg_tolower (namebuf [i]);
@@ -922,7 +919,7 @@ fhandler_disk_file::mmap (caddr_t *addr, size_t len, DWORD access,
       return INVALID_HANDLE_VALUE;
     }
 
-  DWORD high = off >> 32, low = off & 0xffffffff;
+  DWORD high = off >> 32, low = off & UINT32_MAX;
   void *base = NULL;
   /* If a non-zero address is given, try mapping using the given address first.
      If it fails and flags is not MAP_FIXED, try again with NULL address. */
@@ -971,7 +968,7 @@ fhandler_disk_file::msync (HANDLE h, caddr_t addr, size_t len, int flags)
   return 0;
 }
 
-BOOL
+bool
 fhandler_disk_file::fixup_mmap_after_fork (HANDLE h, DWORD access, DWORD offset,
 					   DWORD size, void *address)
 {
@@ -1014,7 +1011,7 @@ fixup_mmaps_after_fork (HANDLE parent)
     {
       mmap_record *rec;
       for (int record_idx = 0;
-      	   (rec = map_list->get_record (record_idx));
+	   (rec = map_list->get_record (record_idx));
 	   ++record_idx)
 	{
 
@@ -1023,7 +1020,7 @@ fixup_mmaps_after_fork (HANDLE parent)
 	      rec->get_offset (), rec->get_size (), rec->get_address ());
 
 	  fhandler_base *fh = rec->alloc_fh ();
-	  BOOL ret = fh->fixup_mmap_after_fork (rec->get_handle (),
+	  bool ret = fh->fixup_mmap_after_fork (rec->get_handle (),
 						rec->get_access (),
 						rec->get_offset (),
 						rec->get_size (),
