@@ -35,7 +35,6 @@ details. */
 #include "sigproc.h"
 #include "pinfo.h"
 #include "registry.h"
-#include "wsock_event.h"
 #include "cygtls.h"
 
 extern "C"
@@ -50,58 +49,6 @@ extern "C"
   int sscanf (const char *, const char *, ...);
 }				/* End of "C" section */
 
-LPWSAOVERLAPPED
-wsock_event::prepare ()
-{
-  LPWSAOVERLAPPED ret = NULL;
-
-  SetLastError (0);
-  if ((event = WSACreateEvent ()) != WSA_INVALID_EVENT)
-    {
-      memset (&ovr, 0, sizeof ovr);
-      ovr.hEvent = event;
-      ret = &ovr;
-    }
-  else if (GetLastError () == ERROR_PROC_NOT_FOUND) /* winsock2 not available */
-    WSASetLastError (0);
-
-  debug_printf ("%d = wsock_event::prepare ()", ret);
-  return ret;
-}
-
-int
-wsock_event::wait (int socket, LPDWORD flags)
-{
-  int ret = -1;
-  WSAEVENT ev[2] = { event, signal_arrived };
-
-  switch (WSAWaitForMultipleEvents (2, ev, FALSE, WSA_INFINITE, FALSE))
-    {
-      case WSA_WAIT_EVENT_0:
-	DWORD len;
-	if (WSAGetOverlappedResult (socket, &ovr, &len, FALSE, flags))
-	  ret = (int) len;
-	break;
-      case WSA_WAIT_EVENT_0 + 1:
-	if (!CancelIo ((HANDLE) socket))
-	  {
-	    debug_printf ("CancelIo() %E, fallback to blocking io");
-	    WSAGetOverlappedResult (socket, &ovr, &len, TRUE, flags);
-	  }
-	else
-	  WSASetLastError (WSAEINTR);
-	break;
-      case WSA_WAIT_FAILED:
-	break;
-      default:			/* Should be impossible. *LOL* */
-	WSASetLastError (WSAEFAULT);
-	break;
-    }
-  WSACloseEvent (event);
-  event = NULL;
-  return ret;
-}
-
 WSADATA wsadata;
 
 static fhandler_socket *
@@ -633,7 +580,7 @@ fdsock (cygheap_fdmanip& fd, const device *dev, SOCKET soc)
     return false;
   fd->set_io_handle ((HANDLE) soc);
   fd->set_flags (O_RDWR | O_BINARY);
-  fd->set_r_no_interrupt (winsock2_active);
+  fd->uninterruptible_io (winsock2_active);
   cygheap->fdtab.inc_need_fixup_before ();
   debug_printf ("fd %d, name '%s', soc %p", (int) fd, dev->name, soc);
   return true;
@@ -1957,7 +1904,7 @@ cygwin_rcmd (char **ahost, unsigned short inport, char *locuser,
 
       if (res_fd >= 0 && fdsock (res_fd, tcp_dev, res))
 	{
-	  ((fhandler_socket *) res_fd)->set_connect_state (CONNECTED);
+	  ((fhandler_socket *) res_fd)->connect_state (connected);
 	  res = res_fd;
 	}
       else
@@ -1974,7 +1921,7 @@ cygwin_rcmd (char **ahost, unsigned short inport, char *locuser,
 	  if (newfd >= 0 && fdsock (newfd, tcp_dev, fd2s))
 	    {
 	      *fd2p = newfd;
-	      ((fhandler_socket *) fd2p)->set_connect_state (CONNECTED);
+	      ((fhandler_socket *) fd2p)->connect_state (connected);
 	    }
 	  else
 	    {
@@ -2037,7 +1984,7 @@ cygwin_rexec (char **ahost, unsigned short inport, char *locuser,
 
       if (res_fd >= 0 && fdsock (res_fd, tcp_dev, res))
 	{
-	  ((fhandler_socket *) res_fd)->set_connect_state (CONNECTED);
+	  ((fhandler_socket *) res_fd)->connect_state (connected);
 	  res = res_fd;
 	}
       else
@@ -2053,7 +2000,7 @@ cygwin_rexec (char **ahost, unsigned short inport, char *locuser,
 
 	  if (newfd >= 0 && fdsock (newfd, tcp_dev, fd2s))
 	    {
-	      ((fhandler_socket *) fd2p)->set_connect_state (CONNECTED);
+	      ((fhandler_socket *) fd2p)->connect_state (connected);
 	      *fd2p = newfd;
 	    }
 	  else
@@ -2227,7 +2174,7 @@ socketpair (int family, int type, int protocol, int *sb)
 	((fhandler_socket *) sb0)->set_sun_path ("");
 	((fhandler_socket *) sb0)->set_addr_family (family);
 	((fhandler_socket *) sb0)->set_socket_type (type);
-	((fhandler_socket *) sb0)->set_connect_state (CONNECTED);
+	((fhandler_socket *) sb0)->connect_state (connected);
 
 	cygheap_fdnew sb1 (sb0, false);
 
@@ -2236,7 +2183,7 @@ socketpair (int family, int type, int protocol, int *sb)
 	    ((fhandler_socket *) sb1)->set_sun_path ("");
 	    ((fhandler_socket *) sb1)->set_addr_family (family);
 	    ((fhandler_socket *) sb1)->set_socket_type (type);
-	    ((fhandler_socket *) sb1)->set_connect_state (CONNECTED);
+	    ((fhandler_socket *) sb1)->connect_state (connected);
 
 	    sb[0] = sb0;
 	    sb[1] = sb1;
