@@ -1,6 +1,6 @@
 /* ntdll.h.  Contains ntdll specific stuff not defined elsewhere.
 
-   Copyright 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
    This file is part of Cygwin.
 
@@ -9,6 +9,7 @@
    details. */
 
 #define STATUS_INFO_LENGTH_MISMATCH ((NTSTATUS) 0xc0000004)
+#define STATUS_BUFFER_TOO_SMALL     ((NTSTATUS) 0xc0000023)
 #define PDI_MODULES 0x01
 #define PDI_HEAPS 0x04
 #define LDRP_IMAGE_DLL 0x00000004
@@ -31,6 +32,7 @@ typedef enum _SYSTEM_INFORMATION_CLASS
   SystemTimeOfDayInformation = 3,
   SystemProcessesAndThreadsInformation = 5,
   SystemProcessorTimes = 8,
+  SystemPagefileInformation = 18,
   /* There are a lot more of these... */
 } SYSTEM_INFORMATION_CLASS;
 
@@ -49,6 +51,15 @@ typedef struct _SYSTEM_BASIC_INFORMATION
   UCHAR NumberProcessors;
 } SYSTEM_BASIC_INFORMATION, *PSYSTEM_BASIC_INFORMATION;
 
+typedef struct _SYSTEM_PAGEFILE_INFORMATION
+{
+  ULONG NextEntryOffset;
+  ULONG CurrentSize;
+  ULONG TotalUsed;
+  ULONG PeakUsed;
+  UNICODE_STRING FileName;
+} SYSTEM_PAGEFILE_INFORMATION, *PSYSTEM_PAGEFILE_INFORMATION;
+
 typedef struct __attribute__ ((aligned (8))) _SYSTEM_PROCESSOR_TIMES
 {
   LARGE_INTEGER IdleTime;
@@ -347,11 +358,25 @@ typedef struct _FILE_NAME_INFORMATION
   WCHAR FileName[MAX_PATH + 100];
 } FILE_NAME_INFORMATION;
 
+typedef struct _FILE_COMPRESSION_INFORMATION
+{
+  LARGE_INTEGER CompressedSize;
+  USHORT CompressionFormat;
+  UCHAR	CompressionUnitShift;
+  UCHAR Unknown;
+  UCHAR ClusterSizeShift;
+} FILE_COMPRESSION_INFORMATION, *PFILE_COMPRESSION_INFORMATION;
+
+typedef enum _FILE_INFORMATION_CLASS
+{
+  FileCompressionInformation = 28
+} FILE_INFORMATION_CLASS, *PFILE_INFORMATION_CLASS;
+
 typedef enum _OBJECT_INFORMATION_CLASS
 {
-   ObjectBasicInformation = 0,
-   ObjectNameInformation = 1,
-   ObjectHandleInformation = 4
+  ObjectBasicInformation = 0,
+  ObjectNameInformation = 1,
+  ObjectHandleInformation = 4
    // and many more
 } OBJECT_INFORMATION_CLASS;
 
@@ -364,6 +389,9 @@ typedef struct _OBJECT_NAME_INFORMATION
    standard Win32 header.  */
 extern "C"
 {
+  NTSTATUS NTAPI NtCreateFile (PHANDLE, ACCESS_MASK, POBJECT_ATTRIBUTES,
+			     PIO_STATUS_BLOCK, PLARGE_INTEGER, ULONG, ULONG,
+			     ULONG, ULONG, PVOID, ULONG);
   NTSTATUS NTAPI NtCreateToken (PHANDLE, ACCESS_MASK, POBJECT_ATTRIBUTES,
 				TOKEN_TYPE, PLUID, PLARGE_INTEGER, PTOKEN_USER,
 				PTOKEN_GROUPS, PTOKEN_PRIVILEGES, PTOKEN_OWNER,
@@ -383,8 +411,12 @@ extern "C"
 				ULONG, ULONG *);
   NTSTATUS NTAPI NtQuerySystemInformation (SYSTEM_INFORMATION_CLASS,
 					   PVOID, ULONG, PULONG);
+  NTSTATUS NTAPI NtQuerySecurityObject (HANDLE, SECURITY_INFORMATION,
+  					PSECURITY_DESCRIPTOR, ULONG, PULONG);
   NTSTATUS NTAPI NtQueryVirtualMemory (HANDLE, PVOID, MEMORY_INFORMATION_CLASS,
 				       PVOID, ULONG, PULONG);
+  NTSTATUS NTAPI NtSetSecurityObject (HANDLE, SECURITY_INFORMATION,
+				      PSECURITY_DESCRIPTOR);
   NTSTATUS NTAPI NtUnmapViewOfSection (HANDLE, PVOID);
   VOID NTAPI RtlInitUnicodeString (PUNICODE_STRING, PCWSTR);
   ULONG NTAPI RtlNtStatusToDosError (NTSTATUS);
