@@ -14,8 +14,8 @@ details. */
 #include <stdio.h>
 #include "cygerrno.h"
 #include "security.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "pinfo.h"
 #include "cygheap.h"
@@ -26,12 +26,8 @@ details. */
    on the first call that needs information from it. */
 
 passwd *passwd_buf;
-/* FIXME: This really should use a constructor, but they are slow */
 static pwdgrp pr (passwd_buf);
 
-/* Position in the passwd cache */
-#define pw_pos  _reent_winsup ()->_pw_pos
-
 /* Parse /etc/passwd line into passwd structure. */
 bool
 pwdgrp::parse_passwd ()
@@ -144,7 +140,7 @@ internal_getpwnam (const char *name, bool check)
 extern "C" struct passwd *
 getpwuid32 (__uid32_t uid)
 {
-  struct passwd *temppw = internal_getpwuid (uid, TRUE);
+  struct passwd *temppw = internal_getpwuid (uid, true);
   pthread_testcancel ();
   return temppw;
 }
@@ -163,7 +159,7 @@ getpwuid_r32 (__uid32_t uid, struct passwd *pwd, char *buffer, size_t bufsize, s
   if (!pwd || !buffer)
     return ERANGE;
 
-  struct passwd *temppw = internal_getpwuid (uid, TRUE);
+  struct passwd *temppw = internal_getpwuid (uid, true);
   pthread_testcancel ();
   if (!temppw)
     return 0;
@@ -202,7 +198,7 @@ getpwuid_r (__uid16_t uid, struct passwd *pwd, char *buffer, size_t bufsize, str
 extern "C" struct passwd *
 getpwnam (const char *name)
 {
-  struct passwd *temppw = internal_getpwnam (name, TRUE);
+  struct passwd *temppw = internal_getpwnam (name, true);
   pthread_testcancel ();
   return temppw;
 }
@@ -220,7 +216,7 @@ getpwnam_r (const char *nam, struct passwd *pwd, char *buffer, size_t bufsize, s
   if (!pwd || !buffer || !nam)
     return ERANGE;
 
-  struct passwd *temppw = internal_getpwnam (nam, TRUE);
+  struct passwd *temppw = internal_getpwnam (nam, true);
   pthread_testcancel ();
 
   if (!temppw)
@@ -254,10 +250,10 @@ getpwnam_r (const char *nam, struct passwd *pwd, char *buffer, size_t bufsize, s
 extern "C" struct passwd *
 getpwent (void)
 {
-  if (pw_pos == 0)
+  if (_my_tls.locals.pw_pos == 0)
     pr.refresh (true);
-  if (pw_pos < pr.curr_lines)
-    return passwd_buf + pw_pos++;
+  if (_my_tls.locals.pw_pos < pr.curr_lines)
+    return passwd_buf + _my_tls.locals.pw_pos++;
 
   return NULL;
 }
@@ -271,13 +267,13 @@ getpwduid (__uid16_t)
 extern "C" void
 setpwent (void)
 {
-  pw_pos = 0;
+  _my_tls.locals.pw_pos = 0;
 }
 
 extern "C" void
 endpwent (void)
 {
-  pw_pos = 0;
+  _my_tls.locals.pw_pos = 0;
 }
 
 extern "C" int
@@ -289,7 +285,7 @@ setpassent ()
 extern "C" char *
 getpass (const char * prompt)
 {
-  char *pass=_reent_winsup ()->_pass;
+  char *pass = _my_tls.locals.pass;
   struct termios ti, newti;
 
   cygheap_fdget fhstdin (0);
