@@ -165,6 +165,16 @@ str2buf2uni (UNICODE_STRING &tgt, WCHAR *buf, const char *srcstr)
   sys_mbstowcs (buf, srcstr, tgt.MaximumLength);
 }
 
+void
+str2buf2uni_cat (UNICODE_STRING &tgt, const char *srcstr)
+{
+  DWORD len = strlen (srcstr) * sizeof (WCHAR);
+  sys_mbstowcs (tgt.Buffer + tgt.Length / sizeof (WCHAR), srcstr,
+		len + tgt.MaximumLength);
+  tgt.Length += len;
+  tgt.MaximumLength += len;
+}
+
 #if 0				/* unused */
 static void
 lsa2wchar (WCHAR *tgt, LSA_UNICODE_STRING &src, int size)
@@ -1117,89 +1127,36 @@ read_sd (const char *file, security_descriptor &sd)
 }
 
 LONG
-write_sd (const char *file, security_descriptor &sd)
+write_sd (HANDLE fh, const char *file, security_descriptor &sd)
 {
-  BOOL dummy;
-  cygpsid owner;
-
-  if (!GetSecurityDescriptorOwner (sd, (PSID *) &owner, &dummy))
+  NTSTATUS ret = STATUS_SUCCESS;
+  int retry = 0;
+  int res = -1;
+  for (; retry < 2; ++retry)
     {
-      __seterrno ();
-      return -1;
+      if (retry && (fh = CreateFile (file, WRITE_OWNER | WRITE_DAC,
+				     FILE_SHARE_READ | FILE_SHARE_WRITE,
+				     &sec_none_nih, OPEN_EXISTING,
+				     FILE_ATTRIBUTE_NORMAL
+				     | FILE_FLAG_BACKUP_SEMANTICS,
+				     NULL)) == INVALID_HANDLE_VALUE)
+	break;
+      if (fh && (ret = NtSetSecurityObject (fh,
+					    DACL_SECURITY_INFORMATION 
+					    | GROUP_SECURITY_INFORMATION
+					    | OWNER_SECURITY_INFORMATION,
+					    sd)) == STATUS_SUCCESS)
+	break;
     }
-  /* Try turning privilege on, may not have WRITE_OWNER or WRITE_DAC access.
-     Must have privilege to set different owner, else BackupWrite misbehaves */
-  static int NO_COPY saved_res; /* 0: never, 1: failed, 2 & 3: OK */
-  int res;
-  if (!saved_res || cygheap->user.issetuid ())
-    {
-      res = 2 + set_process_privilege (SE_RESTORE_NAME, true,
-				       cygheap->user.issetuid ());
-      if (!cygheap->user.issetuid ())
-	saved_res = res;
-    }
-  else
-    res = saved_res;
-  if (res == 1 && owner != cygheap->user.sid ())
-    return -1;
-
-  HANDLE fh;
-  fh = CreateFile (file,
-		   WRITE_OWNER | WRITE_DAC,
-		   FILE_SHARE_READ | FILE_SHARE_WRITE,
-		   &sec_none_nih,
-		   OPEN_EXISTING,
-		   FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS,
-		   NULL);
-
-  if (fh == INVALID_HANDLE_VALUE)
-    {
-      __seterrno ();
-      return -1;
-    }
-
-  LPVOID context = NULL;
-  DWORD bytes_written = 0;
-  WIN32_STREAM_ID header;
-
-  memset (&header, 0, sizeof (header));
-  /* write new security info header */
-  header.dwStreamId = BACKUP_SECURITY_DATA;
-  header.dwStreamAttributes = STREAM_CONTAINS_SECURITY;
-  header.Size.HighPart = 0;
-  header.Size.LowPart = sd.size ();
-  header.dwStreamNameSize = 0;
-  if (!BackupWrite (fh, (LPBYTE) &header,
-		    3 * sizeof (DWORD) + sizeof (LARGE_INTEGER),
-		    &bytes_written, FALSE, TRUE, &context))
-    {
-      __seterrno ();
-      CloseHandle (fh);
-      return -1;
-    }
-
-  /* write new security descriptor */
-  if (!BackupWrite (fh, (LPBYTE) (PSECURITY_DESCRIPTOR) sd,
-		    header.Size.LowPart + header.dwStreamNameSize,
-		    &bytes_written, FALSE, TRUE, &context))
-    {
-      /* Samba returns ERROR_NOT_SUPPORTED.
-	 FAT returns ERROR_INVALID_SECURITY_DESCR.
-	 This shouldn't return as error, but better be ignored. */
-      DWORD ret = GetLastError ();
-      if (ret != ERROR_NOT_SUPPORTED && ret != ERROR_INVALID_SECURITY_DESCR)
-	{
-	  __seterrno ();
-	  BackupWrite (fh, NULL, 0, &bytes_written, TRUE, TRUE, &context);
-	  CloseHandle (fh);
-	  return -1;
-	}
-    }
-
-  /* terminate the restore process */
-  BackupWrite (fh, NULL, 0, &bytes_written, TRUE, TRUE, &context);
-  CloseHandle (fh);
-  return 0;
+  if (retry && fh != INVALID_HANDLE_VALUE)
+    CloseHandle (fh);
+  if (fh == INVALID_HANDLE_VALUE)	/* CreateFile failed */
+    __seterrno ();
+  else if (ret != STATUS_SUCCESS)	/* NtSetSecurityObject failed */
+    __seterrno_from_win_error (RtlNtStatusToDosError (ret));
+  else					/* Everything's fine. */
+    res = 0;
+  return res;
 }
 
 static void
@@ -1368,7 +1325,69 @@ get_nt_attribute (const char *file, mode_t *attribute,
 }
 
 int
-get_file_attribute (int use_ntsec, const char *file,
+get_nt_object_security (HANDLE handle, SE_OBJECT_TYPE object_type,
+			security_descriptor &sd_ret)
+{
+  NTSTATUS ret;
+  ULONG len = 0;
+  ret = NtQuerySecurityObject (handle,
+			       DACL_SECURITY_INFORMATION
+			       | GROUP_SECURITY_INFORMATION
+			       | OWNER_SECURITY_INFORMATION,
+			       sd_ret, len, &len);
+  if (ret == STATUS_BUFFER_TOO_SMALL)
+    {
+      if (!sd_ret.malloc (len))
+	set_errno (ENOMEM);
+      else
+	ret = NtQuerySecurityObject (handle,
+				     DACL_SECURITY_INFORMATION
+				     | GROUP_SECURITY_INFORMATION
+				     | OWNER_SECURITY_INFORMATION,
+				     sd_ret, len, &len);
+    }
+  if (ret != STATUS_SUCCESS)
+    {
+      __seterrno_from_win_error (RtlNtStatusToDosError (ret));
+      return -1;
+    }
+  return 0;
+}
+
+static int
+get_nt_object_attribute (HANDLE handle, SE_OBJECT_TYPE object_type,
+			 mode_t *attribute, __uid32_t *uidret,
+			 __gid32_t *gidret)
+{
+  security_descriptor sd;
+  PSECURITY_DESCRIPTOR psd = NULL;
+
+  if (get_nt_object_security (handle, object_type, sd))
+    {
+      if (object_type == SE_FILE_OBJECT)
+	return -1;
+    }
+  else
+    psd = sd;
+  get_info_from_sd (psd, attribute, uidret, gidret);
+  return 0;
+}
+
+int
+get_object_attribute (HANDLE handle, SE_OBJECT_TYPE object_type,
+		      mode_t *attribute, __uid32_t *uidret, __gid32_t *gidret)
+{
+  if (allow_ntsec && wincap.has_security ())
+    {
+      get_nt_object_attribute (handle, object_type, attribute, uidret, gidret);
+      return 0;
+    }
+  /* The entries are already set to default values */
+  return -1;
+}
+
+int
+get_file_attribute (int use_ntsec, HANDLE handle, const char *file,
 		    mode_t *attribute, __uid32_t *uidret, __gid32_t *gidret)
 {
   int res;
@@ -1376,7 +1395,9 @@ get_file_attribute (int use_ntsec, const char *file,
 
   if (use_ntsec && allow_ntsec && wincap.has_security ())
     {
-      get_nt_attribute (file, attribute, uidret, gidret);
+      if (!handle || get_nt_object_attribute (handle, SE_FILE_OBJECT,
+					      attribute, uidret, gidret))
+	get_nt_attribute (file, attribute, uidret, gidret);
       return 0;
     }
 
@@ -1400,63 +1421,6 @@ get_file_attribute (int use_ntsec, const char *file,
   return res > 0 ? 0 : -1;
 }
 
-static void
-get_nt_object_attribute (HANDLE handle, SE_OBJECT_TYPE object_type,
-			 mode_t *attribute, __uid32_t *uidret,
-			 __gid32_t *gidret)
-{
-  security_descriptor sd;
-  PSECURITY_DESCRIPTOR psd = NULL;
-  LONG ret;
-
-  if (object_type == SE_REGISTRY_KEY)
-    {
-      /* use different code for registry handles, for performance reasons */
-      DWORD len = 0;
-      if ((ret = RegGetKeySecurity ((HKEY) handle,
-				    DACL_SECURITY_INFORMATION
-				    | GROUP_SECURITY_INFORMATION
-				    | OWNER_SECURITY_INFORMATION,
-				    sd, &len)) != ERROR_INSUFFICIENT_BUFFER)
-	__seterrno_from_win_error (ret);
-      else if (!sd.malloc (len))
-	set_errno (ENOMEM);
-      else if ((ret = RegGetKeySecurity ((HKEY) handle,
-					 DACL_SECURITY_INFORMATION
-					 | GROUP_SECURITY_INFORMATION
-					 | OWNER_SECURITY_INFORMATION,
-					 sd, &len)) != ERROR_SUCCESS)
-	__seterrno_from_win_error (ret);
-      else
-        psd = sd;
-      get_info_from_sd (psd, attribute, uidret, gidret);
-    }
-  else if ((ret = GetSecurityInfo (handle, object_type,
-				   DACL_SECURITY_INFORMATION
-				   | GROUP_SECURITY_INFORMATION
-				   | OWNER_SECURITY_INFORMATION,
-				   NULL, NULL, NULL, NULL, &psd)))
-    __seterrno_from_win_error (ret);
-  else
-    {
-      get_info_from_sd (psd, attribute, uidret, gidret);
-      LocalFree (psd);
-    }
-}
-
-int
-get_object_attribute (HANDLE handle, SE_OBJECT_TYPE object_type,
-		      mode_t *attribute, __uid32_t *uidret, __gid32_t *gidret)
-{
-  if (allow_ntsec && wincap.has_security ())
-    {
-      get_nt_object_attribute (handle, object_type, attribute, uidret, gidret);
-      return 0;
-    }
-  /* The entries are already set to default values */
-  return -1;
-}
-
 bool
 add_access_allowed_ace (PACL acl, int offset, DWORD attributes,
 			PSID sid, size_t &len_add, DWORD inherit)
@@ -1808,16 +1772,16 @@ set_security_attribute (int attribute, PSECURITY_ATTRIBUTES psa,
 }
 
 static int
-set_nt_attribute (const char *file, __uid32_t uid, __gid32_t gid,
-		  int attribute)
+set_nt_attribute (HANDLE handle, const char *file,
+		  __uid32_t uid, __gid32_t gid, int attribute)
 {
   if (!wincap.has_security ())
     return 0;
 
   security_descriptor sd;
 
-  int ret;
-  if ((ret = read_sd (file, sd)) <= 0)
+  if ((!handle || get_nt_object_security (handle, SE_FILE_OBJECT, sd))
+      && read_sd (file, sd) <= 0)
     {
       debug_printf ("read_sd %E");
       return -1;
@@ -1826,17 +1790,17 @@ set_nt_attribute (const char *file, __uid32_t uid, __gid32_t gid,
   if (!alloc_sd (uid, gid, attribute, sd))
     return -1;
 
-  return write_sd (file, sd);
+  return write_sd (handle, file, sd);
 }
 
 int
-set_file_attribute (int use_ntsec, const char *file,
+set_file_attribute (bool use_ntsec, HANDLE handle, const char *file,
 		    __uid32_t uid, __gid32_t gid, int attribute)
 {
   int ret = 0;
 
   if (use_ntsec && allow_ntsec)
-    ret = set_nt_attribute (file, uid, gid, attribute);
+    ret = set_nt_attribute (handle, file, uid, gid, attribute);
   else if (allow_ntea && !NTWriteEA (file, ".UNIXATTR", (char *) &attribute,
 				     sizeof (attribute)))
     {
@@ -1849,9 +1813,10 @@ set_file_attribute (int use_ntsec, const char *file,
 }
 
 int
-set_file_attribute (int use_ntsec, const char *file, int attribute)
+set_file_attribute (bool use_ntsec, HANDLE handle, const char *file,
+		    int attribute)
 {
-  return set_file_attribute (use_ntsec, file,
+  return set_file_attribute (use_ntsec, handle, file,
 			     myself->uid, myself->gid, attribute);
 }
 
