@@ -124,7 +124,7 @@ fixup_shms_after_fork ()
   client_request_shm request (&parent);
   if (request.make_request () == -1 || request.retval () == -1)
     {
-      syscall_printf ("-1 [%d] = shmctl ()", request.error_code ());
+      syscall_printf ("-1 [%d] = fixup_shms_after_fork ()", request.error_code ());
       set_errno (request.error_code ());
       return 0;
     }
@@ -160,9 +160,26 @@ shmat (int shmid, const void *shmaddr, int shmflg)
     }
   if (!ssh_entry)
     {
-      /* Invalid shmid */
-      set_errno (EINVAL);
-      return NULL;
+      /* The shmid is unknown to this process so far.  Try to get it from
+         the server if it exists.  Use special internal call to shmget,
+	 which interprets the key as a shmid and only returns a valid
+	 shmid if one exists.  Since shmctl inserts a new entry for this
+	 shmid into ssh_list automatically, we just have to go through
+	 that list again.  If that still fails, well, bad luck. */
+      if (shmid && shmget ((key_t) shmid, 0, IPC_KEY_IS_SHMID) != -1)
+        {
+	  SLIST_FOREACH (ssh_entry, &ssh_list, ssh_next)
+	    {
+	      if (ssh_entry->shmid == shmid)
+		break;
+	    }
+	}
+      if (!ssh_entry)
+        {
+	  /* Invalid shmid */
+	  set_errno (EINVAL);
+	  return (void *) -1;
+	}
     }
   vm_object_t attach_va = NULL;
   if (shmaddr)
@@ -176,7 +193,7 @@ shmat (int shmid, const void *shmaddr, int shmflg)
       if (!attach_va || (vm_offset_t)attach_va % SHMLBA)
 	{
 	  set_errno (EINVAL);
-	  return NULL;
+	  return (void *) -1;
 	}
     }
   /* Try allocating memory before calling cygserver. */
@@ -184,7 +201,7 @@ shmat (int shmid, const void *shmaddr, int shmflg)
   if (!sph_entry)
     {
       set_errno (ENOMEM);
-      return NULL;
+      return (void *) -1;
     }
   DWORD access = (shmflg & SHM_RDONLY) ? FILE_MAP_READ : FILE_MAP_WRITE;
   vm_object_t ptr = MapViewOfFileEx(ssh_entry->hdl, access, 0, 0,
@@ -193,20 +210,20 @@ shmat (int shmid, const void *shmaddr, int shmflg)
     {
       __seterrno ();
       delete sph_entry;
-      return NULL;
+      return (void *) -1;
     }
   /* Use returned ptr address as is, so it's stored using the exact value
      in cygserver. */
   client_request_shm request (shmid, ptr, shmflg & ~SHM_RND);
   if (request.make_request () == -1 || request.ptrval () == NULL)
     {
-      syscall_printf ("-1 [%d] = shmctl ()", request.error_code ());
+      syscall_printf ("-1 [%d] = shmat ()", request.error_code ());
       UnmapViewOfFile (ptr);
       delete sph_entry;
       set_errno (request.error_code ());
       if (request.error_code () == ENOSYS)
         raise (SIGSYS);
-      return NULL;
+      return (void *) -1;
     }
   sph_entry->ptr = ptr;
   sph_entry->hdl = ssh_entry->hdl;
@@ -217,7 +234,7 @@ shmat (int shmid, const void *shmaddr, int shmflg)
 #else
   set_errno (ENOSYS);
   raise (SIGSYS);
-  return NULL;
+  return (void *) -1;
 #endif
 }
 
@@ -286,11 +303,11 @@ extern "C" int
 shmdt (const void *shmaddr)
 {
 #ifdef USE_SERVER
-  syscall_printf ("shmget (shmaddr = %p)", shmaddr);
+  syscall_printf ("shmdt (shmaddr = %p)", shmaddr);
   client_request_shm request (shmaddr);
   if (request.make_request () == -1 || request.retval () == -1)
     {
-      syscall_printf ("-1 [%d] = shmctl ()", request.error_code ());
+      syscall_printf ("-1 [%d] = shmdt ()", request.error_code ());
       set_errno (request.error_code ());
       if (request.error_code () == ENOSYS)
         raise (SIGSYS);
@@ -333,7 +350,7 @@ shmget (key_t key, size_t size, int shmflg)
   client_request_shm request (key, size, shmflg);
   if (request.make_request () == -1 || request.retval () == -1)
     {
-      syscall_printf ("-1 [%d] = shmctl ()", request.error_code ());
+      syscall_printf ("-1 [%d] = shmget ()", request.error_code ());
       delete ssh_new_entry;
       set_errno (request.error_code ());
       if (request.error_code () == ENOSYS)
