@@ -1,6 +1,6 @@
 /* winsup.h: main Cygwin header file.
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -8,6 +8,12 @@ This software is a copyrighted work licensed under the terms of the
 Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
+#ifdef DEBUGIT
+#define spf(a, b, c) small_printf (a, b, c)
+#else
+#define spf(a, b, c) do {} while (0)
+#endif
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -55,6 +61,23 @@ extern unsigned long cygwin_inet_addr (const char *cp);
 }
 #endif
 
+/* Note that MAX_PATH is defined in the windows headers */
+/* There is also PATH_MAX and MAXPATHLEN.
+   PATH_MAX is from Posix and does *not* include the trailing NUL.
+   MAXPATHLEN is from Unix.
+
+   Thou shalt use CYG_MAX_PATH throughout.  It avoids the NUL vs no-NUL
+   issue and is neither of the Unixy ones [so we can punt on which
+   one is the right one to use]. 
+   
+   Windows ANSI calls are limited to MAX_PATH in length. Cygwin calls that
+   thunk through to Windows Wide calls are limited to 32K. We define
+   CYG_MAX_PATH as a convenient, not to short, not too long 'happy medium'.
+   
+   */
+
+#define CYG_MAX_PATH (MAX_PATH)
+
 #ifdef __cplusplus
 
 extern const char case_folded_lower[];
@@ -108,23 +131,18 @@ extern int cygserver_running;
 
 #define TITLESIZE 1024
 
-/* status bit manipulation */
-#define __ISSETF(what, x, prefix) \
-  ((what)->status & prefix##_##x)
-#define __SETF(what, x, prefix) \
-  ((what)->status |= prefix##_##x)
-#define __CLEARF(what, x, prefix) \
-  ((what)->status &= ~prefix##_##x)
-#define __CONDSETF(n, what, x, prefix) \
-  ((n) ? __SETF (what, x, prefix) : __CLEARF (what, x, prefix))
-
 #include "debug.h"
 
 /* Events/mutexes */
-extern HANDLE title_mutex;
+extern HANDLE tty_mutex;
 
 /**************************** Convenience ******************************/
 
+/* Used to define status flag accessor methods */
+#define IMPLEMENT_STATUS_FLAG(type,flag) \
+  void flag (type val) { status.flag = (val); } \
+  type flag () const { return (type) status.flag; }
+
 /* Used when treating / and \ as equivalent. */
 #define isdirsep(ch) \
     ({ \
@@ -133,11 +151,11 @@ extern HANDLE title_mutex;
     })
 
 /* Convert a signal to a signal mask */
-#define SIGTOMASK(sig)	(1<<((sig) - signal_shift_subtract))
+#define SIGTOMASK(sig)	(1 << ((sig) - signal_shift_subtract))
 extern unsigned int signal_shift_subtract;
 
 #ifdef NEW_MACRO_VARARGS
-# define api_fatal(...) __api_fatal ("%P: *** " __VA_ARGS__)
+# define api_fatal(...) __api_fatal (__VA_ARGS__)
 #else
 # define api_fatal(fmt, args...) __api_fatal ("%P: *** " fmt,## args)
 #endif
@@ -162,7 +180,24 @@ extern "C" int dll_dllcrt0 (HMODULE, per_process *);
 extern "C" int dll_noncygwin_dllcrt0 (HMODULE, per_process *);
 
 /* exit the program */
-extern "C" void __stdcall do_exit (int) __attribute__ ((noreturn));
+
+enum exit_states
+  {
+    ES_NOT_EXITING = 0,
+    ES_EVENTS_TERMINATE,
+    ES_THREADTERM,
+    ES_SIGNAL,
+    ES_CLOSEALL,
+    ES_SIGPROCTERMINATE,
+    ES_TITLE,
+    ES_HUP_PGRP,
+    ES_HUP_SID,
+    ES_TTY_TERMINATE,
+    ES_FINAL
+  };
+
+extern exit_states exit_state;
+void __stdcall do_exit (int) __attribute__ ((regparm (1), noreturn));
 
 /* UID/GID */
 void uinfo_init (void);
@@ -180,10 +215,10 @@ void uinfo_init (void);
 void events_init (void);
 void events_terminate (void);
 
-void __stdcall close_all_files (void);
+void __stdcall close_all_files ();
 
-/* Invisible window initialization/termination. */
-HWND __stdcall gethwnd (void);
+/* Check if running in a visible window station. */
+extern bool has_visible_window_station (void);
 
 /* Globals that handle initialization of winsock in a child process. */
 extern HANDLE wsock32_handle;
@@ -198,17 +233,16 @@ extern "C" int try_to_debug (bool waitloop = 1);
 
 void set_file_api_mode (codepage_type);
 
-extern int cygwin_finished_initializing;
+extern bool cygwin_finished_initializing;
 
 /**************************** Miscellaneous ******************************/
 
 void __stdcall set_std_handle (int);
-int __stdcall writable_directory (const char *file);
 int __stdcall stat_dev (DWORD, int, unsigned long, struct __stat64 *);
 
 __ino64_t __stdcall hash_path_name (__ino64_t hash, const char *name) __attribute__ ((regparm(2)));
 void __stdcall nofinalslash (const char *src, char *dst) __attribute__ ((regparm(2)));
-extern "C" char *__stdcall rootdir (char *full_path) __attribute__ ((regparm(1)));
+extern "C" char *__stdcall rootdir (const char *full_path, char *root_path) __attribute__ ((regparm(2)));
 
 /* String manipulation */
 extern "C" char *__stdcall strccpy (char *s1, const char **s2, char c);
@@ -232,12 +266,16 @@ int __stdcall check_null_empty_str_errno (const char *name) __attribute__ ((regp
 int __stdcall check_null_str_errno (const char *name) __attribute__ ((regparm(1)));
 int __stdcall __check_null_invalid_struct (void *s, unsigned sz) __attribute__ ((regparm(2)));
 int __stdcall __check_null_invalid_struct_errno (void *s, unsigned sz) __attribute__ ((regparm(2)));
+int __stdcall __check_invalid_read_ptr (const void *s, unsigned sz) __attribute__ ((regparm(2)));
 int __stdcall __check_invalid_read_ptr_errno (const void *s, unsigned sz) __attribute__ ((regparm(2)));
+int __stdcall check_invalid_virtual_addr (const void *s, unsigned sz) __attribute__ ((regparm(2)));
 
 #define check_null_invalid_struct(s) \
   __check_null_invalid_struct ((s), sizeof (*(s)))
 #define check_null_invalid_struct_errno(s) \
   __check_null_invalid_struct_errno ((s), sizeof (*(s)))
+#define check_invalid_read_struct_errno(s) \
+  __check_invalid_read_ptr_errno ((s), sizeof (*(s)))
 
 struct iovec;
 ssize_t check_iovec_for_read (const struct iovec *, int) __attribute__ ((regparm(2)));
@@ -254,12 +292,18 @@ extern "C" int __small_sprintf (char *dst, const char *fmt, ...) /*__attribute__
 extern "C" int __small_vsprintf (char *dst, const char *fmt, va_list ap) /*__attribute__ ((regparm (3)))*/;
 extern void multiple_cygwin_problem (const char *, unsigned, unsigned);
 
+int symlink_worker (const char *, const char *, bool, bool)
+  __attribute__ ((regparm (3)));
+
 class path_conv;
-int __stdcall stat_worker (const char *name, struct __stat64 *buf, int nofollow,
-			   path_conv *pc = NULL) __attribute__ ((regparm (3)));
-int __stdcall low_priority_sleep (DWORD) __attribute__ ((regparm (1)));
+
+int fcntl_worker (int fd, int cmd, void *arg);
+
+extern "C" int low_priority_sleep (DWORD) __attribute__ ((regparm (1)));
 #define SLEEP_0_STAY_LOW INFINITE
 
+size_t getshmlba (void);
+
 /**************************** Exports ******************************/
 
 extern "C" {
@@ -267,10 +311,6 @@ int cygwin_select (int , fd_set *, fd_set *, fd_set *,
 		   struct timeval *to);
 int cygwin_gethostname (char *__name, size_t __len);
 
-int kill_pgrp (pid_t, int);
-int _kill (int, int);
-int _raise (int sig);
-
 extern DWORD binmode;
 extern char _data_start__, _data_end__, _bss_start__, _bss_end__;
 extern void (*__CTOR_LIST__) (void);
@@ -282,14 +322,6 @@ extern SYSTEM_INFO system_info;
 
 #define WM_ASYNCIO	0x8000		// WM_APP
 
-/* Note that MAX_PATH is defined in the windows headers */
-/* There is also PATH_MAX and MAXPATHLEN.
-   PATH_MAX is from Posix and does *not* include the trailing NUL.
-   MAXPATHLEN is from Unix.
-
-   Thou shalt use MAX_PATH throughout.  It avoids the NUL vs no-NUL
-   issue and is neither of the Unixy ones [so we can punt on which
-   one is the right one to use].  */
 
 #define STD_RBITS (S_IRUSR | S_IRGRP | S_IROTH)
 #define STD_WBITS (S_IWUSR)
@@ -306,13 +338,10 @@ extern HANDLE hMainThread;
 extern HANDLE hMainProc;
 
 extern bool cygwin_testing;
-extern unsigned _cygwin_testing_magic;
-extern HMODULE cygwin_hmodule;
 
 extern char almost_null[];
 
 #define winsock2_active (wsadata.wVersion >= 512)
-#define winsock_active (wsadata.wVersion < 512)
 extern struct WSAData wsadata;
 
 #endif /* defined __cplusplus */
