@@ -1,30 +1,14 @@
 /*
  * malloc.h
+ * This file has no copyright assigned and is placed in the Public Domain.
+ * This file is a part of the mingw-runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within the package.
  *
  * Support for programs which want to use malloc.h to get memory management
  * functions. Unless you absolutely need some of these functions and they are
  * not in the ANSI headers you should use the ANSI standard header files
  * instead.
  *
- * This file is part of the Mingw32 package.
- *
- * Contributors:
- *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- *
- *  THIS SOFTWARE IS NOT COPYRIGHTED
- *
- *  This source code is offered for use in the public domain. You may
- *  use, modify or distribute it freely.
- *
- *  This code is distributed in the hope that it will be useful but
- *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
- *  DISCLAIMED. This includes but is not limited to warranties of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Revision: 1.6 $
- * $Author: dannysmith $
- * $Date: 2001/11/29 04:26:33 $
- *
  */
 
 #ifndef _MALLOC_H_
@@ -79,6 +63,16 @@ _CRTIMP size_t __cdecl _get_sbh_threshold (void);
 _CRTIMP int __cdecl _set_sbh_threshold (size_t);
 _CRTIMP void* __cdecl _expand (void*, size_t); 
 
+/* These require msvcr70.dll or higher. */ 
+#if __MSVCRT_VERSION__ >= 0x0700
+_CRTIMP void * __cdecl _aligned_offset_malloc(size_t, size_t, size_t);
+_CRTIMP void * __cdecl _aligned_offset_realloc(void*, size_t, size_t, size_t);
+
+_CRTIMP void* __cdecl _aligned_malloc (size_t, size_t);
+_CRTIMP void* __cdecl _aligned_realloc (void*, size_t, size_t);
+_CRTIMP void __cdecl _aligned_free (void*);
+#endif /* __MSVCRT_VERSION__ >= 0x0700 */
+
 #ifdef __cplusplus
 }
 #endif
